/******************************************************************************* 
 *  Copyright 2007 Amazon Technologies, Inc.  
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  
 *  You may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at: http://aws.amazon.com/apache2.0
 *  This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 *  CONDITIONS OF ANY KIND, either express or implied. See the License for the 
 *  specific language governing permissions and limitations under the License.
 * ***************************************************************************** 
 *    __  _    _  ___ 
 *   (  )( \/\/ )/ __)
 *   /__\ \    / \__ \
 *  (_)(_) \/\/  (___/
 * 
 *  Amazon LS CSharp Library
 *  API Version: 2007-06-05
 *  Generated: Wed Nov 14 17:05:49 PST 2007 
 * 
 */

using System;
using System.Web;
using System.Net;
using System.Text;
using System.IO;
using System.Text.RegularExpressions;
using System.Security.Cryptography;
using System.Globalization;
using System.Xml.Serialization;
using System.Collections.Generic;
using ImageNexus.BenScharbach.ICLTools.Amazon.LS.Amazon.LS.Model;

namespace ImageNexus.BenScharbach.ICLTools.Amazon.LS.Amazon.LS.Http
{


   /**

    *
    * AmazonLSQuery is the HTTP Query implementation of AmazonLS
    *
    */
    public class AmazonLSQuery : AmazonLS
    {

        private String awsAccessKeyId = null;
        private String awsSecretAccessKey = null;
        private AmazonLSConfig config = null;

        /// <summary>
        /// Constructs AmazonLSQuery with AWS Access Key ID and AWS Secret Key
        /// </summary>
        /// <param name="awsAccessKeyId">AWS Access Key ID</param>
        /// <param name="awsSecretAccessKey">AWS Secret Access Key</param>
        public AmazonLSQuery(String awsAccessKeyId, String awsSecretAccessKey)
            : this(awsAccessKeyId, awsSecretAccessKey, new AmazonLSConfig())
        {
        }


        /// <summary>
        /// Constructs AmazonLSQuery with AWS Access Key ID and AWS Secret Key
        /// </summary>
        /// <param name="awsAccessKeyId">AWS Access Key ID</param>
        /// <param name="awsSecretAccessKey">AWS Secret Access Key</param>
        /// <param name="config">configuration</param>
        public AmazonLSQuery(String awsAccessKeyId, String awsSecretAccessKey, AmazonLSConfig config)
        {
            this.awsAccessKeyId = awsAccessKeyId;
            this.awsSecretAccessKey = awsSecretAccessKey;
            this.config = config;
        }


        // Public API ------------------------------------------------------------//

        
        /// <summary>
        /// Activate Desktop Product 
        /// </summary>
        /// <param name="action">Activate Desktop Product  Action</param>
        /// <returns>Activate Desktop Product  Response from the service</returns>
        /// <remarks>
        /// Allows a distributed devpay product to retrieve an HMAC key pair and user token that can be subsequently used for an end user against other AWS services.
        ///   
        /// </remarks>
        public ActivateDesktopProductResponse ActivateDesktopProduct(ActivateDesktopProduct action)
        {
            return Invoke<ActivateDesktopProductResponse>(action.ToDictionary());
        }

        
        /// <summary>
        /// Activate Hosted Product 
        /// </summary>
        /// <param name="action">Activate Hosted Product  Action</param>
        /// <returns>Activate Hosted Product  Response from the service</returns>
        /// <remarks>
        /// Allows a controlled devpay product to retrieve a user token. Note this is an authenticated request, and as such it must be signed.
        ///   
        /// </remarks>
        public ActivateHostedProductResponse ActivateHostedProduct(ActivateHostedProduct action)
        {
            return Invoke<ActivateHostedProductResponse>(action.ToDictionary());
        }

        
        /// <summary>
        /// Verify Product Subscription By Tokens 
        /// </summary>
        /// <param name="action">Verify Product Subscription By Tokens  Action</param>
        /// <returns>Verify Product Subscription By Tokens  Response from the service</returns>
        /// <remarks>
        /// Allows a product to determine if the user represented by the UserToken still has an active subscription.  Note this is an authenticated request, and as such it must be signed.
        ///   
        /// </remarks>
        public VerifyProductSubscriptionByTokensResponse VerifyProductSubscriptionByTokens(VerifyProductSubscriptionByTokens action)
        {
            return Invoke<VerifyProductSubscriptionByTokensResponse>(action.ToDictionary());
        }

        
        /// <summary>
        /// Verify Product Subscription By Pid 
        /// </summary>
        /// <param name="action">Verify Product Subscription By Pid  Action</param>
        /// <returns>Verify Product Subscription By Pid  Response from the service</returns>
        /// <remarks>
        /// Allows a PersistentIdentifier owner to determine if the user represented by the PersistentIdentifier still has an active subscription.  Note this is an authenticated request, and as such it must be signed.
        ///   
        /// </remarks>
        public VerifyProductSubscriptionByPidResponse VerifyProductSubscriptionByPid(VerifyProductSubscriptionByPid action)
        {
            return Invoke<VerifyProductSubscriptionByPidResponse>(action.ToDictionary());
        }

        
        /// <summary>
        /// Get Active Subscriptions By Pid 
        /// </summary>
        /// <param name="action">Get Active Subscriptions By Pid  Action</param>
        /// <returns>Get Active Subscriptions By Pid  Response from the service</returns>
        /// <remarks>
        /// Gets all products that an end user is currently subscribed to for products owned by the requesting party.  Note this is an authenticated request, and as such it must be signed.
        ///   
        /// </remarks>
        public GetActiveSubscriptionsByPidResponse GetActiveSubscriptionsByPid(GetActiveSubscriptionsByPid action)
        {
            return Invoke<GetActiveSubscriptionsByPidResponse>(action.ToDictionary());
        }

        // Private API ------------------------------------------------------------//

        /**
         * Configure HttpClient with set of defaults as well as configuration
         * from AmazonLSConfig instance
         */
        private HttpWebRequest ConfigureWebRequest(int contentLength)
        {
            HttpWebRequest request = WebRequest.Create(config.ServiceURL) as HttpWebRequest;

            if (config.IsSetProxyHost())
            {
                request.Proxy = new WebProxy(config.ProxyHost, config.ProxyPort);
            }
            request.UserAgent = config.UserAgent;
            request.Method = "POST";
            request.Timeout = 50000;
            request.ContentType = "application/x-www-form-urlencoded";
            request.ContentLength = contentLength;

            return request;
        }

        /**
         * Invoke request and return response
         */
        private T Invoke<T>(IDictionary<String, String> parameters)
        {
            String actionName = parameters["Action"];
            T response = default(T);
            String responseBody = null;
            HttpStatusCode statusCode = default(HttpStatusCode);

            /* Add required request parameters */
            AddRequiredParameters(parameters);

            String queryString = GetParametersAsString(parameters);

            byte[] requestData = new UTF8Encoding().GetBytes(queryString);
            HttpWebRequest request = ConfigureWebRequest(requestData.Length);

            /* Submit the request and read response body */
            try
            {
                using (Stream requestStream = request.GetRequestStream())
                {
                    requestStream.Write(requestData, 0, requestData.Length);
                }
                using (HttpWebResponse httpResponse = request.GetResponse() as HttpWebResponse)
                {
                    statusCode = httpResponse.StatusCode;
                    StreamReader reader = new StreamReader(httpResponse.GetResponseStream(), Encoding.UTF8);
                    responseBody = reader.ReadToEnd();
                }

                /* Attempt to deserialize response into <Action> Response type */
                XmlSerializer serlizer = new XmlSerializer(typeof(T));
                response = (T)serlizer.Deserialize(new StringReader(responseBody));
            }
            /* Web exception is thrown on unsucessful responses */
            catch (WebException we)
            {
                using (HttpWebResponse httpErrorResponse = (HttpWebResponse)we.Response as HttpWebResponse)
                {
                    if (httpErrorResponse == null)
                    {
                        throw new AmazonLSException(we);
                    }
                    statusCode = httpErrorResponse.StatusCode;
                    StreamReader reader = new StreamReader(httpErrorResponse.GetResponseStream(), Encoding.UTF8);
                    responseBody = reader.ReadToEnd();
                }

                /* Attempt to deserialize response into ErrorResponse type */
                try
                {
                    XmlSerializer serlizer = new XmlSerializer(typeof(ErrorResponse));
                    ErrorResponse errorResponse = (ErrorResponse)serlizer.Deserialize(new StringReader(responseBody));
                    Error error = errorResponse.Error[0];

                    /* Throw formatted exception with information available from the error response */
                    throw new AmazonLSException(
                        error.Message,
                        statusCode,
                        error.Code,
                        error.Type,
                        errorResponse.RequestId,
                        errorResponse.ToXML());
                }
                /* Rethrow on deserializer error */
                catch (Exception e)
                {
                    if (e is AmazonLSException)
                    {
                        throw e;
                    }
                    else
                    {
                        AmazonLSException se =  ReportAnyErrors (responseBody, statusCode, e);
                        throw se;
                    }
                }
            }
            /* Catch other exceptions, attempt to convert to formatted exception, 
             * else rethrow wrapped exception */
            catch (Exception e)
            {
                throw new AmazonLSException(e);
            }
            return response;
        }


        /**
         * Look for additional error strings in the response and return formatted exception
         */
        private AmazonLSException ReportAnyErrors(String responseBody, HttpStatusCode status, Exception e)
        {

            AmazonLSException ex = null;

            if (responseBody != null && responseBody.StartsWith("<"))
            {
                Match errorMatcherOne = Regex.Match(responseBody, "<RequestId>(.*)</RequestId>.*<Error>" +
                        "<Code>(.*)</Code><Message>(.*)</Message></Error>.*(<Error>)?", RegexOptions.Multiline);
                Match errorMatcherTwo = Regex.Match(responseBody, "<Error><Code>(.*)</Code><Message>(.*)" +
                        "</Message></Error>.*(<Error>)?.*<RequestID>(.*)</RequestID>", RegexOptions.Multiline);

                if (errorMatcherOne.Success)
                {
                    String requestId = errorMatcherOne.Groups[1].Value;
                    String code = errorMatcherOne.Groups[2].Value;
                    String message = errorMatcherOne.Groups[3].Value;

                    ex = new AmazonLSException(message, status, code, "Unknown", requestId, responseBody);

                }
                else if (errorMatcherTwo.Success)
                {
                    String code = errorMatcherTwo.Groups[1].Value;
                    String message = errorMatcherTwo.Groups[2].Value;
                    String requestId = errorMatcherTwo.Groups[4].Value;

                    ex = new AmazonLSException(message, status, code, "Unknown", requestId, responseBody);
                }
                else
                {
                    ex = new AmazonLSException("Internal Error", status);
                }
            }
            else
            {
                ex = new AmazonLSException("Internal Error", status);
            }
            return ex;
        }

        /**
         * Add authentication related and version parameters
         */
        private void AddRequiredParameters(IDictionary<String, String> parameters)
        {
            parameters.Add("AWSAccessKeyId", this.awsAccessKeyId);
            parameters.Add("Timestamp", GetFormattedTimestamp());
            parameters.Add("Version", config.ServiceVersion);
            parameters.Add("SignatureVersion", config.SignatureVersion);
            parameters.Add("Signature", SignParameters(parameters, this.awsSecretAccessKey));
        }

        /**
         * Convert Disctionary of paremeters to Url encoded query string
         */
        private string GetParametersAsString(IDictionary<String, String> parameters)
        {
            StringBuilder data = new StringBuilder();
            foreach (String key in (IEnumerable<String>)parameters.Keys)
            {
                String value = parameters[key];
                if (value != null && value.Length > 0)
                {
                    data.Append(key);
                    data.Append('=');
                    data.Append(HttpUtility.UrlEncode(parameters[key]));
                    data.Append('&');
                }    
            }
            String stringData = data.ToString();
            if (stringData.EndsWith("&")) 
            {
                stringData = stringData.Remove(stringData.Length - 1, 1);
            }
            return stringData;
        }

        /**
         * Computes RFC 2104-compliant HMAC signature for request parameters
         * Implements AWS Signature, as per following spec:
         *
         * If Signature Version is 0, it signs concatenated Action and Timestamp
         *
         * If Signature Version is 1, it performs the following:
         *
         * Sorts all  parameters (including SignatureVersion and excluding Signature,
         * the value of which is being created), ignoring case.
         *
         * Iterate over the sorted list and append the parameter name (in original case)
         * and then its value. It will not URL-encode the parameter values before
         * constructing this string. There are no separators.
         */
        private String SignParameters(IDictionary<String, String> parameters, String key)
        {
            String signatureVersion = parameters["SignatureVersion"];
            StringBuilder data = new StringBuilder();

            if ("0".Equals(signatureVersion))
            {
                data.Append(parameters["Action"]).Append(parameters["Action"]);
            }
            else if ("1".Equals(signatureVersion))
            {
                IDictionary<String, String> sorted =
                    new SortedDictionary<String, String>(parameters, StringComparer.InvariantCultureIgnoreCase);
                parameters.Remove("Signature");
                foreach (KeyValuePair<String, String> pair in sorted)
                {
                    data.Append(pair.Key);
                    data.Append(pair.Value);
                }
            }
            else
            {
                throw new Exception("Invalid Signature Version specified");
            }
            return Sign(data.ToString(), key);
        }

        /**
         * Computes RFC 2104-compliant HMAC signature.
         */
        private String Sign(String data, String key)
        {
            Encoding encoding = new UTF8Encoding();
            HMACSHA1 signature = new HMACSHA1(encoding.GetBytes(key));
            return Convert.ToBase64String(signature.ComputeHash(
                encoding.GetBytes(data.ToCharArray())));
        }


        /**
         * Formats date as ISO 8601 timestamp
         */
        private String GetFormattedTimestamp()
        {
            DateTime dateTime = DateTime.Now;
            return new DateTime(dateTime.Year, dateTime.Month, dateTime.Day,
                                 dateTime.Hour, dateTime.Minute, dateTime.Second,
                                 dateTime.Millisecond
                                 , DateTimeKind.Local
                               ).ToUniversalTime().ToString("yyyy-MM-dd\\THH:mm:ss.fff\\Z",
                                CultureInfo.InvariantCulture);
        }

    }
}