/*******************************************************************************
 *  Copyright 2007 Amazon Technologies, Inc.
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *
 *  You may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at: http://aws.amazon.com/apache2.0
 *  This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 *  CONDITIONS OF ANY KIND, either express or implied. See the License for the
 *  specific language governing permissions and limitations under the License.
 *******************************************************************************/

using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Xml;
using ImageNexus.BenScharbach.ICLTools.Amazon.LS.Amazon.LS;
using ImageNexus.BenScharbach.ICLTools.Amazon.LS.Amazon.LS.Http;
using ImageNexus.BenScharbach.ICLTools.Amazon.LS.Amazon.LS.Model;
using ImageNexus.BenScharbach.ICLTools.S3Sample;

namespace ImageNexus.BenScharbach.ICLTools.Amazon.LS
{
    /// <summary>
    /// This class is the interface to AWS Services for this sample
    /// application.
    /// </summary>
    public class Services
    {
        public enum AppType {None, Developer, Web, Desktop };

        AWSAuthConnection _connection;
        QueryStringAuthGenerator _queryStringAuthGenerator;
        string _accessKeyId;
        string _secretAccessKey;
        string _userToken;
        readonly string _productToken;
        string _s3Server = "s3.amazonaws.com";
        int _expiresInSeconds = 60;
        bool? _forceProductToken;
        bool _isSecureS3 = true;
        readonly AppType _appType;

        /// <summary>
        /// Instantiate AWS Service connections.
        /// </summary>
        /// <param name="appType"></param>
        /// <param name="accessKeyId">Used to identify the signer.</param>
        /// <param name="secretAccessKey">Key used to sign requests.</param>
        public Services(
            AppType appType,
            string accessKeyId, 
            string secretAccessKey)
            : this(appType,accessKeyId, secretAccessKey, null, null)
        {
        }

        /// <summary>
        /// Instantiate AWS Service connections.
        /// </summary>
        /// <param name="appType"></param>
        /// <param name="accessKeyId">Used to identify the signer.</param>
        /// <param name="secretAccessKey">Key used to sign requests.</param>
        /// <param name="userToken">Security token representing the user.</param>
        /// <param name="productToken">Security token representing the product.</param>
        public Services(
                    AppType appType,
                    string accessKeyId,
                    string secretAccessKey,
                    string userToken,
                    string productToken)
        {
            _appType = appType;
            _accessKeyId = accessKeyId;
            _secretAccessKey = secretAccessKey;
            _userToken = userToken;
            _productToken = productToken;

            Connect();
        }
        private bool ShouldUseProductToken
        {
            get
            {
                return _productToken != null &&
                (
                    (_forceProductToken.HasValue && _forceProductToken.Value)
                  || ((!_forceProductToken.HasValue) && _appType == AppType.Desktop)
                );
            }
        }
        private void Connect()
        {
            var tokensToUse = new List<string>();
            if (_userToken != null)
            {
                tokensToUse.Add(_userToken);
            }
            if (ShouldUseProductToken)
            {
                tokensToUse.Add(_productToken);
            }
           
                _connection = new AWSAuthConnection(
                                    _accessKeyId,
                                    _secretAccessKey,
                                    tokensToUse.ToArray(),
                                    _isSecureS3,
                                    _s3Server,
                                    CallingFormat.Path);
                _queryStringAuthGenerator = new QueryStringAuthGenerator(
                                         _accessKeyId,
                                         _secretAccessKey,
                                         tokensToUse.ToArray(),
                                         _isSecureS3,
                                         _s3Server,
                                         _isSecureS3?Utils.SecurePort: Utils.InsecurePort,
                                         CallingFormat.Path);
            

            _queryStringAuthGenerator.ExpiresIn = _expiresInSeconds*1000;
        }

        /// <summary>
        /// Calls LicenseService to get a UserToken that can be used in requests signed with
        /// existing access key pairs.
        /// </summary>
        /// <param name="accessKeyId">Used to identify the signer.</param>
        /// <param name="secretAccessKey">Key used to sign requests.</param>
        /// <param name="activationKey">The activation key for a product.</param>
        /// <param name="productToken">Security token representing the product.</param>
        /// <param name="amazonLsConfig">Configuration data for the License Service.</param>
        /// <returns>Security token representing the user.</returns>
        public static string ActivateHostedProduct(
            string accessKeyId,
            string secretAccessKey,
            string activationKey,
            string productToken,
            AmazonLSConfig amazonLsConfig)
        {
            try
            {
                AmazonLS licenseService = new AmazonLSQuery(accessKeyId, secretAccessKey, amazonLsConfig);

                var action = new ActivateHostedProduct {ActivationKey = activationKey, ProductToken = productToken};

                var response = licenseService.ActivateHostedProduct(action);
                return response.ActivateHostedProductResult.UserToken;
            }
            catch (AmazonLSException ex)
            {
                var errorMessage = string.Empty;

                errorMessage += "Caught Exception: " + ex.Message;
                errorMessage += "Response Status Code: " + ex.StatusCode;
                errorMessage += "Error Code: " + ex.ErrorCode;
                errorMessage += "Error Type: " + ex.ErrorType;
                errorMessage += "Request ID: " + ex.RequestId;
                errorMessage += "XML: " + ex.XML;

                throw new ApplicationException(errorMessage);
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// Calls LicenseService to obtain calling credentials for an application.
        /// </summary>
        /// <param name="activationKey">The activation key for a product.</param>
        /// <param name="productToken">Security token representing the product.</param>
        /// <param name="amazonLsConfig">Configuration data for the License Service.</param>
        /// <returns>Signing credentials.</returns>
        public static Credentials ActivateDesktopProduct(
                                                string activationKey,
                                                string productToken,
                                                AmazonLSConfig amazonLsConfig)
        {
            try
            {
                //Note that the generated License Webservice library expects to sign all requests
                // to AWS services.  Since ActivateDesktopProduct is an unsigned call, we pass
                // bogus signing credentials.
                AmazonLS service = new AmazonLSQuery("dummy", "dummy", amazonLsConfig);

                var action = new ActivateDesktopProduct {ActivationKey = activationKey, ProductToken = productToken};

                var response = service.ActivateDesktopProduct(action);
                return new Credentials(
                    response.ActivateDesktopProductResult.UserToken,
                    response.ActivateDesktopProductResult.AWSAccessKeyId,
                    response.ActivateDesktopProductResult.SecretAccessKey
                    );
            }
            catch (AmazonLSException ex)
            {
                var errorMessage = string.Empty;

                errorMessage += "Caught Exception: " + ex.Message;
                errorMessage += "Response Status Code: " + ex.StatusCode;
                errorMessage += "Error Code: " + ex.ErrorCode;
                errorMessage += "Error Type: " + ex.ErrorType;
                errorMessage += "Request ID: " + ex.RequestId;
                errorMessage += "XML: " + ex.XML;

                throw new ApplicationException(errorMessage);
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// Processes an S3 operation.  The line string should be in the format
        /// [operation] [param1] [param2] [etc...]
        /// </summary>
        /// <param name="line">The S3 operation line.</param>
        /// <param name="result">Variable to hold the text output of the command.</param>
        /// <returns>True if the operation isn't exit.</returns>
        public bool ProcessS3Operation(string line, out string result)
        {
            string[] arguments = line.Split(' ');
            result = "";
            if (arguments.Length == 0)
            {
                return true;
            }
            string command = arguments[0].ToLowerInvariant();
            bool shouldContinue = true;
            string bucketName;
            int bucketIndex = 0;
            string fileName;
            string s3ObjectKey;
            S3Object s3Object=null;
            try
            {
                switch (command)
                {
                    case "createbucket":
                        bucketName = arguments[1];
                        if (!_connection.CheckBucketExists(bucketName))
                        {
                            _connection.CreateBucket(bucketName, null, null);
                        }
                        break;
                    case "urlforcreatebucket":
                        bucketName = arguments[1];
                       result = _queryStringAuthGenerator.createBucket(bucketName,null, null);
                        break;
                    case "listbucket":
                        bucketName = arguments[1];
                        ListBucketResponse listBucket = _connection.ListBucket(bucketName, null, null, 0, null);
                        
                        string[] listBucketResult = new string[listBucket.Entries.Count];
                        bucketIndex = 0;
                        foreach (ListEntry entry in listBucket.Entries)
                        {
                            listBucketResult[bucketIndex++] = entry.Key.ToString();
                        }
                        result = "[" + string.Join(", ", listBucketResult) + "]" ;
                        break;
                    case "urlforlistbucket":
                        bucketName = arguments[1];
                        result = _queryStringAuthGenerator.listBucket(bucketName, null, null, 0, null);
                        break;
                    case "deletebucket":
                        bucketName = arguments[1];
                        _connection.DeleteBucket(bucketName, null);
                        break;
                    case "urlfordeletebucket":
                          bucketName = arguments[1];
                       result = _queryStringAuthGenerator.deleteBucket(bucketName, null);
                        break;
                    case "createobject":
                        bucketName = arguments[1];
                        s3ObjectKey = arguments[2];
                        fileName = arguments[3];
                        s3Object = new S3Object(System.IO.File.ReadAllBytes(fileName), null);
                        _connection.Put(bucketName, s3ObjectKey, s3Object, null);
                        break;
                    case "urlforcreateobject":
                        bucketName = arguments[1];
                        s3ObjectKey = arguments[2];
                        fileName = arguments[3];
                        s3Object = new S3Object(System.IO.File.ReadAllBytes(fileName), null);
                        result= _queryStringAuthGenerator.put(bucketName, s3ObjectKey, s3Object, null);
                        break;
                    case "deleteobject":
                        bucketName = arguments[1];
                        s3ObjectKey = arguments[2];
                        _connection.Delete(bucketName, s3ObjectKey, null);
                        break;
                    case "urlfordeleteobject":
                       bucketName = arguments[1];
                        s3ObjectKey = arguments[2];
                        _queryStringAuthGenerator.delete(bucketName, s3ObjectKey, null);
                        break;
                    case "listobject":
                        bucketName = arguments[1];
                        s3ObjectKey = arguments[2];
                        GetResponse response = _connection.Get(bucketName, s3ObjectKey, null);
                        
                        result = response.Object.Data.ToString() ;
                        break;
                    case "urlforlistobject":
                        bucketName = arguments[1];
                        s3ObjectKey = arguments[2];
                        result = _queryStringAuthGenerator.get(bucketName, s3ObjectKey, null);
                        break;
                    case "listallmybuckets":
                        ListAllMyBucketsResponse listAllBuckets = _connection.ListAllMyBuckets(null);
                        
                        string[] listAllBucketsResult = new string[listAllBuckets.Buckets.Count];
                        bucketIndex = 0;
                        foreach (Bucket entry in listAllBuckets.Buckets)
                        {
                            listAllBucketsResult[bucketIndex++] = entry.Name.ToString();
                        }
                        result = "[" + string.Join(", ", listAllBucketsResult) + "]";
                        break;
                    case "urlforlistallmybuckets":
                        result= _queryStringAuthGenerator.listAllMyBuckets(null);
                        
                        break;
                    case "putbucketacl":
                        bucketName = arguments[1]; ;
                        string aclFileName = arguments[2]; ;
                        string aclXmlDocText = File.ReadAllText(aclFileName);
                        Response putBucketACLResponse =
                                    _connection.PutBucketAcl(bucketName, aclXmlDocText, null);
                        result =
                                    putBucketACLResponse.getResponseMessage() ;
                        break;
                    case "urlforputbucketacl":
                          bucketName = arguments[1]; ;
                        result= _queryStringAuthGenerator.putBucketACL(bucketName, null);
                      break;
                    case "getbucketacl":
                        bucketName = arguments[1];
                        GetResponse getBucketAclResponse = _connection.GetBucketACL(bucketName, null);

                        S3Object s3AclObject = getBucketAclResponse.Object;
                        if (s3AclObject != null)
                        {
                            result = s3AclObject.Data;
                        }
                        else
                        {
                            result = "Object empty";
                        }

                        break;
                    case "urlforgetbucketacl":
                        bucketName = arguments[1];
                       result = _queryStringAuthGenerator.getBucketACL(bucketName, null);
                        break;
                    case "putbucketlogging":
                        bucketName = arguments[1];
                        fileName = arguments[2];
                        string contents = File.ReadAllText(fileName);
                        Response putBucketLoggingResponse =
                            _connection.PutBucketLogging(bucketName, contents, null);
                        result =
                            putBucketLoggingResponse.getResponseMessage() ;
                        break;

                    case "urlforputbucketlogging":
                        bucketName = arguments[1];
                       result = _queryStringAuthGenerator.putBucketLogging(bucketName, null);
                        break;
                    case "getbucketlogging":
                        bucketName = arguments[1];
                        GetResponse getBucketLoggingResponse = _connection.GetBucketLogging(bucketName, null);

                        s3Object = getBucketLoggingResponse.Object;
                        if (s3Object != null)
                        {
                            result = s3Object.Data;
                        }
                        else
                        {
                            result = "Object empty" ;
                        }

                        break;
                    case "urlforgetbucketlogging":
                        bucketName = arguments[1];
                       result = _queryStringAuthGenerator.getBucketLogging(bucketName, null);
                        break;
                    case "putacl":
                        bucketName = arguments[1];
                        s3ObjectKey = arguments[2];
                        fileName = arguments[3];
                        contents = File.ReadAllText(fileName);
                        Response putACLResponse =
                            _connection.PutAcl(bucketName, s3ObjectKey, contents, null);

                        result = putACLResponse.getResponseMessage() ;
                        break;


                    case "urlforputacl":
                        bucketName = arguments[1];
                        s3ObjectKey = arguments[2];
                        result = _queryStringAuthGenerator.putACL(bucketName, s3ObjectKey, null);
                        break;
                    case "getacl":
                        bucketName = arguments[1];
                        s3ObjectKey = arguments[2];
                        GetResponse getAclResponse = _connection.GetAcl(bucketName, s3ObjectKey, null);

                        s3Object = getAclResponse.Object;
                        if (s3Object != null)
                        {
                            result = s3Object.Data ;
                        }
                        else
                        {
                            result = "Object empty" ;
                        }

                        break;

                    case "urlforgetacl":
                        bucketName = arguments[1];
                        s3ObjectKey = arguments[2];
                        result = _queryStringAuthGenerator.getACL(bucketName, s3ObjectKey, null);
                        break;
                    case "setserver":
                        this._s3Server = arguments[1];
                        if (arguments.Length > 2)
                        {
                            this._isSecureS3 = Boolean.Parse(arguments[2]);
                        }
                        Connect();
                        break;

                    case "setexpireinseconds":
                        this._expiresInSeconds = int.Parse(arguments[1]);
                        Connect();
                        break;

                    case "setaccesskeyid":
                        this._accessKeyId = arguments[1];
                        Connect();
                        break;

                    case "setsecretaccesskey":
                        this._secretAccessKey = arguments[1];
                        Connect();
                        break;


                    case "setuseproducttoken":
                        this._forceProductToken = bool.Parse(arguments[1]);
                        Connect();
                        break;

                    case "setusertoken":
                        this._userToken = arguments[1];
                        Connect();
                        break;

                    

                    case "exit":
                        shouldContinue = false;
                        break;

                    case "help":
                        result = GetS3Help() ;
                        break;

                    case "":
                        //Do nothing
                        break;
                    default:
                        result = command + " is invalid.  Please try again." + Environment.NewLine +
                                    GetS3Help() ;
                        break;
                }
                if (command.StartsWith("urlfor") && this.ShouldUseProductToken)
                {
                    result += 
                        Environment.NewLine + 
                        Environment.NewLine + 
                        "WARNING - you appear to be using a Product Token to generate a pre-signed url.  This is NOT recommended for SECURITY reasons.  Please see README for details";
                }
            }
            catch (FormatException e)
            {
                result = "Invalid arguments for operation." ;
            }
            catch (IndexOutOfRangeException e)
            {
                result = "Invalid arguments for operation." ;
            }
            catch (WebException e)
            {
                result = CreateErrorMessage(e);
            }
            catch (DirectoryNotFoundException e)
            {
                result = e.Message ;
            }
            catch (FileNotFoundException e)
            {
                result = e.Message ;
            }
            catch (Exception e)
            {
                throw e;
            }
            result += Environment.NewLine;
            return shouldContinue;
        }

        /// <summary>
        /// Extracts the Code and Message from an XML response creates
        /// an error message.
        /// </summary>
        /// <param name="e"></param>
        private string CreateErrorMessage(WebException e)
        {
            try
            {
                XmlDocument doc = new XmlDocument();
                doc.LoadXml(e.Message);
                XmlNode errorCode = doc.SelectSingleNode("Error/Code");
                if (errorCode != null)
                {
                    string code = errorCode.InnerText;
                    string message = doc.SelectSingleNode("Error/Message").InnerText;
                    return code + " - " + message + Environment.NewLine;
                }
            }
            catch (XmlException)
            {
            }
            return e.Message + Environment.NewLine;
        }

        /// <summary>
        /// Prints the S3 Shell help.
        /// </summary>
        private string GetS3Help()
        {
            return
                    "Command Options:" + Environment.NewLine +
                    "createBucket <bucket name>" + Environment.NewLine +
                    "urlForCreateBucket <bucket name>" + Environment.NewLine +
                    "createObject <bucket name> <object name> <file name>" + Environment.NewLine +
                    "urlForCreateObject <bucket name> <object name> <file name>" + Environment.NewLine +
                    "deleteBucket <bucket name>" + Environment.NewLine +
                    "urlForDeleteBucket <bucket name>" + Environment.NewLine +
                    "deleteObject <bucket name> <object name>" + Environment.NewLine +
                    "urlForDeleteObject <bucket name> <object name>" + Environment.NewLine +
                    "listAllMyBuckets" + Environment.NewLine +
                    "urlForListAllMyBuckets" + Environment.NewLine +
                    "listBucket <bucket name>" + Environment.NewLine +
                    "urlForListBucket <bucket name>" + Environment.NewLine +
                    "listObject <bucket name> <object name>" + Environment.NewLine +
                    "urlForListObject <bucket name> <object name>" + Environment.NewLine +
                    "getACL <bucket name> <object name>" + Environment.NewLine +
                    "urlForGetACL <bucket name> <object name>" + Environment.NewLine +
                    "getBucketACL <bucket name>" + Environment.NewLine +
                    "urlForGetBucketACL <bucket name>" + Environment.NewLine +
                    "getBucketLogging <bucket name>" + Environment.NewLine +
                    "urlForGetBucketLogging <bucket name>" + Environment.NewLine +
                    "putACL <bucket name> <object name> <file name>" + Environment.NewLine +
                    "urlForPutACL <bucket name> <object name> <file name>" + Environment.NewLine +
                    "putBucketACL <bucket name> <file name>" + Environment.NewLine +
                    "urlForPutBucketACL <bucket name> <file name>" + Environment.NewLine +
                    "putBucketLogging <bucket name> <file name>" + Environment.NewLine +
                    "urlForPutBucketLogging <bucket name> <file name>" + Environment.NewLine +
                    "setServer <servername> [<use secure connection, true or false>]" + Environment.NewLine +
                    "setExpireInSeconds <seconds before expiration>" + Environment.NewLine +
                    "setUserToken <a new user token to use in place of the one stored as a credential>" + Environment.NewLine +
                    "setUseProductToken <use product token even when it is not required, true or false>" + Environment.NewLine +
                    "setAccessKeyId <a new access key id to use in place of the initial key>" + Environment.NewLine +
                    "setSecretAccessKey <a new secret access key id to use in place of the initial key>" + Environment.NewLine +
                    "exit";
        }
    }

    /// <summary>
    /// Container for access credentials.
    /// </summary>
    public class Credentials
    {
        private string userToken_ = null;
        private string accessKeyId_ = null;
        private string secretAccessKey_ = null;

        public Credentials(String userToken, String accessKeyId,
            String secretAccessKey)
        {
            userToken_ = userToken;
            accessKeyId_ = accessKeyId;
            secretAccessKey_ = secretAccessKey;
        }

        public String UserToken
        {
            get { return userToken_; }
            set { userToken_ = value; }
        }

        public String AccessKeyId
        {
            get { return accessKeyId_; }
            set { accessKeyId_ = value; }
        }

        public String SecretAccessKey
        {
            get { return secretAccessKey_; }
            set { secretAccessKey_ = value; }
        }
    }
}
