// This software code is made available "AS IS" without warranties of any        
// kind.  You may copy, display, modify and redistribute the software            
// code either by itself or as incorporated into your code; provided that        
// you do not remove any proprietary notices.  Your use of this software         
// code is at your own risk and you waive any claim against Amazon               
// Digital Services, Inc. or its affiliates with respect to your use of          
// this software code. (c) 2006-2007 Amazon Digital Services, Inc. or its             
// affiliates.          


using System;
using System.Collections;
using System.Net;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;

namespace ImageNexus.BenScharbach.ICLTools.S3Sample
{
    /// An interface into the S3 system.  It is initially configured with
    /// authentication and connection parameters and exposes methods to access and
    /// manipulate S3 data.
    public class AWSAuthConnection
    {
        private readonly string _awsAccessKeyId;
        private readonly string _awsSecretAccessKey;
        private readonly bool _isSecure;
        private readonly string _server;
        private readonly int _port;
        private readonly SortedList _initHeaders = new SortedList();
        private readonly CallingFormat _callingFormat;

        // 4/18/2011 - webrequest timeout value; defaults to 100 seconds.
        private int _timeout;
        
        private const String XAMZ_TOKEN_HEADER = "x-amz-security-token";

        #region Properties

        // 4/18/2011
        /// <summary>
        /// Sets the WebRequest timeout value as milliseconds.
        /// </summary>
        public int WithTimeout
        {
           set { _timeout = value; }
        }

        #endregion

        #region Constructors

        public AWSAuthConnection( string awsAccessKeyId, string awsSecretAccessKey )
            : this( awsAccessKeyId, awsSecretAccessKey, new string[0], true, CallingFormat.Subdomain )
        {
        }

        public AWSAuthConnection(string awsAccessKeyId, string awsSecretAccessKey, CallingFormat format)
            : this(awsAccessKeyId, awsSecretAccessKey, new string[0], true, format)
        {
        }

        public AWSAuthConnection(string awsAccessKeyId, string awsSecretAccessKey, string[] tokens)
            : this(awsAccessKeyId, awsSecretAccessKey, tokens, true, CallingFormat.Subdomain)
        {
        }

        public AWSAuthConnection(string awsAccessKeyId, string awsSecretAccessKey, string[] tokens,
            CallingFormat format )
            : this(awsAccessKeyId, awsSecretAccessKey, tokens, true, format )
        {
        }

        public AWSAuthConnection(string awsAccessKeyId, string awsSecretAccessKey, string[] tokens,
            bool isSecure)
            : this( awsAccessKeyId, awsSecretAccessKey, tokens, isSecure, Utils.Host, 
            CallingFormat.Subdomain )
        {
        }

        public AWSAuthConnection(string awsAccessKeyId, string awsSecretAccessKey, string[] tokens, 
            bool isSecure, CallingFormat format) 
            : this( awsAccessKeyId, awsSecretAccessKey, tokens, isSecure, Utils.Host, format )
        {
        }

        public AWSAuthConnection( string awsAccessKeyId, string awsSecretAccessKey, string[] tokens,
            bool isSecure, string server, CallingFormat format ) 
            : this(awsAccessKeyId, awsSecretAccessKey, tokens, isSecure, server,
                   isSecure ? Utils.SecurePort : Utils.InsecurePort, format )
        {
        }

        public AWSAuthConnection( string awsAccessKeyId, string awsSecretAccessKey, string[] tokens,
            bool isSecure, string server )
            : this( awsAccessKeyId, awsSecretAccessKey, tokens, isSecure, server,
                    isSecure ? Utils.SecurePort : Utils.InsecurePort, CallingFormat.Subdomain )
        {
        }

        public AWSAuthConnection( string awsAccessKeyId, string awsSecretAccessKey, string[] tokens,
            bool isSecure, string server, int port) 
            : this( awsAccessKeyId, awsSecretAccessKey, tokens, isSecure, server,
                    port, CallingFormat.Subdomain )
        {
        }

        public AWSAuthConnection( string awsAccessKeyId, string awsSecretAccessKey, string[] tokens,
            bool isSecure, string server, int port, CallingFormat format )
        {
            _awsAccessKeyId = awsAccessKeyId;
            _awsSecretAccessKey = awsSecretAccessKey;
            _isSecure = isSecure;
            _server = server;
            _port = port;
            _callingFormat = format;
            if (tokens.Length != 0)
            {
                _initHeaders.Add(XAMZ_TOKEN_HEADER, String.Join(",", tokens));
            }
        }

        #endregion

        /// <summary>
        /// Creates a new bucket.
        /// </summary>
        /// <param name="bucket">The name of the bucket to create</param>
        /// <param name="location">Location-constraint for bucket (can be null)</param>
        /// <param name="headers">A Map of string to string representing the headers to pass (can be null)</param>
        public Response CreateBucket( string bucket, String location, SortedList headers )
        {
            if (!ValidateBucketName( bucket ))
                throw new ArgumentException( "Invalid Bucket Name: " + bucket );

            string body;
            if (location == null)
                body = "";
            else if (Location.EU.Equals(location))
                body = "<CreateBucketConstraint><LocationConstraint>" + 
                        location + 
                       "</LocationConstraint></CreateBucketConstraint>";
            else
                throw new ArgumentException( "Invalid Location: "+location );

            return new Response(MakeRequest("PUT", bucket, "", null, headers, new S3Object(body, null)));
        }

        /// <summary>
        /// Check if the specified bucket exists (via a HEAD request).
        /// </summary>
        /// <param name="bucket">The name of the bucket to list</param>
        public bool CheckBucketExists(String bucket)
        {
            try
            {
                var response = MakeRequest("HEAD", bucket, "", null, null, null);
                response.Close();
                return true;
            }
            catch (WebException ex)
            {
                var response = ex.Response as HttpWebResponse;
                if (response != null && 
                    response.StatusCode == HttpStatusCode.NotFound)
                {
                    return false;
                }
                throw;
            }
        }

        /// <summary>
        /// Lists the contents of a bucket.
        /// </summary>
        /// <param name="bucket">The name of the bucket to list</param>
        /// <param name="prefix">All returned keys will start with this string (can be null)</param>
        /// <param name="marker">All returned keys will be lexographically greater than this string (can be null)</param>
        /// <param name="maxKeys">The maximum number of keys to return (can be 0)</param>
        /// <param name="headers">A Map of string to string representing HTTP headers to pass.</param>
        public ListBucketResponse ListBucket( string bucket, string prefix, string marker,
                                              int maxKeys, SortedList headers ) {
            return ListBucket( bucket, prefix, marker, maxKeys, null, headers );
        }

        /// <summary>
        /// Lists the contents of a bucket.
        /// </summary>
        /// <param name="bucket">The name of the bucket to list</param>
        /// <param name="prefix">All returned keys will start with this string (can be null)</param>
        /// <param name="marker">All returned keys will be lexographically greater than this string (can be null)</param>
        /// <param name="maxKeys">The maximum number of keys to return (can be 0)</param>
        /// <param name="headers">A Map of string to string representing HTTP headers to pass.</param>
        /// <param name="delimiter">Keys that contain a string between the prefix and the first
        /// occurrence of the delimiter will be rolled up into a single element.</param>
        public ListBucketResponse ListBucket( string bucket, string prefix, string marker,
                                              int maxKeys, string delimiter, SortedList headers ) {
            var query = Utils.QueryForListOptions(prefix, marker, maxKeys, delimiter);
            return new ListBucketResponse( MakeRequest( "GET", bucket, "", query, headers, null ) );
        }

        // 4/13/2011
        public InitiateMultipartUploadResponse InitiateMultipartUpload(string bucket, string key, SortedList headers)
        {
            // Note: 1st param is the Verb, not Action.
            var query = Utils.QueryForMultiPartUploadInitiate();
            return new InitiateMultipartUploadResponse(MakeRequest("POST", bucket, key, query, headers, null));
        }

        /// <summary>
        /// Deletes an empty Bucket.
        /// </summary>
        /// <param name="bucket">The name of the bucket to delete</param>
        /// <param name="headers">A map of string to string representing the HTTP headers to pass (can be null)</param>
        /// <returns></returns>
        public Response DeleteBucket( string bucket, SortedList headers )
        {
            return new Response( MakeRequest( "DELETE", bucket, "", null, headers, null ) );
        }

        /// <summary>
        /// Writes an object to S3.
        /// </summary>
        /// <param name="bucket">The name of the bucket to which the object will be added.</param>
        /// <param name="key">The name of the key to use</param>
        /// <param name="obj">An S3Object containing the data to write.</param>
        /// <param name="headers">A map of string to string representing the HTTP headers to pass (can be null)</param>
        public Response Put( string bucket, string key, S3Object obj, SortedList headers )
        {
            return new Response( MakeRequest("PUT", bucket, key, null, headers, obj));
        }

        /// <summary>
        /// Reads an object from S3
        /// </summary>
        /// <param name="bucket">The name of the bucket where the object lives</param>
        /// <param name="key">The name of the key to use</param>
        /// <param name="headers">A Map of string to string representing the HTTP headers to pass (can be null)</param>
        public GetResponse Get( string bucket, string key, SortedList headers )
        {
            return new GetResponse(MakeRequest("GET", bucket, key, null, headers, null));
        }

        /// <summary>
        /// Delete an object from S3.
        /// </summary>
        /// <param name="bucket">The name of the bucket where the object lives.</param>
        /// <param name="key">The name of the key to use.</param>
        /// <param name="headers">A map of string to string representing the HTTP headers to pass (can be null)</param>
        /// <returns></returns>
        public Response Delete( string bucket, string key, SortedList headers )
        {
            return new Response(MakeRequest("DELETE", bucket, key, null, headers, null));
        }

        public LocationResponse GetBucketLocation(string bucket)
        {
            return new LocationResponse(MakeRequest("GET", bucket, "", NewSingleParam("location"), null, null));
        }

        /// <summary>
        /// Get the logging xml document for a given bucket
        /// </summary>
        /// <param name="bucket">The name of the bucket</param>
        /// <param name="headers">A map of string to string representing the HTTP headers to pass (can be null)</param>
        public GetResponse GetBucketLogging(string bucket, SortedList headers)
        {
            return new GetResponse(MakeRequest("GET", bucket, "", NewSingleParam("logging"), headers, null));
        }

        /// <summary>
        /// Write a new logging xml document for a given bucket
        /// </summary>
        /// <param name="bucket">The name of the bucket to enable/disable logging on</param>
        /// <param name="loggingXmlDoc">The xml representation of the logging configuration as a String.</param>
        /// <param name="headers">A map of string to string representing the HTTP headers to pass (can be null)</param>
        public Response PutBucketLogging(string bucket, string loggingXmlDoc, SortedList headers)
        {
            return new Response(MakeRequest("PUT", bucket, "", NewSingleParam("logging"), headers, new S3Object(loggingXmlDoc, null)));
        }

        /// <summary>
        /// Get the ACL for a given bucket.
        /// </summary>
        /// <param name="bucket">The the bucket to get the ACL from.</param>
        /// <param name="headers">A map of string to string representing the HTTP headers to pass (can be null)</param>
        public GetResponse GetBucketACL(string bucket, SortedList headers)
        {
            return GetAcl(bucket, null, headers);
        }

        /// <summary>
        /// Get the ACL for a given object
        /// </summary>
        /// <param name="bucket">The name of the bucket where the object lives</param>
        /// <param name="key">The name of the key to use.</param>
        /// <param name="headers">A map of string to string representing the HTTP headers to pass (can be null)</param>
        public GetResponse GetAcl( string bucket, string key, SortedList headers )
        {
            if (key == null)
                key = "";
            return new GetResponse(MakeRequest("GET", bucket, key, NewSingleParam("acl"), headers, null));
        }

        /// <summary>
        /// Write a new ACL for a given bucket
        /// </summary>
        /// <param name="bucket">The name of the bucket to change the ACL.</param>
        /// <param name="aclXmlDoc">An XML representation of the ACL as a string.</param>
        /// <param name="headers">A map of string to string representing the HTTP headers to pass (can be null)</param>
        public Response PutBucketAcl(string bucket, string aclXmlDoc, SortedList headers)
        {
            return PutAcl(bucket, null, aclXmlDoc, headers);
        }

        /// <summary>
        /// Write a new ACL for a given object
        /// </summary>
        /// <param name="bucket">The name of the bucket where the object lives or the
        /// name of the bucket to change the ACL if key is null.</param>
        /// <param name="key">The name of the key to use; can be null.</param>
        /// <param name="aclXmlDoc">An XML representation of the ACL as a string.</param>
        /// <param name="headers">A map of string to string representing the HTTP headers to pass (can be null)</param>
        public Response PutAcl(string bucket, string key, string aclXmlDoc, SortedList headers)
        {
            if ( key == null )
                key = "";
            return new Response(MakeRequest("PUT", bucket, key, NewSingleParam("acl"), headers, new S3Object(aclXmlDoc, null)));
        }

        /// <summary>
        /// List all the buckets created by this account.
        /// </summary>
        /// <param name="headers">A map of string to string representing the HTTP headers to pass (can be null)</param>
        public ListAllMyBucketsResponse ListAllMyBuckets( SortedList headers )
        {
            return new ListAllMyBucketsResponse(MakeRequest("GET", "", "", null, headers, null));
        }

        /// <summary>
        /// Make a new WebRequest
        /// </summary>
        /// <param name="method">The HTTP method to use (GET, PUT, DELETE)</param>
        /// <param name="bucket">The bucket name for this request</param>
        /// <param name="key">The key this request is for</param>
        /// <param name="query"></param>
        /// <param name="headers">A map of string to string representing the HTTP headers to pass (can be null)</param>
        /// <param name="obj">S3Object that is to be written (can be null).</param>
        private WebResponse MakeRequest(string method, string bucket, string key, 
                                        SortedList query, SortedList headers, 
                                        S3Object obj)
        {
            if (!String.IsNullOrEmpty(key))
                key = Utils.urlEncode(key);
            var url = BuildUrl(bucket, key, query);

            var redirectCount = 0;
            for (; ; )
            {
                // prep request
                var req = (HttpWebRequest)WebRequest.Create(url);
                req.Method = method;
                // we handle redirects manually
                req.AllowAutoRedirect = false;
                // we already buffer everything
                req.AllowWriteStreamBuffering = false;
                // Work around an HttpWebRequest bug where it will
                // send the request body even if the server does *not* 
                // send 100 continue
                req.KeepAlive = false;

                // 4/18/2011 - Set Timeout
                if (_timeout > 0) req.Timeout = _timeout;

                // if we are sending >1kb data, ask for 100-Continue
                if (obj != null && obj.Bytes.Length > 1024)
                    req.ServicePoint.Expect100Continue = true;

                AddHeaders(req, _initHeaders);
                AddHeaders(req, headers);
                if (obj != null)
                    AddMetadataHeaders(req, obj.Metadata);
                AddAuthHeader(req, bucket, key, query);

                // 4/13/2011
                req.ContentLength = 0L;

                if (obj != null)
                {
                    // Write actual data
                    //byte[] data = obj.Bytes;
                    req.ContentLength = obj.Bytes.Length;
                    var requestStream = req.GetRequestStream();

                    // 4/13/2011 - Modified to match Amazon SDK with event handling
                    var inputStream = new MemoryStream(obj.Bytes);

                    var transferred = 0L;
                    inputStream.Position = 0L;
                    const int bufferSize = 0x2000;
                    var buffer = new byte[bufferSize]; // was BufferSize
                    var count = 0;
                    while ((count = inputStream.Read(buffer, 0, buffer.Length)) > 0)
                    {
                        transferred += count;
                        requestStream.Write(buffer, 0, count);

                        obj.OnRaiseProgressEvent((long)count, transferred, obj.Bytes.Length);
                    }

                    //requestStream.Write(data, 0, data.Length);
                    requestStream.Close();
                }

                // execute request
                try
                {
                    var response = (HttpWebResponse)req.GetResponse();

                    if (!IsRedirect(response))
                        return response;

                    // retry against redirected url
                    url = response.Headers["Location"];
                    response.Close();
                    req.Abort(); // 4/18/2011

                    if (string.IsNullOrEmpty(url))
                        throw new WebException("Redirect without Location header, may need to change calling format.", WebExceptionStatus.ProtocolError);
                }
                catch (WebException ex)
                {
                    if (ex.Response == null)
                        throw;
                    var msg = Utils.SlurpInputStreamAsString(ex.Response.GetResponseStream());
                    throw new WebException(msg, ex, ex.Status, ex.Response);
                }
                catch (Exception ex )
                {
                    var message = ex.Message;
                }

                redirectCount++;
                if (redirectCount > 10)
                    throw new WebException("Too many redirects.");
            }
        }

        private string BuildUrl(string bucket, string key, SortedList query)
        {
            var url = new StringBuilder();
            url.Append(_isSecure ? "https://" : "http://");
            url.Append(Utils.BuildUrlBase(_server, _port, bucket, _callingFormat));
            if (!string.IsNullOrEmpty(key))
                url.Append(key);
            // build the query string parameter
            if (query != null && query.Count != 0)
                url.Append(Utils.ConvertQueryListToQueryString(query));
            return url.ToString();
        }

        static private Boolean IsRedirect(WebResponse response)
        {
            var httpResp = response as HttpWebResponse;
            if (httpResp != null)
            {
                var status = (int)httpResp.StatusCode;
                return (status >= 300 && status < 400);
            }
            return false;
        }

        /// <summary>
        /// Add the given headers to the WebRequest
        /// </summary>
        /// <param name="req">Web request to add the headers to.</param>
        /// <param name="headers">A map of string to string representing the HTTP headers to pass (can be null)</param>
        private static void AddHeaders( WebRequest req, SortedList headers )
        {
            AddHeaders( req, headers, "" );
        }

        /// <summary>
        /// Add the given metadata fields to the WebRequest.
        /// </summary>
        /// <param name="req">Web request to add the headers to.</param>
        /// <param name="metadata">A map of string to string representing the S3 metadata for this resource.</param>
        private static void AddMetadataHeaders( WebRequest req, SortedList metadata )
        {
            AddHeaders( req, metadata, Utils.MetadataPrefix );
        }

        /// <summary>
        /// Add the given headers to the WebRequest with a prefix before the keys.
        /// </summary>
        /// <param name="req">WebRequest to add the headers to.</param>
        /// <param name="headers">Headers to add.</param>
        /// <param name="prefix">String to prepend to each before ebfore adding it to the WebRequest</param>
        private static void AddHeaders( WebRequest req, SortedList headers, string prefix )
        {
            if (headers == null) return;

            foreach ( string key in headers.Keys )
            {
                if (prefix.Length == 0 && key.Equals("Content-Type"))
                {
                    req.ContentType = headers[key] as string;
                }
                else
                {
                    req.Headers.Add(prefix + key, headers[key] as string);
                }
            }
        }

        /// <summary>
        /// Add the appropriate Authorization header to the WebRequest
        /// </summary>
        /// <param name="request">Request to add the header to</param>
        /// <param name="bucket"></param>
        /// <param name="key"></param>
        /// <param name="query"></param>
        private void AddAuthHeader( WebRequest request, string bucket, string key, SortedList query )
        {
            var headers = ((HttpWebRequest) request).Headers;
            if (headers[Utils.AlternativeDateHeader] == null)
            {
                headers.Add(Utils.AlternativeDateHeader, Utils.GetHttpDate());
            }

            var canonicalString = Utils.MakeCanonicalString( bucket, key, query, request );
            var encodedCanonical = Utils.Encode( _awsSecretAccessKey, canonicalString, false );
            headers.Add( "Authorization", "AWS " + _awsAccessKeyId + ":" + encodedCanonical );
        }


        static readonly Regex BUCKET_NAME_REGEX_PATH 
            = new Regex("^[0-9A-Za-z\\.\\-_]*$", RegexOptions.CultureInvariant);
        static readonly Regex BUCKET_NAME_REGEX_SUBDOMAIN
            = new Regex("^[a-z0-9]([a-z0-9\\-]*[a-z0-9])?(\\.[a-z0-9]([a-z0-9\\-]*[a-z0-9])?)*$", RegexOptions.CultureInvariant);
        static readonly Regex BUCKET_NAME_REGEX_SUBDOMAIN_BAD
            = new Regex("^[0-9]+\\.[0-9]+\\.[0-9]+\\.[0-9]+$", RegexOptions.CultureInvariant);

        private bool ValidateBucketName(string bucketName)
        {
            bool valid;
            if (_callingFormat == CallingFormat.Path)
            {
                valid = null != bucketName &&
                    bucketName.Length >= 3 &&
                    bucketName.Length <= 255 &&
                    BUCKET_NAME_REGEX_PATH.IsMatch( bucketName );
            }
            else
            {
                // If there wasn't a location-constraint, then the current actual 
                // restriction is just that no 'part' of the name (i.e. sequence
                // of characters between any 2 '.'s has to be 63) but the recommendation
                // is to keep the entire bucket name under 63.
                valid = null != bucketName &&
                    bucketName.Length >= 3 &&
                    bucketName.Length <= 63 &&
                    !BUCKET_NAME_REGEX_SUBDOMAIN_BAD.IsMatch( bucketName ) &&
                    BUCKET_NAME_REGEX_SUBDOMAIN.IsMatch( bucketName );
            }
            return valid;
        }

        static private SortedList NewSingleParam(string name)
        {
            var query = new SortedList(1) {{name, null}};
            return query;
        }
    }
}
