// This software code is made available "AS IS" without warranties of any        
// kind.  You may copy, display, modify and redistribute the software            
// code either by itself or as incorporated into your code; provided that        
// you do not remove any proprietary notices.  Your use of this software         
// code is at your own risk and you waive any claim against Amazon               
// Digital Services, Inc. or its affiliates with respect to your use of          
// this software code. (c) 2006-2007 Amazon Digital Services, Inc. or its             
// affiliates.          


using System.Collections;
using System.Net;
using System.Xml;

namespace ImageNexus.BenScharbach.ICLTools.S3Sample
{
    public class ListBucketResponse : Response
    {
        /// <summary>
        /// The name of the bucket being listed.  Null if the request fails.
        /// </summary>
        private readonly string _name;
        public string Name
        {
            get
            {
                return _name;
            }
        }

        /// <summary>
        /// The prefix echoed back from the request.  Null if the request fails.
        /// </summary>
        private readonly string _prefix;
        public string Prefix
        {
            get
            {
                return _prefix;
            }
        }

        /// <summary>
        /// The marker echoed back from the request.  Null if the request fails.
        /// </summary>
        private readonly string _marker;
        public string Marker
        {
            get
            {
                return _marker;
            }
        }

        /// <summary>
        /// The delimiter echoed back from the request.  Null if not specified in
        /// the request or it fails.
        /// </summary>
        private readonly string _delimiter;
        public string Delimiter
        {
            get
            {
                return _delimiter;
            }
        }

        /// <summary>
        /// The maxKeys echoed back from the request if specified.  0 if the request fails.
        /// </summary>
        private readonly int _maxKeys;
        public int MaxKeys
        {
            get
            {
                return _maxKeys;
            }
        }

        /// <summary>
        /// Indicates if there are more results to the list.  True if the current
        /// list results have been truncated.  The value will be false if the request
        /// fails.
        /// </summary>
        private readonly bool _isTruncated;
        public bool IsTruncated
        {
            get
            {
                return _isTruncated;
            }
        }

        /// <summary>
        /// Indicates what to use as a marker for subsequent list requests in the event
        /// that the results are truncated.  Present only when a delimiter is specified.
        /// Null if the requests fails.
        /// </summary>
        private readonly string _nextMarker;
        public string NextMarker
        {
            get
            {
                return _nextMarker;
            }
        }

        /// <summary>
        /// A list of ListEntry objects representing the objects in the given bucket.
        /// Null if the request fails.
        /// </summary>
        private readonly ArrayList _entries;
        public ArrayList Entries
        {
            get
            {
                return _entries;
            }
        }

        /// <summary>
        /// A list of CommonPrefixEntry objects representing the common prefixes of the
        /// keys that matched up to the delimiter.  Null if the request fails.
        /// </summary>
        private readonly ArrayList _commonPrefixEntries;
        public ArrayList CommonPrefixEntries
        {
            get
            {
                return _commonPrefixEntries;
            }
        }

        public ListBucketResponse(WebResponse response)
            : base(response)
        {
            _entries = new ArrayList();
            _commonPrefixEntries = new ArrayList();
            var rawBucketXML = Utils.SlurpInputStreamAsString(response.GetResponseStream());

            var doc = new XmlDocument();
            doc.LoadXml( rawBucketXML );
            foreach (XmlNode node in doc.ChildNodes)
            {
                if (!node.Name.Equals("ListBucketResult")) continue;

                foreach (XmlNode child in node.ChildNodes)
                {
                    if (child.Name.Equals("Contents"))
                    {
                        _entries.Add( new ListEntry(child) );
                    }
                    else if (child.Name.Equals("CommonPrefixes"))
                    {
                        _commonPrefixEntries.Add(new CommonPrefixEntry(child));
                    }
                    else if (child.Name.Equals("Name"))
                    {
                        _name = Utils.GetXmlChildText( child );
                    }
                    else if (child.Name.Equals("Prefix"))
                    {
                        _prefix = Utils.GetXmlChildText(child);
                    }
                    else if (child.Name.Equals("Marker"))
                    {
                        _marker = Utils.GetXmlChildText(child);
                    }
                    else if (child.Name.Equals("Delimiter"))
                    {
                        _delimiter = Utils.GetXmlChildText(child);
                    }
                    else if (child.Name.Equals("MaxKeys"))
                    {
                        _maxKeys = int.Parse(Utils.GetXmlChildText(child));
                    }
                    else if (child.Name.Equals("IsTruncated"))
                    {
                        _isTruncated = bool.Parse( Utils.GetXmlChildText(child) );
                    }
                    else if (child.Name.Equals("NextMarker"))
                    {
                        _nextMarker = Utils.GetXmlChildText(child);
                    }
                }
            }
        }
    }
}
