// This software code is made available "AS IS" without warranties of any        
// kind.  You may copy, display, modify and redistribute the software            
// code either by itself or as incorporated into your code; provided that        
// you do not remove any proprietary notices.  Your use of this software         
// code is at your own risk and you waive any claim against Amazon               
// Digital Services, Inc. or its affiliates with respect to your use of          
// this software code. (c) 2006-2007 Amazon Digital Services, Inc. or its             
// affiliates.          


using System;
using System.Collections;
using System.Text;

namespace ImageNexus.BenScharbach.ICLTools.S3Sample
{
    /// This class mimics the behavior of AWSAuthConnection, except instead of actually performing
    /// the operation, QueryStringAuthGenerator will return URLs with query string parameters that
    /// can be used to do the same thing.  These parameters include an expiration date, so that
    /// if you hand them off to someone else, they will only work for a limited amount of time.
    /// PLEASE NOTE - For security reasons, it is HIGHLY RECOMMENDED to avoid using product tokens
    /// in signed urls.  Please see the README for further details.
    public class QueryStringAuthGenerator
    {
        private const long NotSet = -1;
        public static readonly long DEFAULT_EXPIRES_IN = 60*1000;
        private readonly string _awsAccessKeyId;
        private readonly string _awsSecretAccessKey;
        private readonly bool _isSecure;
        private readonly int _port;
        private readonly string _server;
        private readonly string[] _tokens;
        private CallingFormat _callingFormat;

        private long _expires = NotSet;
        private long _expiresIn = NotSet;

        // by default, expire in 1 minute

        public QueryStringAuthGenerator(string awsAccessKeyId, string awsSecretAccessKey)
            : this(awsAccessKeyId, awsSecretAccessKey, true, CallingFormat.Subdomain)
        {
        }

        public QueryStringAuthGenerator(string awsAccessKeyId, string awsSecretAccessKey,
                                        bool isSecure, CallingFormat format)
            : this(awsAccessKeyId, awsSecretAccessKey, isSecure, Utils.Host, format)
        {
        }

        public QueryStringAuthGenerator(string awsAccessKeyId, string awsSecretAccessKey,
                                        bool isSecure)
            : this(awsAccessKeyId, awsSecretAccessKey, isSecure, Utils.Host, CallingFormat.Subdomain)
        {
        }

        public QueryStringAuthGenerator(string awsAccessKeyId, string awsSecretAccessKey,
                                        bool isSecure, string server, CallingFormat format)
            : this(awsAccessKeyId, awsSecretAccessKey, null, isSecure, server,
                   isSecure ? Utils.SecurePort : Utils.InsecurePort, format)
        {
        }

        public QueryStringAuthGenerator(string awsAccessKeyId, string awsSecretAccessKey,
                                        bool isSecure, string server)
            : this(awsAccessKeyId, awsSecretAccessKey, null, isSecure, server,
                   isSecure ? Utils.SecurePort : Utils.InsecurePort, CallingFormat.Subdomain)
        {
        }

        public QueryStringAuthGenerator(string awsAccessKeyId, string awsSecretAccessKey,
                                        bool isSecure, string server, int port)
            : this(awsAccessKeyId, awsSecretAccessKey, null, isSecure, server,
                   port, CallingFormat.Subdomain)
        {
        }

        public QueryStringAuthGenerator(
            string awsAccessKeyId,
            string awsSecretAccessKey,
            string[] tokens,
            bool isSecure, string server,
            int port,
            CallingFormat format)
        {
            _awsAccessKeyId = awsAccessKeyId;
            _awsSecretAccessKey = awsSecretAccessKey;
            _isSecure = isSecure;
            _server = server;
            _port = port;
            _expiresIn = DEFAULT_EXPIRES_IN;
            _expires = NotSet;
            _callingFormat = format;
            _tokens = tokens;
        }

        /// <summary>
        /// Sets/Gets the current CallingFormat
        /// </summary>
        public CallingFormat CallingFormat
        {
            get { return _callingFormat; }
            set { _callingFormat = value; }
        }

        /// <summary>
        /// Sets/Gets the milliseconds since the Epoch that this
        /// expires at
        /// </summary>
        public long Expires
        {
            get { return _expires; }
            set
            {
                _expires = value;
                _expiresIn = NotSet;
            }
        }

        public long ExpiresIn
        {
            get { return _expiresIn; }
            set
            {
                _expiresIn = value;
                _expires = NotSet;
            }
        }

        public string createBucket(string bucket, SortedList headers, SortedList metadata)
        {
            return generateURL("PUT", bucket, "", mergeMeta(headers, metadata));
        }

        public string listBucket(string bucket, string prefix, string marker,
                                 int maxKeys, SortedList headers)
        {
            return listBucket(bucket, prefix, marker, maxKeys, null, headers);
        }

        public string listBucket(string bucket, string prefix, string marker,
                                 int maxKeys, string delimiter, SortedList headers)
        {
            SortedList query = Utils.QueryForListOptions(prefix, marker, maxKeys, delimiter);
            return generateURL("GET", bucket, "", query, headers);
        }

        public string deleteBucket(string bucket, SortedList headers)
        {
            return generateURL("DELETE", bucket, "", headers);
        }

        public string put(string bucket, string key, S3Object obj, SortedList headers)
        {
            SortedList metadata = null;
            if (obj != null)
            {
                metadata = obj.Metadata;
            }

            return generateURL("PUT", bucket, key, mergeMeta(headers, metadata));
        }

        public string get(string bucket, string key, SortedList headers)
        {
            return generateURL("GET", bucket, key, headers);
        }

        public string delete(string bucket, string key, SortedList headers)
        {
            return generateURL("DELETE", bucket, key, headers);
        }

        public string getBucketLogging(string bucket, SortedList headers)
        {
            var query = new SortedList {{"logging", ""}};
            return generateURL("GET", bucket, "", query, headers);
        }

        public string putBucketLogging(string bucket, SortedList headers)
        {
            var query = new SortedList {{"logging", ""}};
            return generateURL("PUT", bucket, "", query, headers);
        }

        public string getBucketACL(string bucket, SortedList headers)
        {
            var query = new SortedList {{"acl", ""}};
            return generateURL("GET", bucket, "", query, headers);
        }

        public string getACL(string bucket, string key, SortedList headers)
        {
            var query = new SortedList {{"acl", ""}};
            return generateURL("GET", bucket, key, query, headers);
        }

        public string putBucketACL(string bucket, SortedList headers)
        {
            var query = new SortedList {{"acl", ""}};
            return generateURL("PUT", bucket, "", query, headers);
        }

        public string putACL(string bucket, string key, SortedList headers)
        {
            var query = new SortedList {{"acl", ""}};
            return generateURL("PUT", bucket, key, query, headers);
        }

        public string listAllMyBuckets(SortedList headers)
        {
            return generateURL("GET", "", "", headers);
        }

        public string makeBaseURL(string bucket, string key)
        {
            key = Utils.urlEncode(key);
            return makeBaseURL(new StringBuilder(), bucket, key).ToString();
        }

        private StringBuilder makeBaseURL(StringBuilder builder, string bucket, string key)
        {
            builder.Append(_isSecure ? "https://" : "http://");
            builder.Append(Utils.BuildUrlBase(_server, _port, bucket, _callingFormat));
            if (!string.IsNullOrEmpty(bucket) &&
                !string.IsNullOrEmpty(key))
            {
                builder.Append(key);
            }
            return builder;
        }

        private string generateURL(string method, string bucket, string key, SortedList headers)
        {
            return generateURL(method, bucket, key, new SortedList(), headers);
        }

        private string generateURL(string method, string bucket, string key, SortedList queryParameters,
                                   SortedList headers)
        {
            var expires = 0L;
            if (_expiresIn != NotSet)
            {
                expires = Utils.CurrentTimeMillis() + _expiresIn;
            }
            else if (_expires != NotSet)
            {
                expires = _expires;
            }
            else
            {
                throw new Exception("Illegal expire state!");
            }

            // convert to seconds
            expires /= 1000;

            // escape key to prepare for being part of url
            key = Utils.urlEncode(key);

            var canonicalString = Utils.MakeCanonicalString(method, bucket, key, queryParameters, headers,
                                                               "" + expires);
            var encodedCanonical = Utils.Encode(_awsSecretAccessKey, canonicalString, true);

            queryParameters.Add("Signature", encodedCanonical);
            queryParameters.Add("Expires", "" + expires);
            if (_awsAccessKeyId != null)
            {
                queryParameters.Add("AWSAccessKeyId", _awsAccessKeyId);
            }
            if (_tokens != null && _tokens.Length > 0)
            {
                queryParameters.Add("x-amz-security-token", string.Join(",", _tokens));
            }

            var builder = new StringBuilder();
            makeBaseURL(builder, bucket, key);
            builder.Append(Utils.ConvertQueryListToQueryString(queryParameters));
            return builder.ToString();
        }

        private SortedList mergeMeta(SortedList headers, SortedList metadata)
        {
            var merged = new SortedList();
            if (headers != null)
            {
                foreach (string key in headers.Keys)
                {
                    merged.Add(key, headers[key]);
                }
            }

            if (metadata != null)
            {
                foreach (string key in metadata.Keys)
                {
                    var existing = merged[key] as string;
                    if (existing != null)
                    {
                        existing += "," + metadata[key];
                    }
                    else
                    {
                        existing = metadata[key] as string;
                    }
                    merged.Add(key, existing);
                }
            }

            return merged;
        }
    }
}