// This software code is made available "AS IS" without warranties of any        
// kind.  You may copy, display, modify and redistribute the software            
// code either by itself or as incorporated into your code; provided that        
// you do not remove any proprietary notices.  Your use of this software         
// code is at your own risk and you waive any claim against Amazon               
// Digital Services, Inc. or its affiliates with respect to your use of          
// this software code. (c) 2006-2007 Amazon Digital Services, Inc. or its             
// affiliates.          


using System;
using System.Collections;
using System.Net;
using System.Text;

namespace ImageNexus.BenScharbach.ICLTools.S3Sample
{
    internal class S3Test
    {
        private const string AwsAccessKeyId = "<INSERT YOUR AWS ACCESS KEY ID HERE>";
        private const string AwsSecretAccessKey = "<INSERT YOUR AWS SECRET ACCESS KEY HERE>";

        private const HttpStatusCode HttpOk = HttpStatusCode.OK;
        private const HttpStatusCode HttpNoContent = HttpStatusCode.NoContent;
        private const int UnspecifiedMaxKeys = -1;

        private static readonly string BucketName = AwsAccessKeyId.ToLower() + "-test-bucket";
        private static readonly CallingFormat[] CallingFormats = {CallingFormat.Path, CallingFormat.Subdomain};
        private static int _assertionCount;

        private static int Main(string[] args)
        {
            if (AwsAccessKeyId.StartsWith("<INSERT"))
            {
                Console.WriteLine("Please examine S3Test.cs and update it with your credentials");
                return -1;
            }

            testUrlEscaping();

            // test all operation for both regular and vanity domains
            // regular: http://s3.amazonaws.com/key
            // subdomain: http://bucket.s3.amazonaws.com/key
            // testing pure vanity domains (http://<vanity domain>/key) is not covered here
            // but is possible with some assitional setup 

            foreach (CallingFormat format in CallingFormats)
            {
                var conn = new AWSAuthConnection(AwsAccessKeyId, AwsSecretAccessKey, new string[0], false, format);

                using (Response response = conn.CreateBucket(BucketName, Location.Default, null))
                {
                    assertEquals(
                        HttpOk,
                        response.Status,
                        "couldn't create bucket");
                }

                using (ListBucketResponse listBucketResponse = conn.ListBucket(BucketName, null, null, 0, null))
                {
                    assertEquals(
                        HttpOk,
                        listBucketResponse.Status,
                        "Couldn't get list");
                    assertEquals(
                        0,
                        listBucketResponse.Entries.Count,
                        "list wasn't empty");
                    verifyBucketResponseParameters(listBucketResponse, BucketName, "", "", UnspecifiedMaxKeys, null,
                                                   false, null);
                }

                // start delimiter tests

                const string text = "this is a test";
                const string key = "example.txt";
                const string innerKey = "test/inner.txt";
                const string badcharKey = "x+/last{key}.txt";
                const string lastKey = "z-last-key.txt";

                using (Response response = conn.Put(BucketName, key, new S3Object(text, null), null))
                {
                    assertEquals(
                        HttpOk,
                        response.Status,
                        "couldn't put simple object");
                }

                using (Response response = conn.Put(BucketName, innerKey, new S3Object(text, null), null))
                {
                    assertEquals(
                        HttpOk,
                        response.Status,
                        "couldn't put simple object");
                }

                using (Response response = conn.Put(BucketName, lastKey, new S3Object(text, null), null))
                {
                    assertEquals(
                        HttpOk,
                        response.Status,
                        "couldn't put simple object");
                }

                using (Response response = conn.Put(BucketName, badcharKey, new S3Object(text, null), null))
                {
                    assertEquals(
                        HttpOk,
                        response.Status,
                        "couldn't put simple object");
                }

                // plain list
                using (ListBucketResponse listBucketResponse = conn.ListBucket(BucketName, null, null, 0, null))
                {
                    assertEquals(
                        listBucketResponse.Status,
                        HttpOk,
                        "couldn't get list");
                    assertEquals(4, listBucketResponse.Entries.Count, "Unexpected list size");
                    assertEquals(0, listBucketResponse.CommonPrefixEntries.Count, "Unexpected common prefix size");
                    verifyBucketResponseParameters(listBucketResponse, BucketName, "", "", UnspecifiedMaxKeys, null,
                                                   false, null);
                }

                // root "directory"
                using (ListBucketResponse listBucketResponse = conn.ListBucket(BucketName, null, null, 0, "/", null))
                {
                    assertEquals(
                        HttpOk,
                        listBucketResponse.Status,
                        "couldn't get list");
                    assertEquals(2, listBucketResponse.Entries.Count, "Unexpected list size");
                    assertEquals(2, listBucketResponse.CommonPrefixEntries.Count, "Unexpected common prefix size");
                    verifyBucketResponseParameters(listBucketResponse, BucketName, "", "", UnspecifiedMaxKeys, "/",
                                                   false, null);
                }

                // root "directory" with a max-keys of "1"
                using (ListBucketResponse listBucketResponse = conn.ListBucket(BucketName, null, null, 1, "/", null))
                {
                    assertEquals(
                        HttpOk,
                        listBucketResponse.Status,
                        "couldn't get list");
                    assertEquals(1, listBucketResponse.Entries.Count, "Unexpected list size");
                    assertEquals(0, listBucketResponse.CommonPrefixEntries.Count, "Unexpected common prefix size");
                    verifyBucketResponseParameters(listBucketResponse, BucketName, "", "", 1, "/", true, "example.txt");
                }

                // root "directory" with a max-keys of "2"
                string marker = null;
                using (ListBucketResponse listBucketResponse = conn.ListBucket(BucketName, null, null, 2, "/", null))
                {
                    assertEquals(
                        HttpOk,
                        listBucketResponse.Status,
                        "couldn't get list");
                    assertEquals(1, listBucketResponse.Entries.Count, "Unexpected list size");
                    assertEquals(1, listBucketResponse.CommonPrefixEntries.Count, "Unexpected common prefix size");
                    verifyBucketResponseParameters(listBucketResponse, BucketName, "", "", 2, "/", true, "test/");
                    marker = listBucketResponse.NextMarker;
                }
                using (ListBucketResponse listBucketResponse = conn.ListBucket(BucketName, null, marker, 2, "/", null))
                {
                    assertEquals(
                        HttpOk,
                        listBucketResponse.Status,
                        "couldn't get list");
                    assertEquals(1, listBucketResponse.Entries.Count, "Unexpected list size");
                    assertEquals(1, listBucketResponse.CommonPrefixEntries.Count, "Unexpected common prefix size");
                    verifyBucketResponseParameters(listBucketResponse, BucketName, "", marker, 2, "/", false, null);
                }

                // test "directory" (really just prefix)
                using (ListBucketResponse listBucketResponse = conn.ListBucket(BucketName, "test/", null, 0, "/", null))
                {
                    assertEquals(
                        HttpOk,
                        listBucketResponse.Status,
                        "couldn't get list");
                    assertEquals(1, listBucketResponse.Entries.Count, "Unexpected list size");
                    assertEquals(0, listBucketResponse.CommonPrefixEntries.Count, "Unexpected common prefix size");
                    verifyBucketResponseParameters(listBucketResponse, BucketName, "test/", "", UnspecifiedMaxKeys, "/",
                                                   false, null);
                }

                // test "directory" 'x+' (really just prefix)
                using (ListBucketResponse listBucketResponse = conn.ListBucket(BucketName, "x+/", null, 0, "/", null))
                {
                    assertEquals(
                        HttpOk,
                        listBucketResponse.Status,
                        "couldn't get list");
                    assertEquals(1, listBucketResponse.Entries.Count, "Unexpected list size");
                    assertEquals(0, listBucketResponse.CommonPrefixEntries.Count, "Unexpected common prefix size");
                    verifyBucketResponseParameters(listBucketResponse, BucketName, "x+/", "", UnspecifiedMaxKeys, "/",
                                                   false, null);
                }

                // remove innerkey
                using (Response response = conn.Delete(BucketName, innerKey, null))
                {
                    assertEquals(
                        HttpNoContent,
                        response.Status,
                        "couldn't delete entry");
                }

                // remove badcharKey
                using (Response response = conn.Delete(BucketName, badcharKey, null))
                {
                    assertEquals(
                        HttpNoContent,
                        response.Status,
                        "couldn't delete entry");
                }

                // remove last key
                using (Response response = conn.Delete(BucketName, lastKey, null))
                {
                    assertEquals(
                        HttpNoContent,
                        response.Status,
                        "couldn't delete entry");
                }

                // End of delimiter tests

                var metadata = new SortedList {{"title", "title"}};
                using (Response response = conn.Put(BucketName, key, new S3Object(text, metadata), null))
                {
                    assertEquals(
                        HttpOk,
                        response.Status,
                        "couldn't put complex object");
                }

                using (GetResponse getResponse = conn.Get(BucketName, key, null))
                {
                    assertEquals(
                        HttpOk,
                        getResponse.Status,
                        "couldn't get object");
                    assertEquals(text, getResponse.Object.Data, "didn't get the right data back");
                    assertEquals(1, getResponse.Object.Metadata.Count, "didn't get the right metadata back");
                    assertEquals(
                        "title",
                        getResponse.Object.Metadata["title"],
                        "didn't get the right metadata back");
                    assertEquals(
                        (long) text.Length,
                        getResponse.Connection.ContentLength,
                        "didn't get the right content-length");
                }

                // end delimiter tests

                using (Response response = conn.Put(BucketName, key, new S3Object(text, null), null))
                {
                    assertEquals(
                        HttpOk,
                        response.Status,
                        "couldn't put simple object");
                }

                var metadataWithSpaces = new SortedList();
                const string titleWithSpaces = " \t  title with leading and trailing spaces    ";
                metadataWithSpaces.Add("title", titleWithSpaces);
                using (Response response = conn.Put(BucketName, key, new S3Object(text, metadataWithSpaces), null))
                {
                    assertEquals(
                        HttpOk,
                        response.Status,
                        "couldn't put metadata with leading and trailing spaces");
                }

                using (GetResponse getResponse = conn.Get(BucketName, key, null))
                {
                    assertEquals(
                        HttpOk,
                        getResponse.Status,
                        "couldn't get object");
                    assertEquals(1, getResponse.Object.Metadata.Count, "didn't get the right metadata back");
                    assertEquals(
                        titleWithSpaces.Trim(),
                        getResponse.Object.Metadata["title"],
                        "didn't get the right metadata back");
                }

                const string weirdKey = "&=/%# ++/++";
                using (Response response = conn.Put(BucketName, weirdKey, new S3Object(text, null), null))
                {
                    assertEquals(
                        HttpOk,
                        response.Status,
                        "couldn't put weird key");
                }

                using (GetResponse getResponse = conn.Get(BucketName, weirdKey, null))
                {
                    assertEquals(
                        HttpOk,
                        getResponse.Status,
                        "couldn't get weird key");
                }

                string acl = null;
                using (GetResponse getResponse = conn.GetAcl(BucketName, key, null))
                {
                    assertEquals(
                        HttpOk,
                        getResponse.Status,
                        "couldn't get acl");
                    acl = getResponse.Object.Data;
                }


                using (Response response = conn.PutAcl(BucketName, key, acl, null))
                {
                    assertEquals(
                        HttpOk,
                        response.Status,
                        "couldn't put acl");
                }

                string bucketACL = null;
                using (GetResponse getResponse = conn.GetBucketACL(BucketName, null))
                {
                    assertEquals(
                        HttpOk,
                        getResponse.Status,
                        "couldn't get bucket acl");
                    bucketACL = getResponse.Object.Data;
                }

                using (Response response = conn.PutBucketAcl(BucketName, bucketACL, null))
                {
                    assertEquals(
                        HttpOk,
                        response.Status,
                        "couldn't put bucket acl");
                }

                // bucket logging tests
                string bucketLogging;
                using (GetResponse getResponse = conn.GetBucketLogging(BucketName, null))
                {
                    assertEquals(
                        HttpOk,
                        getResponse.Status,
                        "couldn't get bucket logging config");
                    bucketLogging = getResponse.Object.Data;
                }

                using (Response response = conn.PutBucketLogging(BucketName, bucketLogging, null))
                {
                    assertEquals(
                        HttpOk,
                        response.Status,
                        "couldn't put bucket logging config");
                }

                // end bucket logging tests

                using (ListBucketResponse listBucketResponse = conn.ListBucket(BucketName, null, null, 0, null))
                {
                    assertEquals(
                        HttpOk,
                        listBucketResponse.Status,
                        "couldn't list bucket");
                    ArrayList entries = listBucketResponse.Entries;
                    assertEquals(2, entries.Count, "didn't get back the right number of entries");
                    // depends on weirdKey < $key
                    assertEquals(weirdKey, ((ListEntry) entries[0]).Key, "first key isn't right");
                    assertEquals(key, ((ListEntry) entries[1]).Key, "second key isn't right");
                    verifyBucketResponseParameters(listBucketResponse, BucketName, "", "", UnspecifiedMaxKeys, null,
                                                   false, null);
                }

                using (ListBucketResponse listBucketResponse = conn.ListBucket(BucketName, null, null, 1, null))
                {
                    assertEquals(
                        HttpOk,
                        listBucketResponse.Status,
                        "couldn't list bucket");
                    assertEquals(
                        1,
                        listBucketResponse.Entries.Count,
                        "didn't get back the right number of entries");
                    verifyBucketResponseParameters(listBucketResponse, BucketName, "", "", 1, null, true, null);
                }
                using (ListBucketResponse listBucketResponse = conn.ListBucket(BucketName, null, null, 0, null))
                {
                    assertEquals(
                        HttpOk,
                        listBucketResponse.Status,
                        "couldn't list bucket");

                    // delete the bucket's content
                    foreach (ListEntry entry in listBucketResponse.Entries)
                    {
                        using (Response response = conn.Delete(BucketName, entry.Key, null))
                        {
                            assertEquals(
                                HttpNoContent,
                                response.Status,
                                "couldn't delete entry");
                        }
                    }
                }

                ArrayList buckets = null;
                using (ListAllMyBucketsResponse listAllMyBucketsResponse = conn.ListAllMyBuckets(null))
                {
                    assertEquals(
                        HttpOk,
                        listAllMyBucketsResponse.Status,
                        "couldn't list all my buckets");
                    buckets = listAllMyBucketsResponse.Buckets;
                }

                using (Response response = conn.DeleteBucket(BucketName, null))
                {
                    assertEquals(
                        HttpNoContent,
                        response.Status,
                        "couldn't delete bucket");
                }

                using (ListAllMyBucketsResponse listAllMyBucketsResponse = conn.ListAllMyBuckets(null))
                {
                    assertEquals(
                        HttpOk,
                        listAllMyBucketsResponse.Status,
                        "couldn't list all my buckets");
                    assertEquals(
                        buckets.Count - 1,
                        listAllMyBucketsResponse.Buckets.Count,
                        "bucket count is incorrect");
                }

                var generator =
                    new QueryStringAuthGenerator(AwsAccessKeyId, AwsSecretAccessKey, false);

                checkURL(
                    generator.createBucket(BucketName, null, null),
                    "PUT",
                    HttpOk,
                    "couldn't create bucket");
                checkURL(
                    generator.put(BucketName, key, new S3Object("test data", null), null),
                    "PUT",
                    HttpOk,
                    "put object",
                    "test data");
                checkURL(
                    generator.get(BucketName, key, null),
                    "GET",
                    HttpOk,
                    "get object");
                checkURL(
                    generator.listBucket(BucketName, null, null, 0, null),
                    "GET",
                    HttpOk,
                    "list bucket");
                checkURL(
                    generator.listAllMyBuckets(null),
                    "GET",
                    HttpOk,
                    "list all my buckets");
                checkURL(
                    generator.getACL(BucketName, key, null),
                    "GET",
                    HttpOk,
                    "get acl");
                checkURL(
                    generator.putACL(BucketName, key, null),
                    "PUT",
                    HttpOk,
                    "put acl",
                    acl);
                checkURL(
                    generator.getBucketACL(BucketName, null),
                    "GET",
                    HttpOk,
                    "get bucket acl");
                checkURL(
                    generator.putBucketACL(BucketName, null),
                    "PUT",
                    HttpOk,
                    "put bucket acl",
                    bucketACL);
                checkURL(
                    generator.delete(BucketName, key, null),
                    "DELETE",
                    HttpNoContent,
                    "delete object");
                checkURL(
                    generator.deleteBucket(BucketName, null),
                    "DELETE",
                    HttpNoContent,
                    "delete bucket");
            }

            Console.WriteLine("OK (" + _assertionCount + " tests passed)");

            return 0;
        }

        private static void verifyBucketResponseParameters(ListBucketResponse listBucketResponse,
                                                           string bucketName, string prefix, string marker,
                                                           int maxKeys, string delimiter, bool isTruncated,
                                                           string nextMarker)
        {
            assertEquals(bucketName, listBucketResponse.Name, "Bucket name should match.");
            assertEquals(prefix, listBucketResponse.Prefix, "Bucket prefix should match.");
            assertEquals(marker, listBucketResponse.Marker, "Bucket marker should match.");
            assertEquals(delimiter, listBucketResponse.Delimiter, "Bucket delimiter should match.");
            if (UnspecifiedMaxKeys != maxKeys)
            {
                assertEquals(maxKeys, listBucketResponse.MaxKeys, "Bucket max-keys should match.");
            }
            assertEquals(isTruncated, listBucketResponse.IsTruncated, "Bucket should not be truncated.");
            assertEquals(nextMarker, listBucketResponse.NextMarker, "Bucket nextMarker should match.");
        }

        private static void assertEquals(int expected, int actual, string message)
        {
            _assertionCount++;
            if (expected != actual)
            {
                throw new Exception(message + ": expected " + expected + " but got " + actual);
            }
        }

        private static void assertEquals(byte[] expected, byte[] actual, string message)
        {
            _assertionCount++;
            if (expected.Length != actual.Length)
            {
                throw new Exception("Lengths do not match: " + expected.Length + " vs " + actual.Length);
            }
            for (int i = 0; i < expected.Length; ++i)
            {
                if (expected[i] != actual[i])
                {
                    throw new Exception("Expected <" + expected[i] + "> but got <" + actual[i] + "> at index " + i);
                }
            }
        }

        private static void assertEquals(object expected, object actual, string message)
        {
            _assertionCount++;
            if (expected != actual && (actual == null || !actual.Equals(expected)))
            {
                throw new Exception(message + ": expected " + expected + " but got " + actual);
            }
        }

        private static void checkURL(string url, string method, HttpStatusCode code, string message)
        {
            checkURL(url, method, code, message, null);
        }

        private static void checkURL(string url, string method, HttpStatusCode code, string message, string data)
        {
            if (data == null) data = "";

            WebRequest connection = WebRequest.Create(url);
            connection.Method = method;
            if ("PUT".Equals(method))
            {
                var encoding = new ASCIIEncoding();
                byte[] bytes = encoding.GetBytes(data);

                connection.ContentLength = bytes.Length;
                connection.GetRequestStream().Write(bytes, 0, bytes.Length);
            }

            try
            {
                assertEquals(code, (connection.GetResponse() as HttpWebResponse).StatusCode, message);
            }
            catch (WebException ex)
            {
                string msg = ex.Response != null
                                 ? Utils.SlurpInputStreamAsString(ex.Response.GetResponseStream())
                                 : ex.Message;
                throw new WebException(msg, ex, ex.Status, ex.Response);
            }
            connection.GetResponse().Close();
        }

        private static void testUrlEscaping()
        {
            for (int i = 1; i < 256; i++)
            {
                // These are special cases that the below test can't properly check for
                if (i == '\\' || i == '#' || i == '%' || i == '+')
                    continue;
                String path = "/ab" + new String((char) i, 2) + "cd";
                const string query = "?zyx=123";
                String escaped = Utils.urlEncode(path) + query;
                var uri = new Uri("http://nowhere" + path + query);
                assertEquals(escaped, uri.PathAndQuery,
                             "Utils.urlEncode() did not match Uri processing");
            }
        }
    }
}