// This software code is made available "AS IS" without warranties of any        
// kind.  You may copy, display, modify and redistribute the software            
// code either by itself or as incorporated into your code; provided that        
// you do not remove any proprietary notices.  Your use of this software         
// code is at your own risk and you waive any claim against Amazon               
// Digital Services, Inc. or its affiliates with respect to your use of          
// this software code. (c) 2006-2007 Amazon Digital Services, Inc. or its             
// affiliates.          


using System;
using System.Collections;
using System.Globalization;
using System.IO;
using System.Net;
using System.Security.Cryptography;
using System.Text;
using System.Web;
using System.Xml;

namespace ImageNexus.BenScharbach.ICLTools.S3Sample
{
    public enum CallingFormat
    {
        Path, // http://s3.amazonaws.com/key
        Subdomain, // http://bucket.s3.amazonaws.com/key
        Vanity // http://mydomain.com/key -- a vanity domain which resolves to s3.amazonaws.com
    }

    public static class Location
    {
        public const string Default = null;
        public const string EU = "EU";
    }

    public class Utils
    {
        public static readonly string MetadataPrefix = "x-amz-meta-";
        public static readonly string AmazonHeaderPrefix = "x-amz-";
        public static readonly string AlternativeDateHeader = "x-amz-date";

        private static string _host = "s3.amazonaws.com";

        private static int _securePort = 443;

        private static int _insecurePort = 80;

        public static string Host
        {
            get { return _host; }
            set { _host = value; }
        }

        public static int SecurePort
        {
            get { return _securePort; }
            set { _securePort = value; }
        }

        public static int InsecurePort
        {
            get { return _insecurePort; }
            set { _insecurePort = value; }
        }

        public static string MakeCanonicalString(string bucket, string key, WebRequest request)
        {
            return MakeCanonicalString(bucket, key, new SortedList(), request);
        }

        public static string MakeCanonicalString(string bucket, string key, SortedList query, WebRequest request)
        {
            var headers = new SortedList();
            foreach (string header in request.Headers)
            {
                headers.Add(header, request.Headers[header]);
            }
            return MakeCanonicalString(request.Method, bucket, key, query, headers, null);
        }

        public static string MakeCanonicalString(string verb, string bucketName, string key, SortedList queryParams,
                                                 SortedList headers, string expires)
        {
            var buf = new StringBuilder();
            buf.Append(verb);
            buf.Append("\n");

            var interestingHeaders = new SortedList();
            if (headers != null)
            {
                foreach (string header in headers.Keys)
                {
                    string lk = header.ToLower();
                    if (lk.Equals("content-type") ||
                        lk.Equals("content-md5") ||
                        lk.Equals("date") ||
                        lk.StartsWith(AmazonHeaderPrefix))
                    {
                        interestingHeaders.Add(lk, headers[header]);
                    }
                }
            }
            if (interestingHeaders[AlternativeDateHeader] != null)
            {
                interestingHeaders.Add("date", "");
            }

            // if the expires is non-null, use that for the date field.  this
            // trumps the x-amz-date behavior.
            if (expires != null)
            {
                interestingHeaders.Add("date", expires);
            }

            // these headers require that we still put a new line after them,
            // even if they don't exist.
            {
                string[] newlineHeaders = {"content-type", "content-md5"};
                foreach (string header in newlineHeaders)
                {
                    if (interestingHeaders.IndexOfKey(header) == -1)
                    {
                        interestingHeaders.Add(header, "");
                    }
                }
            }

            // Finally, add all the interesting headers (i.e.: all that startwith x-amz- ;-))
            foreach (string header in interestingHeaders.Keys)
            {
                if (header.StartsWith(AmazonHeaderPrefix))
                {
                    buf.Append(header).Append(":").Append((interestingHeaders[header] as string).Trim());
                }
                else
                {
                    buf.Append(interestingHeaders[header]);
                }
                buf.Append("\n");
            }

            // Build the path using the bucket and key
            buf.Append("/");
            if (bucketName != null && !bucketName.Equals(""))
            {
                buf.Append(bucketName);
                buf.Append("/");
            }

            // Append the key (it may be an empty string)
            if (!string.IsNullOrEmpty(key))
            {
                buf.Append(key);
            }

            // if there is an acl, logging, or torrent paramter, add them to the string.
            if (queryParams != null)
            {
                if (queryParams.IndexOfKey("acl") != -1)
                {
                    buf.Append("?acl");
                }
                else if (queryParams.IndexOfKey("torrent") != -1)
                {
                    buf.Append("?torrent");
                }
                else if (queryParams.IndexOfKey("logging") != -1)
                {
                    buf.Append("?logging");
                }
                else if (queryParams.IndexOfKey("location") != -1)
                {
                    buf.Append("?location");
                }
            }

            return buf.ToString();
        }

        public static string Encode(string awsSecretAccessKey, string canonicalString, bool urlEncode)
        {
            Encoding ae = new UTF8Encoding();
            var signature = new HMACSHA1(ae.GetBytes(awsSecretAccessKey));
            string b64 = Convert.ToBase64String(
                signature.ComputeHash(ae.GetBytes(
                    canonicalString.ToCharArray()))
                );

            return urlEncode ? HttpUtility.UrlEncode(b64) : b64;
        }

        public static byte[] SlurpInputStream(Stream stream)
        {
            using (var ms = new MemoryStream())
            {
                var buffer = new byte[32*1024];
                while (true)
                {
                    int nRead = stream.Read(buffer, 0, buffer.Length);
                    if (nRead <= 0)
                    {
                        return ms.ToArray();
                    }
                    ms.Write(buffer, 0, nRead);
                }
            }
        }

        public static string SlurpInputStreamAsString(Stream stream)
        {
            var encoding = new ASCIIEncoding();
            return encoding.GetString(SlurpInputStream(stream));
        }

        public static string GetXmlChildText(XmlNode data)
        {
            var builder = new StringBuilder();
            foreach (XmlNode node in data.ChildNodes)
            {
                if (node.NodeType == XmlNodeType.Text ||
                    node.NodeType == XmlNodeType.CDATA)
                {
                    builder.Append(node.Value);
                }
            }
            return builder.ToString();
        }

        public static DateTime ParseDate(string dateStr)
        {
            return DateTime.Parse(dateStr);
        }

        public static string GetHttpDate()
        {
            // Setting the Culture will ensure we get a proper HTTP Date.
            string date = DateTime.UtcNow.ToString("ddd, dd MMM yyyy HH:mm:ss ", CultureInfo.InvariantCulture) + "GMT";
            return date;
        }

        public static long CurrentTimeMillis()
        {
            return (long) (DateTime.UtcNow - new DateTime(1970, 1, 1)).TotalMilliseconds;
        }

        /// <summary>
        /// Calculates the endpoint based on the calling format.
        /// </summary>
        public static string BuildUrlBase(string server, int port, string bucket, CallingFormat format)
        {
            var endpoint = new StringBuilder();

            switch (format)
            {
                case CallingFormat.Path:
                    endpoint.Append(server);
                    endpoint.Append(":");
                    endpoint.Append(port);
                    if (bucket != null && ! bucket.Equals(""))
                    {
                        endpoint.Append("/");
                        endpoint.Append(bucket);
                    }
                    break;
                case CallingFormat.Subdomain:
                    if (bucket.Length != 0)
                    {
                        endpoint.Append(bucket);
                        endpoint.Append(".");
                    }
                    endpoint.Append(server);
                    endpoint.Append(":");
                    endpoint.Append(port);
                    break;
                case CallingFormat.Vanity:
                    endpoint.Append(bucket);
                    endpoint.Append(":");
                    endpoint.Append(port);
                    break;
            }
            endpoint.Append("/");
            return endpoint.ToString();
        }

        /// <summary>
        /// Escape a string that is destined to be part of a URL path.
        /// The Uri class tries to be smart about cleaning up the URL
        /// but we need to know exactly what the URL will look like 
        /// on the wire.  This is the result of trial-and-error
        /// analysis of what the Uri escapes and what it doesn't.
        /// </summary>
        public static string urlEncode(string path)
        {
            const string extra = "?something";
            var uri = new Uri("http://nowhere/a" + path + extra);
            String result = uri.PathAndQuery;
            // trim leading '/a' and trailing '?something'
            if (result.StartsWith("/a"))
                result = result.Substring(2);
            if (result.EndsWith(extra))
                result = result.Substring(0, result.Length - extra.Length);
            // special case '+', as S3 interprets unescaped + as space...
            result = result.Replace("+", "%2B");
            // if Uri parser lost some of the text, fall back to just UrlEncode
            if (path != HttpUtility.UrlDecode(result))
            {
                result = HttpUtility.UrlEncode(path);
                result = result.Replace("%2f", "/");
            }
            return result;
        }

        // 4/13/2011
        public static SortedList QueryForMultiPartUploadInitiate()
        {
            var queryStrings = new SortedList();
            queryStrings.Add("uploads", HttpUtility.UrlEncode(""));
            return queryStrings;
        }

        public static SortedList QueryForListOptions(string prefix, string marker, int maxKeys)
        {
            return QueryForListOptions(prefix, marker, maxKeys, null);
        }

        public static SortedList QueryForListOptions(string prefix, string marker, int maxKeys, string delimiter)
        {
            var queryStrings = new SortedList();
            if (prefix != null) queryStrings.Add("prefix", HttpUtility.UrlEncode(prefix));
            if (marker != null) queryStrings.Add("marker", HttpUtility.UrlEncode(marker));
            if (maxKeys != 0) queryStrings.Add("max-keys", "" + maxKeys);
            if (delimiter != null) queryStrings.Add("delimiter", HttpUtility.UrlEncode(delimiter));
            return queryStrings;
        }

        public static string ConvertQueryListToQueryString(SortedList query)
        {
            var queryString = new StringBuilder();
            var firstParameter = true;
            if (query != null)
            {
                foreach (string key in query.Keys)
                {
                    var value = (string) query[key];
                    queryString.Append(firstParameter ? "?" : "&");
                    queryString.Append(key);
                    if (!string.IsNullOrEmpty(value))
                        queryString.Append("=").Append(value);
                    firstParameter = false;
                }
            }
            return queryString.ToString();
        }
    }
}