/******************************************************************************* 
 *  Copyright 2007 Amazon Technologies, Inc.  
 *  Licensed under the Apache License, Version 2.0 (the "License"); 
 *  
 *  You may not use this file except in compliance with the License. 
 *  You may obtain a copy of the License at: http://aws.amazon.com/apache2.0
 *  This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR 
 *  CONDITIONS OF ANY KIND, either express or implied. See the License for the 
 *  specific language governing permissions and limitations under the License.
 *******************************************************************************/

using System;
using System.IO;
using System.Xml;

namespace ImageNexus.BenScharbach.ICLTools.CredentialStore.Security
{
    /// <summary>
    /// This class stores credentials in a file, encrypting
    /// the individual pieces of data.
    /// </summary>
    public sealed class AmazonFileCredentialStore : AbstractCredentialStore
    {
        private readonly string _dataFolder = Environment.GetFolderPath(Environment.SpecialFolder.LocalApplicationData);

        private readonly string _desktopCredentialsFilename;
        private string _userToken;
        private string _accessKeyId;
        private string _secretAccessKey;


        #region Properties

        // 4/29/2011
        /// <summary>
        /// Does the Credential File exist?
        /// </summary>
        public bool CredentialsExist { get; private set; }

        /// <summary>
        /// UserToken as string.
        /// </summary>
        public String UserToken
        {
            get { return _userToken; }
            set { _userToken = value; }
        }

        /// <summary>
        /// Access key ID as string.
        /// </summary>
        public String AccessKeyId
        {
            get { return _accessKeyId; }
            set { _accessKeyId = value; }
        }

        /// <summary>
        /// Secret Access key as string.
        /// </summary>
        public String SecretAccessKey
        {
            get { return _secretAccessKey; }
            set { _secretAccessKey = value; }
        }

        #endregion


        /// <summary>
        /// Instantiates a AmazonFileCredentialStore.
        /// </summary>
        /// <param name="password">The password that will be used to encrypt data.</param>
        /// <param name="filename">The filename to store the credentials to.</param>
        public AmazonFileCredentialStore(string password, string filename)
            : base(password)
        {
            // 4/11/2011 - Create full path.
            // Create credentials filepath.
            _desktopCredentialsFilename = Path.Combine(_dataFolder, filename);

            // 4/29/2011 - Checks if file exist.
            CredentialsExist = Read();
        }
        

        /// <summary>
        /// Reads the file.  
        /// </summary>
        public override bool Read()
        {
            if (!DoesFileStoreExist()) return false;

            var doc = new XmlDocument();
            try
            {
                // Open the file and read it back.
                var document = File.ReadAllText(_desktopCredentialsFilename);
                doc.LoadXml(document);

                _accessKeyId = Decrypt(
                    doc.SelectSingleNode("credentials/accessKeyId").InnerText);
                _secretAccessKey = Decrypt(
                    doc.SelectSingleNode("credentials/secretAccessKey").InnerText);
                _userToken = Decrypt(
                    doc.SelectSingleNode("credentials/userToken").InnerText);

                return true;
            }
            catch (Exception ex)
            {
                //Do nothing- we'll assume that the file isn't good.
                ex.ToString();
                return false;
            }
        }

        // 4/11/2011
        /// <summary>
        /// Checks if the CredentialStore file exist?
        /// </summary>
        /// <returns>true/false</returns>
        public bool DoesFileStoreExist()
        {
            return File.Exists(_desktopCredentialsFilename);
        }

        /// <summary>
        /// Stores the credentials in a file.
        /// </summary>
        public override void Store()
        {
            var xml = CreateXmlDocument(Encrypt(_userToken),
                Encrypt(_accessKeyId), Encrypt(_secretAccessKey));
            using (var writer = new StreamWriter(_desktopCredentialsFilename))
            {
                writer.Write(xml);
            }
        }

        /// <summary>
        /// Helper method to create a DOM 
        /// </summary>
        /// <param name="userToken">Security token representing the user.</param>
        /// <param name="accessKeyId">Used to identify the signer.</param>
        /// <param name="secretAccessKey">Key used to sign requests.</param>
        /// <returns>The xml document.</returns>
        private static string CreateXmlDocument(
            string userToken,
            string accessKeyId,
            string secretAccessKey)
        {
            var doc = new XmlDocument();
            // Write down the XML declaration
            var xmlDeclaration = doc.CreateXmlDeclaration("1.0", "utf-8", null);

            // Create the root element
            var rootNode = doc.CreateElement("credentials");
            doc.InsertBefore(xmlDeclaration, doc.DocumentElement);
            doc.AppendChild(rootNode);

            // Create userToken element 
            var userTokenElement = doc.CreateElement("userToken");
            userTokenElement.InnerText = userToken;
            rootNode.AppendChild(userTokenElement);

            // Create userAccessKeyId element 
            var userAccessKeyIdElement = doc.CreateElement("accessKeyId");
            userAccessKeyIdElement.InnerText = accessKeyId;
            rootNode.AppendChild(userAccessKeyIdElement);

            // Create userSecretAccess element 
            var userSecretAccessElement = doc.CreateElement("secretAccessKey");
            userSecretAccessElement.InnerText = secretAccessKey;
            rootNode.AppendChild(userSecretAccessElement);

            return doc.InnerXml;
        }

        /// <summary>
        /// Override ToString.
        /// </summary>
        /// <returns></returns>
        public override string ToString()
        {
            return "Filename: " + _desktopCredentialsFilename +
                " Password: " + Password +
                " UserToken: " + _userToken +
                " AccessKeyId: " + _accessKeyId +
                " SecretAccessKey: " + _secretAccessKey;
        }
    }
}
