/*******************************************************************************
 * This class was derived from:
 * http://msdn2.microsoft.com/en-us/library/system.security.cryptography.rijndaelmanaged.aspx
 *
 * Modifications to RijndaelManaged Class Copyright 2008 Amazon Technologies, Inc. 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * 
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at: http://aws.amazon.com/apache2.0
 * This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 *
 * Terms of use for RijndaelManaged Class:
 * This license governs use of the accompanying software. If you use the
 * software, you accept this license. If you do not accept the license, do not
 * use the software.
 *
 * 1. Definitions
 *      The terms reproduce, reproduction, derivative works, and
 *      distribution have the same meaning here as under U.S. copyright law.
 *
 *      A contribution is the original software, or any additions or changes
 *      to the software.
 *
 *      A contributor is any person that distributes its contribution under
 *      this license.
 *
 *      Licensed patents are a contributors patent claims that read directly
 *      on its contribution.
 *
 *  2. Grant of Rights
 *      (A) Copyright Grant- Subject to the terms of this license, including the
 *          license conditions and limitations in section 3, each contributor
 *          grants you a non-exclusive, worldwide, royalty-free copyright license
 *          to reproduce its contribution, prepare derivative works of its
 *          contribution, and distribute its contribution or any derivative
 *          works that you create.
 *      (B) Patent Grant- Subject to the terms of this license, including the
 *          license conditions and limitations in section 3, each contributor
 *          grants you a non-exclusive, worldwide, royalty-free license under
 *          its licensed patents to make, have made, use, sell, offer for sale,
 *          import, and/or otherwise dispose of its contribution in the software
 *          or derivative works of the contribution in the software.
 *  3. Conditions and Limitations
 *      (A) No Trademark License- This license does not grant you rights to use
 *          any contributors name, logo, or trademarks.
 *      (B) If you bring a patent claim against any contributor over patents that
 *          you claim are infringed by the software, your patent license from such
 *          contributor to the software ends automatically.
 *      (C) If you distribute any portion of the software, you must retain all
 *          copyright, patent, trademark, and attribution notices that are present
 *          in the software.
 *      (D) If you distribute any portion of the software in source code form, you
 *          may do so only under this license by including a complete copy of this
 *          license with your distribution. If you distribute any portion of the
 *          software in compiled or object code form, you may only do so under a
 *          license that complies with this license.
 *      (E) The software is licensed as-is. You bear the risk of using it. The
 *          contributors give no express warranties, guarantees or conditions. You
 *          may have additional consumer rights under your local laws which this
 *          license cannot change. To the extent permitted under your local laws,
 *          the contributors exclude the implied warranties of merchantability,
 *          fitness for a particular purpose and non-infringement.
 *
 *******************************************************************************/

using System;
using System.Text;
using System.IO;
using System.Security.Cryptography;

namespace ImageNexus.BenScharbach.ICLTools.CredentialStore.Security
{
    /// <summary>
    /// This class encrypts.  Default is AES 256.
    /// </summary>
    class Encryption
    {
        private readonly ICryptoTransform _encryptor;
        private readonly ICryptoTransform _decryptor;

        // Some sensible defaults.
        private const string Iv = "0000000000000000";
        private const string Salt = "Tn@2b8I/x.W$2^bY";
        private const int KeyLength = 32;

        /// <summary>
        /// Makes an encryptor.
        /// </summary>
        /// <param name="passPhrase"></param>
        public Encryption(string passPhrase)
        {
            var encoding = new ASCIIEncoding();
            var saltBytes = encoding.GetBytes(Salt);
            var ivBytes = encoding.GetBytes(Iv);

            var keyBytes = (new PasswordDeriveBytes(passPhrase, saltBytes)).GetBytes(KeyLength);

            // Declare the RijndaelManaged object
            // used to encrypt the data.
            RijndaelManaged aesAlg = null;

            // Create a RijndaelManaged object.
            aesAlg = new RijndaelManaged();

            // Create a decrytor to perform the stream transform.
            _encryptor = aesAlg.CreateEncryptor(keyBytes, ivBytes);

            // Create a decrytor to perform the stream transform.
            _decryptor = aesAlg.CreateDecryptor(keyBytes, ivBytes);
        }

        /// <summary>
        /// Encrypts plaintext.
        /// </summary>
        /// <param name="plainText">The text to encrypt.</param>
        /// <returns>ToBase64(AES-XXX(plaintext))</returns>
        public string Encrypt(string plainText)
        {
            // Declare the streams used
            // to encrypt to an in memory
            // array of bytes.
            MemoryStream msEncrypt = null;
            CryptoStream csEncrypt = null;
            StreamWriter swEncrypt = null;

            try
            {
                // Create the streams used for encryption.
                msEncrypt = new MemoryStream();
                csEncrypt = new CryptoStream(msEncrypt, _encryptor, CryptoStreamMode.Write);
                swEncrypt = new StreamWriter(csEncrypt);

                //Write all data to the stream.
                swEncrypt.Write(plainText);
            }
            finally
            {
                // Clean things up.

                // Close the streams.
                if (swEncrypt != null)
                    swEncrypt.Close();
                if (csEncrypt != null)
                    csEncrypt.Close();
                if (msEncrypt != null)
                    msEncrypt.Close();
            }

            // Base64 encode the encrypted byte array.
            var cipherText = Convert.ToBase64String(msEncrypt.ToArray());

            // Return encrypted string.
            return cipherText;
        }

        /// <summary>
        /// Decrypts back to plaintext.
        /// </summary>
        /// <param name="cipherText">The encrypted text.</param>
        /// <returns>AES-XXX(FromBase64(cipherText))</returns>
        public string Decrypt(string cipherText)
        {
            // Decode from Base64.
            var cipherTextBytes = Convert.FromBase64String(cipherText);

            // TDeclare the streams used
            // to decrypt to an in memory
            // array of bytes.
            MemoryStream    msDecrypt   = null;
            CryptoStream    csDecrypt   = null;
            StreamReader    srDecrypt   = null;

            // Declare the string used to hold
            // the decrypted text.
            string plaintext;

            try
            {
                // Create the streams used for decryption.
                msDecrypt = new MemoryStream(cipherTextBytes);
                csDecrypt = new CryptoStream(msDecrypt, _decryptor, CryptoStreamMode.Read);
                srDecrypt = new StreamReader(csDecrypt);

                // Read the decrypted bytes from the decrypting stream
                // and place them in a string.
                plaintext = srDecrypt.ReadToEnd();
            }
            finally
            {
                // Clean things up.

                // Close the streams.
                if (srDecrypt != null)
                    srDecrypt.Close();
                if (csDecrypt != null)
                    csDecrypt.Close();
                if (msDecrypt != null)
                    msDecrypt.Close();
            }

            return plaintext;
        }
    }
}
