﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Threading;
using System.Windows.Media.Imaging;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsMisc;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.Properties;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ICLTools.ICLExtensions;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.CloudDataModel.DirectoryDataModel
{
    // 11/8/2010
    /// <summary>
    /// The <see cref="DirectoryCloudItem"/> class holds a single instance of the <see cref="DirectoryInfo"/>.
    /// </summary>
    public class DirectoryCloudItem : IDirectoryItem, IDirectoryCloudItem
    {
        #region Fields

        // 7/4/2010 - Folder Icons
        private static bool _folderIconsLoaded;
        private static BitmapImage _folderIconOpen;
        private static BitmapImage _folderIconClosed;

        // 10/25/2010 - When set the WPF foreground color is set to yellow.
        private bool _isPasswordProtected;

        // 10/25/2010 - Add directory access error event
        public event EventHandler DirectoryAccessError;

        #endregion

        #region Properties

        // 1/10/2012
        /// <summary>
        /// Retrievies an instance of the <see cref="IMainWindow"/> interface.
        /// </summary>
        public virtual IMainWindow MainWindowParent { get; private set; }

        // 4/16/2011
        /// <summary>
        ///  Gets the parent instance for this data item.
        /// </summary>
        public IDirectoryItemViewModel Parent { get; set; }

        // 11/8/2010
        /// <summary>
        /// Gets the full path of the directory. 
        /// </summary>
        public string FullName { get; private set; }

        public string Name
        {
            get { throw new NotImplementedException(); }
        }

        public int GetDirectoriesCount
        {
            get { throw new NotImplementedException(); }
        }
        
        /// <summary>
        /// Gets the current directory name.
        /// </summary>
        public string DirectoryName { get; private set; }

        /// <summary>
        /// Gets the folder icon Open image.
        /// </summary>
        public BitmapImage FolderIconOpen
        {
            get { return _folderIconOpen; }
        }
        /// <summary>
        /// Gets the folder icon Closed image.
        /// </summary>
        public BitmapImage FolderIconClosed
        {
            get { return _folderIconClosed; }
        }

        public bool IsPasswordProtected
        {
            get { return _isPasswordProtected; }
        }

        // 9/3/2011
        /// <summary>
        /// Was this item deleted from cloud succesfully?
        /// </summary>
        /// <remarks>
        /// This is checked when the ObservableCollection is called to remove
        /// all checked items.  This guarantees that ONLY the items succesfully removed
        /// from the cloud are removed from the WPF view.
        /// </remarks>
        public bool DeletedOnCloud { get; set; }

        #endregion

        /// <summary>
        /// constructor
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="IMainWindow"/></param>
        /// <param name="parent">Parent instance of <see cref="IDirectoryItemViewModel"/></param>
        /// <param name="directoryName">Directory's name</param>
        /// <param name="directoryFullName">Gets the full path of the directory. </param>
        [CatchAllExceptionHandling(false, null, null)]
        public DirectoryCloudItem(IMainWindow mainWindow, IDirectoryItemViewModel parent, string directoryName, string directoryFullName)
        {
            DirectoryName = directoryName;
            FullName = directoryFullName;
            Parent = parent; // 4/16/2011
            MainWindowParent = mainWindow; // 1/10/2012

            // 7/4/2010 - Load Folder Icons
            if (_folderIconsLoaded) return;

            var folderOpen = new Bitmap(Resources.folderIconOpen);
            _folderIconOpen = folderOpen.ConvertToTransparentBitmapSource().ToBitmapImage();
            var folderClosed = new Bitmap(Resources.folderIconClosed);
            _folderIconClosed = folderClosed.ConvertToTransparentBitmapSource().ToBitmapImage();

            _folderIconsLoaded = true;
        }

        // 4/17/2011
        [ShowWaitCursor]
        [CatchAllExceptionHandling(false, null, null)]
        public IEnumerable<IDirectoryItem> GetCurrentDirectoryChildDirectories()
        {
            var directories = MainWindowParent.GetCloudDirectoriesForDirectory(Parent, FullName);

            // Iterate data back to caller
            foreach (var directoryCloudItem in directories)
            {
                yield return directoryCloudItem;
            }

        }

        // 8/14/2011
        [CatchAllExceptionHandling(false, null, null)]
        public List<IDirectoryItem> GetDirectories()
        {
            var directories = MainWindowParent.GetCloudDirectoriesForDirectory(Parent, FullName);

            return directories;
        }

        // 4/17/2011
        [ShowWaitCursor]
        [CatchAllExceptionHandling(false, null, null)]
        public IEnumerable<IFileItem> GetCurrentDirectoryFiles(Func<FileInfo, bool> filterBy)
        {
            var files = MainWindowParent.GetCloudFilesForDirectory(Parent, FullName);

            // Iterate data back to caller
            foreach (var fi in files)
            {
                yield return fi;
            }
        }

        // 8/14/2011
        [CatchAllExceptionHandling(false, null, null)]
        public List<IFileItem> GetFiles()
        {
            var files = MainWindowParent.GetCloudFilesForDirectory(Parent, FullName);

            return files;
        }

        // 8/14/2011
        /// <summary>
        /// Helper method used to obtain recursively the total count of all children items.
        /// </summary>
        /// <param name="cancellationToken">Instance of <see cref="CancellationToken"/>.</param>
        /// <param name="setProgressValue"></param>
        /// <returns>Total children count value.</returns>
        public int GetTotalCountOfAllChildren(CancellationToken cancellationToken, Action<double> setProgressValue)
        {
            var iterationCount = 0;
            return GetTotalCountOfAllChildren(this, cancellationToken, ref iterationCount, setProgressValue);
        }

        // 8/14/2011
        /// <summary>
        /// Helper method used to obtain recursively the total count of all children items.
        /// </summary>
        /// <param name="directoryCloudItem"></param>
        /// <param name="cancellationToken">Instance of <see cref="CancellationToken"/>.</param>
        /// <param name="iterationCount"></param>
        /// <param name="setProgressValue"></param>
        /// <returns>Total children count value.</returns>
        private static int GetTotalCountOfAllChildren(IDirectoryItem directoryCloudItem, CancellationToken cancellationToken,
                                                    ref int iterationCount, Action<double> setProgressValue)
        {
            var count = 0;
            var directoryItems = directoryCloudItem.GetDirectories();
            if (directoryItems != null)
            {
                count = directoryItems.Count;

                // iterate directories collection
                foreach (var directoryItem in directoryItems)
                {
                    // 11/15/2010 - Set ProgressBar value using Callback method
                    if (setProgressValue != null)
                        setProgressValue(iterationCount++);

                    // 12/3/2010 - Check if Task cancel request.
                    if (cancellationToken.IsCancellationRequested)
                    {
                        try
                        {
                            cancellationToken.ThrowIfCancellationRequested();
                        }
                        catch (OperationCanceledException)
                        {
                            // Empty
                        }

                        return 0;
                    }

                    if (directoryItem == null) continue;

                    count += GetTotalCountOfAllChildren(directoryItem, cancellationToken, ref iterationCount, setProgressValue);
                }

            }

            // Get Files count.
            var fileItems = directoryCloudItem.GetFiles();
            if (fileItems != null) count += fileItems.Count;

            return count;
        }
    }
}