﻿using System;
using System.IO;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.CloudDataModel.DirectoryDataModel
{
    // 7/2/2010
    public class FileCloudItem : IFileItem
    {
        #region Fields

        // 11/8/2010
        private readonly string _fileName;
        private readonly string _fileExtension;
        private readonly string _fullPathName;
        private readonly string _directoryName;
        private readonly long _fileSize;
        private readonly string _fileLastModified;

        #endregion

        #region Properties

        #region IFileItem Properties

        // 4/17/2011
        /// <summary>
        /// This returns a null for the cloud service.
        /// </summary>
        public FileInfo FileInfo
        {
            get { return null; }
        }

        /// <summary>
        /// Gets the current file name.
        /// </summary>
        public string FileName { get { return _fileName; } }

        // 11/1/2010
        /// <summary>
        /// Gets the current file name with extension.
        /// </summary>
        public string FileNameWithExtension
        {
            get
            {
                return FileName + FileExtension;
            }
        }

        /// <summary>
        /// Gets the full path name for given file; the original restore path.
        /// </summary>
        public string FullPathName { get { return _fullPathName; } }

        // 11/1/2010
        /// <summary>
        /// Gets the current file size.
        /// </summary>
        public long FileSize
        {
            get
            {
                return _fileSize;
            }
        }

        // 11/9/2010
        /// <summary>
        /// Gets the current unique Key used in cloud.
        /// </summary>
        public string CloudKey { get; private set; }

        /// <summary>
        /// Gets the file's extension.
        /// </summary>
        public string FileExtension { get { return _fileExtension; } }

        /// <summary>
        /// Gets the directory's name.
        /// </summary>
        public string DirectoryName { get { return _directoryName; } }

        /// <summary>
        /// Gets the current file's <see cref="DateTime"/> it was last modified.
        /// </summary>
        public DateTime FileLastModified
        {
            get
            {
                return string.IsNullOrEmpty(_fileLastModified) ? new DateTime() : Convert.ToDateTime(_fileLastModified);
            }
        }

        // 5/14/2011
        /// <summary>
        /// Was this item deleted from cloud succesfully?
        /// </summary>
        /// <remarks>
        /// This is checked when the ObservableCollection is called to remove
        /// all checked items.  This guarantees that ONLY the items succesfully removed
        /// from the cloud are removed from the WPF view.
        /// </remarks>
        public bool DeletedOnCloud { get; set; }

        // 9/3/2011
        /// <summary>
        /// Gets or sets the current parent <see cref="IFileItemViewModel"/> instance.
        /// </summary>
        public IFileItemViewModel Parent { get; set; }

        #endregion
        
        #endregion

        /// <summary>
        /// constructor
        /// </summary>
        /// <param name="cloudKey">Unique key used in cloud storage.</param>
        /// <param name="fileName">File's name.</param>
        /// <param name="fullPathName">File's full path name.</param>
        /// <param name="directoryName">The directory's full path.</param>
        /// <param name="fileExtension">File's extension.</param>
        /// <param name="fileSize">The current file size.</param>
        /// <param name="fileLastModified">The file's last modified date string.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public FileCloudItem(string cloudKey, string fileName, string fullPathName, string directoryName, 
                             string fileExtension, long fileSize, string fileLastModified)
        {
            // store data
            CloudKey = cloudKey; // 11/9/2010
            _fileName = fileName;
            _fileExtension = fileExtension;
            _fullPathName = fullPathName;
            _directoryName = directoryName;
            _fileSize = fileSize;
            _fileLastModified = fileLastModified;
        }
    }
}