﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Security;
using System.Threading;
using System.Windows.Media.Imaging;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.Properties;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.ViewModel.DirectoryViewModel;
using ImageNexus.BenScharbach.ICLTools.ICLExtensions;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.DesktopDataModel.DirectoryDataModel
{
    // 7/2/2010
    /// <summary>
    /// The <see cref="DirectoryInfoItem"/> class holds a single instance of the <see cref="DirectoryInfo"/>.
    /// </summary>
    public class DirectoryInfoItem : IDirectoryItem
    {
        #region Fields

        private readonly DirectoryInfo _directoryInfoI;

        // 7/4/2010 - Folder Icons
        private static bool _folderIconsLoaded;
        private static BitmapImage _folderIconOpen;
        private static BitmapImage _folderIconClosed;

        // 10/25/2010 - When set the WPF foreground color is set to yellow.
        private bool _isPasswordProtected;
       
        // 10/25/2010 - Add directory access error event
        public event EventHandler DirectoryAccessError;

        #endregion
        
        #region Properties

        // 4/16/2011
        /// <summary>
        ///  Gets the parent instance for this data item.
        /// </summary>
        public IDirectoryItemViewModel Parent { get; set; }

        // 4/16/2011
        /// <summary>
        /// Gets the full path of the directory.
        /// </summary>
        public string FullName
        {
            get {  return (_directoryInfoI != null) ? _directoryInfoI.FullName : string.Empty; }
        }

        // 4/16/2011
        /// <summary>
        /// Gets the name of this directory instance.
        /// </summary>
        public string Name
        {
            get { return (_directoryInfoI != null) ? _directoryInfoI.Name : string.Empty; }
        }

        // 4/16/2011
        /// <summary>
        /// Returns the current instance sub-directory count.
        /// </summary>
        public int GetDirectoriesCount
        {
            get { return (_directoryInfoI != null) ? _directoryInfoI.GetDirectories().Length : 0; }
        }

        /// <summary>
        /// Gets the current directory name.
        /// </summary>
        public string DirectoryName { get; private set; }
        
        /// <summary>
        /// Gets the folder icon Open image.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(BitmapImage), null)]
        public BitmapImage FolderIconOpen
        {
            get
            {
                // 11/16/2010 Check if null
                if (_folderIconOpen == null)
                    CreateFileBitmapIcon(Resources.folder_documents_icon, out _folderIconOpen);

                return _folderIconOpen;
            }
        }
        
        /// <summary>
        /// Gets the folder icon Closed image.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(BitmapImage), null)]
        public BitmapImage FolderIconClosed
        {
            get
            {
                // 11/16/2010 Check if null
                if (_folderIconOpen == null)
                    CreateFileBitmapIcon(Resources.folder_documents_icon, out _folderIconClosed);

                return _folderIconClosed;
            }
        }

        // 4/16/2011
        /// <summary>
        /// Is directory password protected?
        /// </summary>
        public bool IsPasswordProtected
        {
            get { return _isPasswordProtected; }
        }

        #endregion

        /// <summary>
        /// constructor
        /// </summary>
        /// <param name="directoryInfo">Instance of <see cref="DirectoryInfo"/></param>
        [CatchAllExceptionHandling(false, null, null)]
        public DirectoryInfoItem(DirectoryInfo directoryInfo)
        {
            _directoryInfoI = directoryInfo;
            DirectoryName = directoryInfo.Name;

        }

        // 4/16/2011: Moved from DirectoryInfoViewModel
        // 11/2/2010; 11/14/2010 - Updated to communicate Progress info using Callbacks.
        /// <summary>
        /// Returns a enumerable collection for the <see cref="DirectoryInfoViewModel"/> instances.
        /// </summary>
        /// <returns>Instance of <see cref="IEnumerable{DirectoryInfoViewModel}"/>.</returns>
        [CatchAllExceptionHandling(true, typeof(IEnumerable<object>), null)]
        public IEnumerable<IDirectoryItem> GetCurrentDirectoryChildDirectories()
        {
            var directoryInfos = GetSystemDirectories(); // 11/1/2010 - Cache data
            
            foreach (var directoryInfo in directoryInfos)
            {
                yield return new DirectoryInfoItem(directoryInfo);

            }

        }

        // 8/14/2011
        [CatchAllExceptionHandling(false, null, null)]
        public List<IDirectoryItem> GetDirectories()
        {
            var directoriesList = new List<IDirectoryItem>();

            foreach (var currentDirectoryChildDirectory in GetCurrentDirectoryChildDirectories())
            {
                if (currentDirectoryChildDirectory == null) continue;
                directoriesList.Add(currentDirectoryChildDirectory);
            }

            return directoriesList;
        }

        // 4/16/2011: Moved from DirectoryInfoViewModel
        // 11/2/2010
        /// <summary>
        /// Returns a enumerable collection for the <see cref="FileInfoViewModel"/> instances.
        /// </summary>
        /// <returns>Instance of <see cref="IEnumerable{FileInfoViewModel}"/>.</returns>
        [CatchAllExceptionHandling(true, typeof(IEnumerable<object>), null)]
        public IEnumerable<IFileItem> GetCurrentDirectoryFiles(Func<FileInfo, bool> filterBy)
        {
            var fileInfos = _directoryInfoI.GetFiles(); // 11/1/2010 - Cache data
            
            foreach (var fi in fileInfos)
            {
                // Only add files with the '.x' or '.fbx' extensions.
                //_filterBy = fi => ((fi.Extension != ".X" && fi.Extension != ".FBX") && fi.Extension != ".x") && fi.Extension != ".fbx";
                if (filterBy != null && !filterBy(fi)) continue;

                yield return new FileInfoItem(fi);

            }

        }

        // 8/14/2011
        [CatchAllExceptionHandling(false, null, null)]
        public List<IFileItem> GetFiles()
        {
            var filesList = new List<IFileItem>();

            foreach (var currentDirectoryFile in GetCurrentDirectoryFiles(null))
            {
                if (currentDirectoryFile == null) continue;
                filesList.Add(currentDirectoryFile);
            }

            return filesList;
        }
        
        // 11/30/2010; 12/3/2010 - Updated to include new 'CancellationToken' param.
        /// <summary>
        /// Helper method used to obtain recursively the total count of all children items.
        /// </summary>
        /// <param name="cancellationToken">Instance of <see cref="CancellationToken"/>.</param>
        /// <param name="setProgressValue"></param>
        /// <returns>Total children count value.</returns>
        [CatchAllExceptionHandling(true, typeof(int), 0)]
        public int GetTotalCountOfAllChildren(CancellationToken cancellationToken, Action<double> setProgressValue)
        {
            var iterationCount = 0;
            /*var timer = new Timer(delegate(object state)
            {
                // 11/15/2010 - Set ProgressBar value using Callback method
                if (setProgressValue != null)
                    setProgressValue(iterationCount);

                // Add Small pause, to let UI update.
                Thread.Sleep(10);

            }, null, 0, 500);   */  

            
            return GetTotalCountOfAllChildren(this, cancellationToken, ref iterationCount, setProgressValue, _directoryInfoI);
        }
        
        // 8/7/2011: Updated to static method.
        // 4/16/2011: Moved from DirectoryInfoViewModel
        // 11/30/2010; 12/3/2010 - Updated to include new 'CancellationToken' param.
        /// <summary>
        /// Helper method used to obtain recursively the total count of all children items.
        /// </summary>
        /// <param name="directoryInfoItem"></param>
        /// <param name="cancellationToken">Instance of <see cref="CancellationToken"/>.</param>
        /// <param name="iterationCount"></param>
        /// <param name="setProgressValue"></param>
        /// <param name="directoryItem">Base starting point instance of <see cref="DirectoryInfo"/>.</param>
        /// <returns>Total children count value.</returns>
        private static int GetTotalCountOfAllChildren(DirectoryInfoItem directoryInfoItem, CancellationToken cancellationToken, 
                                                    ref int iterationCount, Action<double> setProgressValue, DirectoryInfo directoryItem)
        {
            //var counter = iterationCount;
            var count = 0;
            if (GetDirectories(directoryInfoItem, directoryItem) != null)
            {
                count = GetDirectories(directoryInfoItem, directoryItem).Length;

                // iterate directories collection
                foreach (var directoryInfo in GetDirectories(directoryInfoItem, directoryItem))
                {
                    // 11/15/2010 - Set ProgressBar value using Callback method
                    if (setProgressValue != null)
                        setProgressValue(iterationCount++);

                    // 12/3/2010 - Check if Task cancel request.
                    if (cancellationToken.IsCancellationRequested)
                    {
                        try
                        {
                            cancellationToken.ThrowIfCancellationRequested();
                        }
                        catch (OperationCanceledException)
                        {
                            // Empty
                        }

                        return 0;
                    }  
                   
                    if (directoryInfo == null) continue;

                    count += GetTotalCountOfAllChildren(directoryInfoItem, cancellationToken, ref iterationCount, setProgressValue, directoryInfo);
                }

            }

            // Get Files count.
            var filesCount = GetFilesCount(directoryItem);
            count += filesCount != null ? filesCount.Value : 0;

            // Set value back into ref var.
            //iterationCount = counter;

            return count;
        }

        #region MiscMethods

        // 10/25/2010
        /// <summary>
        /// Triggers the DirectoryAccess error event.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void OnDirectoryAccessError()
        {
            if (DirectoryAccessError != null)
                DirectoryAccessError(null, EventArgs.Empty);
        }

        // 11/16/2010
        /// <summary>
        /// Helper method used to create the file icon.
        /// </summary>
        /// <param name="bitmapResourceToUse">Resource constant name to use</param>
        /// <param name="bitmapSource">(OUT) Instance of <see cref="BitmapImage"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        private static void CreateFileBitmapIcon(Bitmap bitmapResourceToUse, out BitmapImage bitmapSource)
        {
            var folderOpen = new Bitmap(bitmapResourceToUse);
            // 11/16/2010: Updated to BitmapSource, rather than BitmapImage to get Transparency to work.
            bitmapSource = folderOpen.ConvertToTransparentBitmapSource().ToBitmapImage(); // 11/16/2010 - Transparent.
        }

        /// <summary>
        /// Returns an array of <see cref="DirectoryInfo"/>.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(DirectoryInfo[]), null)]
        private DirectoryInfo[] GetSystemDirectories()
        {
            try // 10/25/2010 - Add Try/Catch
            {
                _isPasswordProtected = false;
                return _directoryInfoI.GetDirectories();
            }
            catch (UnauthorizedAccessException)
            {
                _isPasswordProtected = true;
                OnDirectoryAccessError();
            }
            catch (SecurityException)
            {
                _isPasswordProtected = true;
                OnDirectoryAccessError();
            }
            catch (IOException)
            {
                OnDirectoryAccessError();
            }

            return new DirectoryInfo[0];
        }

        // 8/7/2011
        /// <summary>
        /// Returns an array of <see cref="DirectoryInfo"/>.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(DirectoryInfo[]), null)]
        private static DirectoryInfo[] GetDirectories(DirectoryInfoItem directoryInfoItem, DirectoryInfo directoryInfo)
        {
            try
            {
                return directoryInfo.GetDirectories();
            }
            catch (UnauthorizedAccessException)
            {
                directoryInfoItem.OnDirectoryAccessError();
            }
            catch (SecurityException)
            {
                directoryInfoItem.OnDirectoryAccessError();
            }
            catch (IOException)
            {
                directoryInfoItem.OnDirectoryAccessError();
            }

            return new DirectoryInfo[0];
        }

        // 11/30/2010
        /// <summary>
        /// Helper method to obtain the current files count for the given <paramref name="directoryInfoItem"/>.
        /// </summary>
        /// <param name="directoryInfoItem">Instance of <see cref="DirectoryInfo"/>.</param>
        /// <returns>Total count value for files.</returns>
        [DirectoryInfoExceptionHandling]
        [CatchAllExceptionHandling(true, typeof(int?), null)]
        private static int? GetFilesCount(DirectoryInfo directoryInfoItem)
        {
            return directoryInfoItem.GetFiles().Length;
        }

        #endregion
    }
}