﻿using System;
using System.IO;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.DesktopDataModel.DirectoryDataModel
{
    /// <summary>
    /// Implements the <see cref="IFileItem"/> interface.
    /// </summary>
    public class FileInfoItem : IFileItem
    {
        // 7/7/2010 - Store FileInfo instance.
        private readonly FileInfo _fileInfo;

        // 11/8/2010
        private readonly string _fileName;
        private readonly string _fileExtension;
        private readonly string _fullPathName;
        private readonly string _directoryName;
        private readonly long _fileSize;
        private readonly string _fileLastModified;

        #region Properties

        // 7/7/2010
        /// <summary>
        /// Gets the current <see cref="FileInfo"/>
        /// </summary>
        public FileInfo FileInfo
        {
            get { return _fileInfo; }
        }

        /// <summary>
        /// Gets the current file name.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(string), "")]
        public string FileName
        {
            get
            {
                // Remove Extension from name
                var fileName = Path.GetFileNameWithoutExtension((FileInfo != null) ? FileInfo.FullName : string.Empty);
                return fileName;
            }
        }

        // 11/1/2010
        /// <summary>
        /// Gets the current file name with extension.
        /// </summary>
        public string FileNameWithExtension
        {
            get
            {
                if (_fileInfo != null) return FileName + _fileInfo.Extension;

                return string.Empty;
            }
        }

        /// <summary>
        /// Gets the full path name for given file.
        /// </summary>
        /// <remarks>
        /// Simply an alias for the internal <see cref="FileSystemInfo.FullName"/> property.
        /// </remarks>
        [CatchAllExceptionHandling(true, typeof(string), "")]
        public string FullPathName
        {
            get
            {
                return FileInfo != null ? FileInfo.FullName : string.Empty;
            }
        }

        // 11/1/2010
        /// <summary>
        /// Gets the current file size.
        /// </summary>
        public long FileSize
        {
            get
            {
                return _fileInfo != null ? _fileInfo.Length : 0;
            }
        }

        // 11/9/2010
        /// <summary>
        /// Gets the current unique Key used in cloud.
        /// </summary>
        public string CloudKey { get; private set; }

        /// <summary>
        /// Gets the file's extension.
        /// </summary>
        public string FileExtension { get { return _fileExtension; } }

        /// <summary>
        /// Gets the directory's name.
        /// </summary>
        public string DirectoryName { get { return _directoryName; } }
        
        /// <summary>
        /// Gets the current file's <see cref="DateTime"/> it was last modified.
        /// </summary>
        public DateTime FileLastModified
        {
            get
            {
                return _fileInfo != null ? _fileInfo.LastWriteTime : DateTime.Now;
            }
        }

        // 5/14/2011
        /// <summary>
        /// Was this item deleted from cloud succesfully?
        /// </summary>
        /// <remarks>
        /// This is checked when the ObservableCollection is called to remove
        /// all checked items.  This guarantees that ONLY the items succesfully removed
        /// from the cloud are removed from the WPF view.
        /// </remarks>
        public bool DeletedOnCloud { get; set; }

        // 9/3/2011
        /// <summary>
        /// Gets or sets the current parent <see cref="IFileItemViewModel"/> instance.
        /// </summary>
        public IFileItemViewModel Parent { get; set; }

        #endregion

        /// <summary>
        /// constructor
        /// </summary>
        /// <param name="fileInfo">Instance of <see cref="FileInfo"/></param>
        [CatchAllExceptionHandling(false, null, null)]
        public FileInfoItem(FileInfo fileInfo)
        {
            // store instance
            _fileInfo = fileInfo;
           
        }
    }
}