using System;
using System.Collections.Generic;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.DesktopDataModel.CheckedDataModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.BaseClasses;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.Enums;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.CheckedDataModel;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel
{
    // 1/9/2012
    /// <summary>
    /// Base class for the DirectoryTreeViewModel Adapter
    /// </summary>
    public class DirectoryTreeViewModelAdapterBase : ViewModelBase, ICheckedItems
    {
        /// <summary>
        /// CheckedCollection's unique key.
        /// </summary>
        private readonly int _checkedCollectionUniqueKeyS;

        #region Properties

        /// <summary>
        /// Gets if checked collection has been updated by user.
        /// </summary>
        public bool IsDirty
        {
            get
            {
                // get singleton instance of CheckedCollection
                var checkedCollection = CheckedCollection<IFileItem, IDirectoryItem>.GetInstance(_checkedCollectionUniqueKeyS);

                return checkedCollection != null && checkedCollection.IsDirty;
            }
        }

        /// <summary>
        /// CheckedCollection's unique key.
        /// </summary>
        public int CheckedCollectionUniqueKey
        {
            get { return _checkedCollectionUniqueKeyS; }
        }

        #endregion

        #region Events

        // 1/12/2012
        /// <summary>
        /// Event used to reset all directories checkedItems to false.
        /// </summary>
        public event Action ResetDirectoryCheckedItems;

        #endregion

        /// <summary>
        /// Base constructor which initializes the unique key.
        /// </summary>
        /// <param name="mainWindow"> </param>
        /// <param name="viewModelType"> </param>
        /// <param name="checkedCollectionUniqueKeyS">Enter the unique key for this collection.</param>
        public DirectoryTreeViewModelAdapterBase(IMainWindow mainWindow, ViewModelTypeEnum viewModelType, int checkedCollectionUniqueKeyS)
            : base(mainWindow, viewModelType)
        {
            _checkedCollectionUniqueKeyS = checkedCollectionUniqueKeyS;
        }

       
        /// <summary>
        /// Clears all internal collections within the <see cref="CheckedCollection{T,TU}"/>.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public void ClearAllCheckedItems()
        {
            // get singleton instance of CheckedCollection
            var checkedCollection = CheckedCollection<IFileItem, IDirectoryItem>.GetInstance(_checkedCollectionUniqueKeyS);

            // Clear all internal collections.
            if (checkedCollection != null) checkedCollection.ClearAllCheckedItems();
        }

        /// <summary>
        /// Retrieves a collection of <see cref="FileCheckedItem"/> instances which have the
        /// 'IsChecked' flag set to true.  
        /// </summary>
        /// <param name="checkedItems">(OUT) Collection of files set with 'IsChecked' flag to true.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void GetListOfCheckedItems(out List<IFileCheckedItem<IFileItem>> checkedItems)
        {
            // Set Default
            checkedItems = default(List<IFileCheckedItem<IFileItem>>);

            // get singleton instance of CheckedCollection
            var checkedCollection = CheckedCollection<IFileItem, IDirectoryItem>.GetInstance(_checkedCollectionUniqueKeyS);

            // get 'Checked' items
            if (checkedCollection != null) checkedCollection.GetCheckedItems(out checkedItems);
        }

        /// <summary>
        /// Retrieves a collection of <see cref="DirectoryCheckedItem"/> instances which have the
        /// 'IsChecked' flag set to true.  
        /// </summary>
        /// <param name="checkedItems">(OUT) Collection of directories set with 'IsChecked' flag to true.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void GetListOfCheckedItems(out List<IDirectoryCheckedItem<IDirectoryItem>> checkedItems)
        {
            // Set Default
            checkedItems = default(List<IDirectoryCheckedItem<IDirectoryItem>>);

            // get singleton instance of CheckedCollection
            var checkedCollection = CheckedCollection<IFileItem, IDirectoryItem>.GetInstance(_checkedCollectionUniqueKeyS);

            // get 'Checked' items
            if (checkedCollection != null) checkedCollection.GetCheckedItems(out checkedItems);
        }

        /// <summary>
        /// Clears the internal collection and updates with given <paramref name="checkedItems"/>
        /// collection.
        /// </summary>
        /// <param name="checkedItems">Collection of files which are marked as 'IsChecked'.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetCheckedItems(List<IFileCheckedItem<IFileItem>> checkedItems)
        {
            // get singleton instance of CheckedCollection
            var checkedCollection = CheckedCollection<IFileItem, IDirectoryItem>.GetInstance(_checkedCollectionUniqueKeyS);

            // Set 'Checked' items
            if (checkedCollection != null) checkedCollection.SetCheckedItems(checkedItems);
        }

        /// <summary>
        /// Clears the internal collection and updates with given <paramref name="checkedItems"/>
        /// collection.
        /// </summary>
        /// <param name="checkedItems">Collection of directories which are marked as 'IsChecked'.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetCheckedItems(List<IDirectoryCheckedItem<IDirectoryItem>> checkedItems)
        {
            // get singleton instance of CheckedCollection
            var checkedCollection = CheckedCollection<IFileItem, IDirectoryItem>.GetInstance(_checkedCollectionUniqueKeyS);

            // Set 'Checked' items
            if (checkedCollection != null) checkedCollection.SetCheckedItems(checkedItems);
        }

        // 1/12/2012
        /// <summary>
        /// Triggers the Event handler for <see cref="ResetDirectoryCheckedItems"/>.
        /// </summary>
        public void OnResetDirectoryCheckedItems()
        {
            if (ResetDirectoryCheckedItems != null)
            {
                ResetDirectoryCheckedItems();
            }
        }
    }
}