﻿using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsMisc;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.DesktopDataModel.CheckedDataModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.Enums;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.BaseClasses;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.Enums;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ICLCommon.TreeViewInterface;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.CheckedDataModel;
using Microsoft.Practices.Unity;

// reflection namespace

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.ViewModel.DirectoryViewModel
{
    // 7/2/2010
    /// <summary>
    /// The <see cref="DirectoryInfoViewModel"/> inherits from <see cref="DirectoryViewModelBase.BaseClasses.TreeViewItemViewModel"/>, which represents
    /// a single Directory in the treeView.
    /// </summary>
    public sealed class DirectoryInfoViewModel : TreeViewItemViewModel, IDirectoryItemViewModel
    {
        #region Fields

        // 4/16/2011 - Updated to interface.
        private readonly IDirectoryItem _directoryItem;

        // 10/26/2010 - CheckedItem
        private readonly DirectoryCheckedItem _directoryCheckedItem;

        // 7/6/2010 - Filter used for base children selection.
        private readonly Func<FileInfo, bool> _filterBy;

        // 7/7/2010 - Store TreeView's Directory path.
        internal string TreeViewsDirectoryPath;

        // Store ILockUnlockUI interface
        private static ILockUnlockUI _lockUnlockUiInterface;

        // 3/2/2012 - Master Cancellation tokensource
        private static CancellationTokenSource _tokenSource;

        // NOTE: Left as Null intentionally to know if WPF called 'Get' before 'Children' collection populated.
        // 11/1/2010 - Observable collection for ListView
        private ObservableCollection<IFileItemViewModel> _files;

        // NOTE: Left as Null intentionally to know if WPF called 'Get' before 'Children' collection populated.
        // 11/1/2010 - Observable collection for ListView
        private ObservableCollection<IDirectoryItemViewModel> _directories;
        
        // 1/10/2012
        private readonly ICheckedItems _checkedItems;

        // 3/2/2012 - Set on instance that Checked operation started at.
        private bool _startCheckedOperation;

        #endregion

        #region Events

        // 1/12/2012
        /// <summary>
        /// Occurs when the directory's 'IsChecked' value is updated.
        /// </summary>
        public event EventHandler DirectoryCheckedItemUpdated;

        // 1/12/2012
        /// <summary>
        /// Occurs when the schedule number value is updated.
        /// </summary>
        public event EventHandler ScheduleNumberUpdated;

        // 8/15/2011
        /// <summary>
        /// Occurs at the start of the populate directories calls.
        /// </summary>
        public static event EventHandler StartPopluateDirectories;

        // 8/15/2011
        /// <summary>
        /// Occurs at the end of the populate directories calls.
        /// </summary>
        public static event EventHandler EndPopulateDirectories;

        // 8/15/2011
        /// <summary>
        /// Occurs at the start of the populate files calls.
        /// </summary>
        public static event EventHandler StartPopulateFiles;

        // 8/15/2011
        /// <summary>
        /// Occurs at the end of the populate files calls.
        /// </summary>
        public static event EventHandler EndPopulateFiles;

        // 8/15/2011
        /// <summary>
        /// Occurs at the start of the update property values calls.
        /// </summary>
        public static event EventHandler StartUpdatePropertyValues;

        // 8/15/2011
        /// <summary>
        /// Occurs at the end of the update property values calls.
        /// </summary>
        public static event EventHandler EndUpdatePropertyValues;

        #endregion

        #region Properties

        // 10/23/2010
        /// <summary>
        /// Gets/sets whether the TreeViewItem 
        /// associated with this object is checked.
        /// </summary>
        public override bool IsChecked
        {
            get
            {
                return _directoryCheckedItem != null && _directoryCheckedItem.IsChecked;
            }
            set
            {
                if (_directoryCheckedItem != null)
                {
                    _directoryCheckedItem.IsChecked = value;
                }

                base.IsChecked = value;
                
                // 3/1/2012 - Create Linked TokenSource. This allows for multiple linked copied of the original TokenSource, thereby
                //            allowing the original to be nullified, but each linked copy will still funtion.
                CancellationTokenSource tokenSource = null;
                if (_tokenSource != null)
                {
                    tokenSource = CancellationTokenSource.CreateLinkedTokenSource(_tokenSource.Token);
                }

                // 1/12/2012 - Update all children directories
                foreach (var directoryItemViewModel in DirectoriesOnly)
                {
                    // 3/1/2012 - Check if Task cancel param1.
                    if (tokenSource != null && tokenSource.Token.IsCancellationRequested)
                    {
                        try
                        {
                            tokenSource.Token.ThrowIfCancellationRequested();
                        }
                        catch (OperationCanceledException)
                        {
                        }

                        return;
                    }

                    directoryItemViewModel.IsChecked = value;
                }

                // 1/12/2012 - Check if files need loading
                if (_files == null)
                {
                    _files = DoPopulateFiles();
                }

                // 1/12/2012 - Trigger event to update files
                OnDirectoryCheckedItemUpdated();

                // (Recurse up the tree updates)
                // 12/9/2010 - Update Parent's Indeterminate state.
                //if (Parent != null) ((DirectoryInfoViewModel)Parent).UpdateAllChildrenCheckedProperty();

                // 3/2/2012 - Check if instance that started Checked operation; this correctly will null our the original tokenSource.
                if (_startCheckedOperation)
                {
                    // Null original static copy
                    _tokenSource = null;
                    _startCheckedOperation = false;
                }
            }
        }

        // 3/2/2012
        /// <summary>
        /// Gets/sets whether the TreeViewItem 
        /// associated with this object is selected.
        /// </summary>
        public override bool IsSelected
        {
            get
            {
                return base.IsSelected;
            }
            set
            {
                base.IsSelected = value;
            }
        }

        // 12/8/2010
        /// <summary>
        /// Gets/sets whether the TreeViewItem 
        /// associated with this object has a schedule number.
        /// </summary>
        public override int ScheduleNumber
        {
            get
            {
                return _directoryCheckedItem != null ? _directoryCheckedItem.ScheduleNumber : 0;
            }
            set
            {
                if (_directoryCheckedItem != null && value != _directoryCheckedItem.ScheduleNumber)
                {
                    // Set into CheckedItem
                    _directoryCheckedItem.ScheduleNumber = value;
                    OnPropertyChanged("ScheduleNumber");
                }

                // 1/12/2012 - Update all children directories
                foreach (var directoryItemViewModel in DirectoriesOnly)
                {
                    directoryItemViewModel.ScheduleNumber = value;
                }

                // 1/12/2012 - Check if files need loading
                if (_files == null)
                {
                    _files = DoPopulateFiles();
                }

                // 1/12/2012 - Trigger event to update files
                OnScheduleNumberUpdated();
            }
        }

        // 12/9/2010
        /// <summary>
        /// Gets or sets if this directory is in indeterminate mode - 
        /// In other words, some children items are checked.
        /// </summary>
        /// <remarks>
        /// This is used in the UI to let the end-user know when some
        /// directory has items checked.
        /// </remarks>
        public bool IsIndeterminate
        {
            get
            {
                return _directoryCheckedItem != null && _directoryCheckedItem.IsIndeterminate;
            }
            set
            {
                if (_directoryCheckedItem != null && value != _directoryCheckedItem.IsIndeterminate)
                {
                    // Set into CheckedItem
                    _directoryCheckedItem.IsIndeterminate = value;
                    OnPropertyChanged("IsIndeterminate");
                }
            }
        }

        // 10/26/2010
        /// <summary>
        /// Gets/sets whether the TreeViewItem 'CheckBox'
        /// associated with this object has the given foreground color.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(string), "")]
        public override string CheckBoxBackground
        {
            get
            {
                // 1/9/2012 - Retrieve interface CheckedItems.
                var checkedItems = MainWindowParent.UnityContainer.Resolve<DirectoryTreeViewModelAdapterBase>
                    (ViewModelType.ToString());
                var checkedCollection = GetCheckedCollection(checkedItems.CheckedCollectionUniqueKey);

                // 10/26/2010 - 1st get total Checked count.
                var filesCount = checkedCollection.TotalCountForFilesInDirectory(DirectoryItem.FullName);
                var directoriesCount = checkedCollection.TotalCountForDirectoriesInDirectory(DirectoryItem.FullName);

                // 10/26/2010 - Now check if in inderminate mode.
                if (filesCount == 0 && directoriesCount == 0)
                    CheckBoxBackground = "Yellow";
                else
                    CheckBoxBackground = "Red";


                return base.CheckBoxBackground;
            }
            set
            {
                base.CheckBoxBackground = value;
            }
        }

        /// <summary>
        /// Gets the current directory name.
        /// </summary>
        public string DirectoryName
        {
            get
            {
                return DirectoryItem != null ? DirectoryItem.DirectoryName : string.Empty;
            }
        }

        // 7/4/2010; // 11/16/2010: Updated to BitmapSource, rather than BitmapImage to get Transparency to work.
        /// <summary>
        /// Gets the current directory image.
        /// </summary>
        public BitmapImage DirectoryImage
        {
            get
            {
                return DirectoryItem != null ? DirectoryItem.FolderIconOpen : null;
            }
        }

        // 11/1/2010
        /// <summary>
        /// Returns an observable <see cref="FileInfoViewModel"/> collection.
        /// </summary>
        public ObservableCollection<IFileItemViewModel> Files
        {
            get
            {
                OnStartPopulateFiles(); // 8/15/2011

                // 11/2/2010 - check if null
                if (_files == null)
                    // 11/30/2010 - Refactored to method.
                    _files = DoPopulateFiles();

                OnEndPopulateFiles(); // 8/15/2011

                return _files;
            }
        }

        // 11/2/2010
        /// <summary>
        /// Returns an observable <see cref="DirectoryInfoViewModel"/> collection.
        /// </summary>
        public ObservableCollection<IDirectoryItemViewModel> DirectoriesOnly
        {
            get
            {
                OnStartPopluateDirectories(); // 8/15/2011

                // 11/2/2010 - check if null
                if (_directories == null)
                    // 11/30/2010 - Refactored to method.
                    _directories = DoPopulateDirectories();

                OnEndPopulateDirectories(); // 8/15/2011

                return _directories;
            }
        }

        // 11/30/2010
        /// <summary>
        /// Gets the current <see cref="IDirectoryItem"/>.
        /// </summary>
        public IDirectoryItem DirectoryItem
        {
            get { return _directoryItem; }
        } 

        #endregion

        // 12/14/2010: Updated the Constructor now have the new 'IMainWindowTreeView' interface.
        /// <summary>
        /// constructor
        /// </summary>
        /// <param name="mainWindow"> </param>
        /// <param name="viewModelEnum"> </param>
        /// <param name="treeViewUserControl">Instance of the WPF <see cref="TreeViewUserControl"/>.</param>
        /// <param name="directoryItem">Instance of <see cref="IDirectoryItem"/></param>
        /// <param name="parentDirectory">Instance of parent <see cref="DirectoryInfoViewModel"/></param>
        /// <param name="filterBy">Lambda function as filterBy for base children files</param>
        [CatchAllExceptionHandling(false, null, null)]
        public DirectoryInfoViewModel(IMainWindow mainWindow, ViewModelTypeEnum viewModelEnum, TreeViewUserControl treeViewUserControl, IDirectoryItem directoryItem, DirectoryInfoViewModel parentDirectory, Func<FileInfo, bool> filterBy)
            : base(mainWindow, viewModelEnum, treeViewUserControl, parentDirectory, true)
        {
            // Note: Seems to be used only on the Cloud calls.
            // 4/16/2011 - Set Parent
            directoryItem.Parent = (IDirectoryItemViewModel) this;

            // 1/9/2012 - Retrieve interface CheckedItems.
            var directoryTreeViewModelAdapterBase = MainWindowParent.UnityContainer.Resolve<DirectoryTreeViewModelAdapterBase>(ViewModelType.ToString());
            _checkedItems = (ICheckedItems)directoryTreeViewModelAdapterBase;

            _directoryItem = directoryItem;
            _lockUnlockUiInterface = (ILockUnlockUI)mainWindow; // 12/14/2010

            // 12/9/2010 - Reconnect checked instances if exist; otherwise, create new instance.
            ReconnectCheckedItems(directoryItem, out _directoryCheckedItem);

            // 10/25/2010 - Subscribe to the DirectoryAccess error event.
            DirectoryItem.DirectoryAccessError += _directoryInfoItem_DirectoryAccessError;
            
            // 12/3/2010 - Subscribe to the Directory Checkbox Click event.
            treeViewUserControl.DirectoryCheckboxClicked += treeViewUserControl_DirectoryCheckboxClicked;

            // 5/15/2011 - Subscribe to the Refresh event
            treeViewUserControl.DirectoryRefreshClicked += treeViewUserControl_DirectoryRefreshClicked;

            // 1/12/2012 - Subscribe to the reset directory checked items event.
            directoryTreeViewModelAdapterBase.ResetDirectoryCheckedItems += directoryTreeViewModelAdapterBase_ResetDirectoryCheckedItems;

            // 7/6/2010 - Set FilterBy lambda func.
            _filterBy = filterBy;
            
            // 7/7/2010 - Set TreeView's directory path; this is extended when children added.
            TreeViewsDirectoryPath = (parentDirectory != null)
                                          ? parentDirectory.TreeViewsDirectoryPath + directoryItem.DirectoryName +
                                            @"\"
                                          : directoryItem.DirectoryName + @"\";
            
        }

        // 1/12/2012
        /// <summary>
        /// Occurs when the directory checkedItems are required to be reset.
        /// </summary>
        private void directoryTreeViewModelAdapterBase_ResetDirectoryCheckedItems()
        {
            IsChecked = false;
        }

        // 12/3/2010
        /// <summary>
        /// Eventhandler for the Directory CheckBox Clicked event.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void treeViewUserControl_DirectoryCheckboxClicked(object sender, RoutedEventArgs e)
// ReSharper restore InconsistentNaming
        {
            // Retrieve CheckBox instance.
            var directoryCheckBox = ((CheckBox) sender);
            if (directoryCheckBox == null) return;

            // Check if proper instance to handle call.
            var directoryItem = ((DirectoryInfoViewModel)(directoryCheckBox).DataContext).DirectoryItem;
            if (directoryItem != _directoryItem) return;

            // Get CheckBox state
            var checkBoxState = directoryCheckBox.IsChecked;
            if (checkBoxState == null) return;

            // 3/1/2012 - Create new Master Cancellation TokenSource; linked copied will be made at each location.
            _tokenSource = new CancellationTokenSource();

            // 3/2/2012 - Test setting start of checked control for this instance
            _startCheckedOperation = true;

            // Lock MainWindow TreeView UI.
            //_lockUnlockUiInterface.LockUI();

            // (Recurse down the tree updates)
            // Update all Children Property value with new state -
            //StartUpdateAllChildrensPropertyValue(_directoryItem, "IsChecked", checkBoxState.Value);

        }
        
        /// <summary>
        /// Captures the DirectoryAccess event error.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void _directoryInfoItem_DirectoryAccessError(object sender, EventArgs e)
// ReSharper restore InconsistentNaming
        {
            // Check if Security error - if so, then orange, otherwise red.
            Foreground = DirectoryItem.IsPasswordProtected ? "Orange" : "Red";
        }
        
        /// <summary>
        /// Helper method which creates a new ObservableCollection and 
        /// populates with the current child directories for this directory instance.
        /// </summary>
        [ShowWaitCursor]
        [DirectoryInfoExceptionHandling]
        [CatchAllExceptionHandling(true, typeof(ObservableCollection<IDirectoryItemViewModel>), null)]
        private ObservableCollection<IDirectoryItemViewModel> DoPopulateDirectories()
        {
            var directories = new ObservableCollection<IDirectoryItemViewModel>();

            // Retrieve UserControl
            var treeViewUserControl = TreeViewUserControl;

            // 3/1/2012 - Create Linked TokenSource
            CancellationTokenSource tokenSource = null;
            if (_tokenSource != null)
            {
                tokenSource = CancellationTokenSource.CreateLinkedTokenSource(_tokenSource.Token);
            }

            // Populate with current directory files.
            foreach (var di in _directoryItem.GetCurrentDirectoryChildDirectories())
            {
                // 3/1/2012 - Check if Task cancel param1.
                if (tokenSource != null && tokenSource.Token.IsCancellationRequested)
                {
                    try
                    {
                        tokenSource.Token.ThrowIfCancellationRequested();
                    }
                    catch (OperationCanceledException)
                    {
                    }

                    break;
                }

                // 4/16/2011
                var divm = new DirectoryInfoViewModel(MainWindowParent, ViewModelType, treeViewUserControl, di, this, _filterBy);

                // 11/1/2010 - Add to local directories collection.
                directories.Add(divm);
            }
            
            return directories;
        }
       
        /// <summary>
        /// Helper method which creates a new ObservableCollection and 
        /// populates with the current child files for this directory instance.
        /// </summary>
        [ShowWaitCursor]
        [DirectoryInfoExceptionHandling]
        [CatchAllExceptionHandling(true, typeof(ObservableCollection<IFileItemViewModel>), null)]
        private ObservableCollection<IFileItemViewModel> DoPopulateFiles()
        {
            var files = new ObservableCollection<IFileItemViewModel>();

            // Retrieve UserControl
            var treeViewUserControl = TreeViewUserControl;

            // 3/1/2012 - Create Linked TokenSource
            CancellationTokenSource tokenSource = null;
            if (_tokenSource != null)
            {
                tokenSource = CancellationTokenSource.CreateLinkedTokenSource(_tokenSource.Token);
            }

           
            // Populate with current directory files.
            foreach (var fi in _directoryItem.GetCurrentDirectoryFiles(_filterBy))
            {
                // 3/1/2012 - Check if Task cancel param1.
                if (tokenSource != null && tokenSource.Token.IsCancellationRequested)
                {
                    try
                    {
                        tokenSource.Token.ThrowIfCancellationRequested();
                    }
                    catch (OperationCanceledException)
                    {
                    }

                    break;
                }

                // 4/16/2011
                var fileInfoViewModel = new FileInfoViewModel(MainWindowParent, ViewModelType, treeViewUserControl, fi, this);

                // 11/1/2010 - Add to local files collection.
                files.Add(fileInfoViewModel);
            }

            return files;
        }
        
        // 11/15/2010 - Updated with new 'DirectoryInfoExceptionHandling' attribute.
        /// <summary>
        /// Invoked when the child items need to be loaded on demand.
        /// Subclasses can override this to populate the Children collection.
        /// </summary>
        [DirectoryInfoExceptionHandling]
        [CatchAllExceptionHandling(false, null, null)]
        protected override void LoadChildren()
        {
            // check if existing subDirectories exist
            if (DirectoryItem.GetDirectoriesCount > 0)
            {
                AddDirectoriesToChildrenCollection();

                // 10/23/2010 - fix: Add any additional files contain in this folder level.
                AddFilesToChildrenCollection();
            }
            else // 11/1/2010 - Add 'else' to keep files from duplicating when added to treeView.
                // else, check if files exist
                AddFilesToChildrenCollection();
           
        }

        // 10/26/2010
        /// <summary>
        /// Retrieves all files for the <see cref="_directoryItem"/> instance, and
        /// adds them to the <see cref="DirectoryViewModelBase.BaseClasses.TreeViewItemViewModel.Children"/> collection.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void AddFilesToChildrenCollection()
        {
            // 11/2/2010 - Check if 'Files' null - If yes, then populate with records below.
            if (_files == null)
            {
                _files = new ObservableCollection<IFileItemViewModel>();

                // Retrieve UserControl
                var treeViewUserControl = TreeViewUserControl;

                // 3/1/2012 - Create Linked TokenSource
                CancellationTokenSource tokenSource = null;
                if (_tokenSource != null)
                {
                    tokenSource = CancellationTokenSource.CreateLinkedTokenSource(_tokenSource.Token);
                }
               
                // Update both 'files' and 'children' collections.
                foreach (var fi in _directoryItem.GetCurrentDirectoryFiles(_filterBy))
                {
                    // 3/1/2012 - Check if Task cancel param1.
                    if (tokenSource != null && tokenSource.Token.IsCancellationRequested)
                    {
                        try
                        {
                            tokenSource.Token.ThrowIfCancellationRequested();
                        }
                        catch (OperationCanceledException)
                        {
                        }

                        break;
                    }

                    // 4/16/2011
                    var fileInfoViewModel = new FileInfoViewModel(MainWindowParent, ViewModelType, treeViewUserControl, fi, this);

                    // 11/1/2010 - Add to local files collection.
                    _files.Add(fileInfoViewModel);

                    // Add to overall children collection.
                    Children.Add(fileInfoViewModel);
                }

                return;
            }

            // Else copy already obtained 'Files' to 'Children' collection.
            foreach (var fi in _files)
            {
                // Add to overall children collection.
                Children.Add(fi);
            }
        }

        // 10/26/2010
        /// <summary>
        /// Retrieves all directories for the <see cref="_directoryItem"/> instance, and
        /// adds them to the <see cref="DirectoryViewModelBase.BaseClasses.TreeViewItemViewModel.Children"/> collection.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void AddDirectoriesToChildrenCollection()
        {
            // 11/2/2010 - Check if 'directories' null - If yes, then populate with records below.
            if (_directories == null)
            {
                _directories = new ObservableCollection<IDirectoryItemViewModel>();

                // Retrieve UserControl
                var treeViewUserControl = TreeViewUserControl;

                // 3/1/2012 - Create Linked TokenSource
                CancellationTokenSource tokenSource = null;
                if (_tokenSource != null)
                {
                    tokenSource = CancellationTokenSource.CreateLinkedTokenSource(_tokenSource.Token);
                }

                // Update both 'directories' and 'children' collections.
                foreach (var di in _directoryItem.GetCurrentDirectoryChildDirectories())
                {
                    // 3/1/2012 - Check if Task cancel param1.
                    if (tokenSource != null && tokenSource.Token.IsCancellationRequested)
                    {
                        try
                        {
                            tokenSource.Token.ThrowIfCancellationRequested();
                        }
                        catch (OperationCanceledException)
                        {
                        }

                        break;
                    }

                    // 4/16/2011
                    var divm = new DirectoryInfoViewModel(MainWindowParent, ViewModelType, treeViewUserControl, di, this, _filterBy);

                    // 11/1/2010 - Add to local directories collection.
                    _directories.Add(divm);

                    // Add to overall children collection.
                    Children.Add(divm);
                }

                return;
            }
            
            // 11/3/2010 - Fixed.
            // Else copy already obtained 'directories' to 'Children' collection.
            foreach (var di in _directories)
            {
                Children.Add(di);
            }

        }

        // 8/14/2011
        /// <summary>
        /// Unlocks TreeView UI when tasks completes.
        /// </summary>
        /// <param name="obj"></param>
        protected override void TaskCompleted(System.Threading.Tasks.Task obj)
        {
            // UnLock MainWindow TreeView UI.
            _lockUnlockUiInterface.UnLockUI();
        }

        // 12/3/2010
        /// <summary>
        /// EventHandler to capture the 'Cancel' button contain within
        /// the 'MainWindow' footer section.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public static void CancelAsyncOperation(object sender, EventArgs e)
        {
            if (_tokenSource != null) 
                _tokenSource.Cancel(false);
        }

        // 8/15/2011
        /// <summary>
        /// Triggers the <see cref="StartPopluateDirectories"/> event.
        /// </summary>
        private void OnStartPopluateDirectories()
        {
            if (StartPopluateDirectories != null)
                StartPopluateDirectories(this, EventArgs.Empty);
        }

        // 8/15/2011
        /// <summary>
        /// Triggers the <see cref="EndPopulateDirectories"/> event.
        /// </summary>
        private void OnEndPopulateDirectories()
        {
            if (EndPopulateDirectories != null)
                EndPopulateDirectories(this, EventArgs.Empty);
        }

        // 8/15/2011
        /// <summary>
        /// Triggers the <see cref="StartPopulateFiles"/> event.
        /// </summary>
        private void OnStartPopulateFiles()
        {
            if (StartPopulateFiles != null)
                StartPopulateFiles(this, EventArgs.Empty);
        }

        // 8/15/2011
        /// <summary>
        /// Triggers the <see cref="EndPopulateFiles"/> event.
        /// </summary>
        private void OnEndPopulateFiles()
        {
            if (EndPopulateFiles != null)
                EndPopulateFiles(this, EventArgs.Empty);
        }


        // 8/15/2011
        /// <summary>
        /// Triggers the <see cref="StartUpdatePropertyValues"/> event.
        /// </summary>
        private void OnStartUpdatePropertyValues()
        {
            if (StartUpdatePropertyValues != null)
                StartUpdatePropertyValues(this, EventArgs.Empty);
        }

        // 8/15/2011
        /// <summary>
        /// Triggers the <see cref="EndUpdatePropertyValues"/> event.
        /// </summary>
        private void OnEndUpdatePropertyValues()
        {
            if (EndUpdatePropertyValues != null)
                EndUpdatePropertyValues(this, EventArgs.Empty);
        }

        // 1/12/2012
        /// <summary>
        /// Triggers the <see cref="DirectoryCheckedItemUpdated"/> event.
        /// </summary>
        private void OnDirectoryCheckedItemUpdated()
        {
            if (DirectoryCheckedItemUpdated != null)
                DirectoryCheckedItemUpdated(this, EventArgs.Empty);
        }

        // 1/12/2012
        /// <summary>
        /// Triggers the <see cref="ScheduleNumberUpdated"/> event.
        /// </summary>
        private void OnScheduleNumberUpdated()
        {
            if (ScheduleNumberUpdated != null)
                ScheduleNumberUpdated(this, EventArgs.Empty);
        }

        // 5/14/2011
        /// <summary>
        /// Occurs when the user clicks the Refresh action button.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        void treeViewUserControl_DirectoryRefreshClicked(object sender, EventArgs e)
        {
            // Retrieve control instance.
            var menuItem = ((MenuItem)sender);
            if (menuItem == null) return;

            // Check if proper instance to handle call.
            //var directoryInfoItem = ((DirectoryCloudItemViewModel)menuItem.DataContext).DirectoryItem;
            //if (directoryInfoItem != _directoryCloudItem) return;

            // 1st - iterate files to remove from children collection
            foreach (var fileItemViewModel in _files)
            {
                if (fileItemViewModel == null) continue;

                Children.Remove(fileItemViewModel);
            }

            // 2nd - clear files collection
            _files.Clear();

            // 3rd - null collection
            _files = null;

            // 3/1/2012 - Create Linked TokenSource. This allows for multiple linked copied of the original TokenSource, thereby
            //            allowing the original to be nullified, but each linked copy will still funtion.
            _tokenSource = new CancellationTokenSource();

            // 4th - Recreate Files collection from cloud
            AddFilesToChildrenCollection();

            // 5th - Force rebinding to make WPF refresh the UI.
            IsSelected = false;
            IsSelected = true;
        }

        #region MiscMethods

        // 9/25/2011
        /// <summary>
        /// Reconnect checked instances if exist; otherwise, create new instance.
        /// </summary>
        /// <param name="directoryItem">Instance of <see cref="IDirectoryItem"/></param>
        /// <param name="directoryCheckedItem">(OUT) Instance of <see cref="DirectoryCheckedItem">.</see></param>
        private void ReconnectCheckedItems(IDirectoryItem directoryItem, out DirectoryCheckedItem directoryCheckedItem)
        {
            // 1/9/2012 - Retrieve interface CheckedItems.
            var checkedItems = MainWindowParent.UnityContainer.Resolve<DirectoryTreeViewModelAdapterBase>
                (ViewModelType.ToString());

#if DEBUG
            // checks if the uniqueKey is what is expected for the enum type.
            if (ViewModelType == ViewModelTypeEnum.DesktopViewModel && checkedItems.CheckedCollectionUniqueKey != (int)DirectoryTreeViewModelAdapterEnum.Desktop)
            {
                throw new InvalidOperationException("UniqueKey is wrong!");
            }

            if (ViewModelType == ViewModelTypeEnum.CloudViewModel && checkedItems.CheckedCollectionUniqueKey != (int)DirectoryTreeViewModelAdapterEnum.Cloud)
            {
                throw new InvalidOperationException("UniqueKey is wrong!");
            }
#endif

            var checkedCollection = GetCheckedCollection(checkedItems.CheckedCollectionUniqueKey);
            if (checkedCollection.IsInDirectoriesCollection(DirectoryItem.FullName))
            {
                directoryCheckedItem = (DirectoryCheckedItem)checkedCollection.GetDirectoryItem(DirectoryItem.FullName);

                // 12/10/2010 - Set Base
                base.IsChecked = directoryCheckedItem.IsChecked;
            }
            // Note: Otherwise, create pointer for checked item; however, this doesn't imply that this is a TRUE checked item.
            else
            {
                // 10/26/2010 - Create instance of CheckItem
                directoryCheckedItem = new DirectoryCheckedItem(DirectoryItem) { OriginalItem = directoryItem };

                // 11/9/2010 - Save Ref to data type.
                // 12/10/2010 - And add to collection.
                checkedCollection.AddUpdateItemFromCollection(directoryCheckedItem);
            }

            // 10/9/2011 - Set Inderminate
            IsIndeterminate = directoryCheckedItem.IsIndeterminate;

            //((DirectoryInfoViewModel)Parent).UpdateAllChildrenCheckedProperty();
        }

        // Note: Desktop
        // 12/9/2010
        /// <summary>
        /// Updates the 'IsChecked' property for this directory, depending on
        /// the condition of the 'IsChecked' property for its children.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        internal void UpdateAllChildrenCheckedProperty()
        {
            // 1st check '_files' and '_directories' children collection are ALL 'TRUE'.
            var allFilesChecked = _files != null && _files.AsParallel().All(fileItemViewModel => fileItemViewModel.IsChecked);
            var allDirectoriesChecked = _directories != null && _directories.AsParallel().All(directoryItemViewModel => directoryItemViewModel.IsChecked);

            // Update 'IsChecked'.
            IsChecked = allFilesChecked && allDirectoriesChecked;

            // Note: 9/25/2011 - The IsChecked call above will propagate up the hierarchy, and then it will come back down the hierarchy here!

            // 1st check '_files' and '_directories' children collection are ALL 'TRUE'.
            allFilesChecked = _files != null && _files.AsParallel().All(fileItemViewModel => fileItemViewModel.IsChecked);
            allDirectoriesChecked = _directories != null && _directories.AsParallel().All(directoryItemViewModel => directoryItemViewModel.IsChecked);

            // 2nd check '_files' and '_directories' children collection are ALL 'FALSE'.
            // NOTE: To be used to determine 'Indeterminate' mode.
            var allFilesNotChecked = _files != null && _files.AsParallel().All(fileItemViewModel => !fileItemViewModel.IsChecked);
            var allDirectoriesNotChecked = _directories != null && _directories.AsParallel().All(directoryItemViewModel => !directoryItemViewModel.IsChecked);

            // 12/10/2010
            // 3rd check '_directores' children collection are ALL not 'Indeterminate'.
            var allDirectoriesNotIndeterminate = _directories != null && _directories.AsParallel().All(directoryItemViewModel => !directoryItemViewModel.IsIndeterminate);

            // Update 'IsIndeterminate'.
            IsIndeterminate = !(allFilesNotChecked && allDirectoriesNotChecked) &&
                              !(allFilesChecked && allDirectoriesChecked) || !allDirectoriesNotIndeterminate;

            // Update Parent's Indeterminate state.
            if (Parent != null) ((DirectoryInfoViewModel)Parent).UpdateAllChildrenCheckedProperty();
        }

        #endregion

    }
}