﻿using System;
using System.Drawing;
using System.Windows.Media.Imaging;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.Delegates;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.DesktopDataModel.CheckedDataModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.Enums;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.Properties;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.BaseClasses;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.Enums;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl;
using ImageNexus.BenScharbach.ICLTools.ICLExtensions;
using Microsoft.Practices.Unity;

// Adds the Unity Extenstions.

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.ViewModel.DirectoryViewModel
{
    // 10/23/2010
    /// <summary>
    /// The <see cref="FileInfoViewModel"/> class inherits from <see cref="DirectoryViewModelBase.BaseClasses.TreeViewItemViewModel"/>, which represents a 
    /// single file in the treeView.
    /// </summary>
    public sealed class FileInfoViewModel : TreeViewItemViewModel, IFileItemViewModel
    {
        #region Fields

        // 4/16/2011 - Updated to interface.
        private readonly IFileItem _fileItem;
        // 11/16/2010 - Converted to BitmapSource, instead of BitmapImage for transparency to work.
        private static BitmapSource _fileIcon;

        // 10/26/2010 - CheckedItem
        private readonly FileCheckedItem _fileCheckedItem;

        #endregion

        #region Events

        // 7/4/2010 - Event
        /// <summary>
        /// Occurs when a file is selected.
        /// </summary>
        public static event SelectedItemEventHandler FileSelected; 

        // 9/25/2011
        /// <summary>
        /// Occurs when the <see cref="IsChecked"/> value is updated with a different value than the previous.
        /// </summary>
        public event EventHandler CheckedValueChanged;

        #endregion

        #region Properties

        // 7/7/2010
        /// <summary>
        /// Gets the current <see cref="FileItem"/>.
        /// </summary>
        public IFileItem FileItem
        {
            get { return _fileItem; }
        }

        // 7/7/2010
        /// <summary>
        /// Gets the full TreeView's directory path, starting from root directory.
        /// </summary>
        /// <example>
        /// If actual directory is => "c:\Users\Ben\MyDocs\Game\GameContent\Textures\DesertTextures\texture1.png", and
        /// the TreeView is set to start at => "Textures" directory level, then this property would return the following
        /// result for the given example; "Textures\DesertTextures\texture1".
        /// </example>
        public string TreeViewDirectoryPath
        {
            get 
            { 
                if (Parent != null) 
                    return ((DirectoryInfoViewModel)Parent).TreeViewsDirectoryPath + @"\" + FileName;
                
                return string.Empty;
            }
        }

        /// <summary>
        /// Gets the current file name.
        /// </summary>
        public string FileName
        {
            get
            {
                return FileItem != null ? FileItem.FileName : string.Empty;
            }
        }

        // 11/1/2010
        /// <summary>
        /// Gets the current file name with extension.
        /// </summary>
        public string FileNameWithExtension
        {
            get 
            {
                if (FileItem != null) return FileItem.FileNameWithExtension;
                
                return string.Empty;
            }
        }

        // 11/1/2010
        /// <summary>
        /// Gets the current file size.
        /// </summary>
        public long FileSize
        {
            get 
            { 
                return FileItem != null ? FileItem.FileSize : 0;
            }
        }

        // 11/1/2010
        /// <summary>
        /// Gets the current file's <see cref="DateTime"/> it was last modified.
        /// </summary>
        public DateTime FileLastModified
        {
            get 
            {
                return FileItem != null ? FileItem.FileLastModified : DateTime.Now;
            }
        }

        // 11/16/2010 - Converted to BitmapSource, instead of BitmapImage for transparency to work.
        /// <summary>
        /// Gets the current Icon image for this file.
        /// </summary>
        public BitmapSource FileIcon
        {
            get
            {
                // 11/15/2010
                if (_fileIcon == null)
                    CreateFileBitmapIcon();

                return _fileIcon;
            }
        }

        /// <summary>
        /// Gets/sets whether the TreeViewItem 
        /// associated with this object is selected.
        /// </summary>
        public override bool IsSelected
        {
            get
            {
                return base.IsSelected;
            }
            set
            {
                // is true
                if (value && FileSelected != null)
                {
                    FileSelected(this, new SelectedItemEventArgs { ImageName = (FileItem != null) ? FileItem.FileName : string.Empty });
                }

                base.IsSelected = value;
            }
        }

        // 10/23/2010
        /// <summary>
        /// Gets/sets whether the TreeViewItem 
        /// associated with this object is checked.
        /// </summary>
        public override bool IsChecked
        {
            get
            {
                return _fileCheckedItem != null && _fileCheckedItem.IsChecked;
            }
            set
            {
                if (_fileCheckedItem != null)
                {
                    _fileCheckedItem.IsChecked = value;
                }

                base.IsChecked = value;
            }
        }

        // 12/8/2010
        /// <summary>
        /// Gets/sets whether the TreeViewItem 
        /// associated with this object has a schedule number.
        /// </summary>
        [CatchAllExceptionHandling(false, typeof(int), 0)]
        public override int ScheduleNumber
        {
            get { return _fileCheckedItem != null ? _fileCheckedItem.ScheduleNumber : 0; }

            
            set
            {
                if (_fileCheckedItem != null && value != _fileCheckedItem.ScheduleNumber)
                {
                    // Set into CheckedItem
                    _fileCheckedItem.ScheduleNumber = value;
                    OnPropertyChanged("ScheduleNumber");
                }
            }
        }

        #endregion

        /// <summary>
        /// constructor
        /// </summary>
        /// <param name="mainWindow"> </param>
        /// <param name="viewModelType"> </param>
        /// <param name="treeViewUserControl">Instance of the WPF <see cref="ImageNexusWpfTreeViewControl.TreeViewUserControl"/>.</param>
        /// <param name="fileItem"></param>
        /// <param name="parentDirectory">Instance of parent <see cref="DirectoryInfoViewModel"/></param>
        [CatchAllExceptionHandling(false, null, null)]
        public FileInfoViewModel(IMainWindow mainWindow, ViewModelTypeEnum viewModelType, TreeViewUserControl treeViewUserControl, IFileItem fileItem, DirectoryInfoViewModel parentDirectory)
            : base(mainWindow, viewModelType, treeViewUserControl, parentDirectory, false)
        {
            _fileItem = fileItem;

            // 12/9/2010 - Reconnect checked instances if exist; otherwise, create new instance.
            ReconnectCheckedItems(fileItem, out _fileCheckedItem);

            // 10/9/2011 - Subscribe to the File Checkbox Click event.
            treeViewUserControl.FileCheckboxClicked += treeViewUserControl_FileCheckboxClicked;

            // 2/12/2012 - Subscribe to...
            if (parentDirectory != null)
            {
                // the Directory 'IsChecked' updated event.
                parentDirectory.DirectoryCheckedItemUpdated += parentDirectory_DirectoryCheckedItemUpdated;
                // the Directory 'ScheduleNumber' updated event.
                parentDirectory.ScheduleNumberUpdated += parentDirectory_ScheduleNumberUpdated;
            }
        }

        // 1/12/2012
        /// <summary>
        /// Occurs when the Directory's 'ScheduleNumber' value is updated.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void parentDirectory_ScheduleNumberUpdated(object sender, EventArgs e)
        {
            var parentDirectory = (DirectoryInfoViewModel)sender;
            if (parentDirectory == null) return;

            // Set folder value to same as directory
            ScheduleNumber = parentDirectory.ScheduleNumber;
        }

        // 1/12/2012
        /// <summary>
        /// Occurs when the Directory's 'Ischecked' value is updated.
        /// </summary>
        private void parentDirectory_DirectoryCheckedItemUpdated(object sender, EventArgs e)
        {
            var parentDirectory = (DirectoryInfoViewModel) sender;
            if (parentDirectory == null) return;

            // Set folder value to same as directory
            IsChecked = parentDirectory.IsChecked;
        }

        #region MiscMethods

        // 10/9/2011
        /// <summary>
        /// Eventhandler for the File CheckBox Clicked event.
        /// </summary>
        private void treeViewUserControl_FileCheckboxClicked(object sender, System.Windows.RoutedEventArgs e)
        {
            // Retrieve CheckBox instance.
            /*var fileCheckBox = ((CheckBox)sender);
            if (fileCheckBox == null) return;

            // Check if proper instance to handle call.
            var fileItem = ((FileInfoViewModel)(fileCheckBox).DataContext).FileItem;
            if (fileItem != _fileItem) return;

            // Get CheckBox state
            var checkBoxState = fileCheckBox.IsChecked;
            if (checkBoxState == null) return;

            // Note: Desktop
            // Update all directory checked items and interminate
            ((DirectoryInfoViewModel)Parent).UpdateAllChildrenCheckedProperty();*/
        }

        // 9/25/2011
        /// <summary>
        /// Reconnect checked instances if exist; otherwise, create new instance.
        /// </summary>
        /// <param name="fileItem">Instance of <see cref="IFileItem"/>.</param>
        /// <param name="fileCheckedItem">(OUT) Instance of <see cref="FileCheckedItem"/>.</param>
        private void ReconnectCheckedItems(IFileItem fileItem, out FileCheckedItem fileCheckedItem)
        {
            // 1/9/2012 - Retrieve interface CheckedItems.
            var checkedItems = MainWindowParent.UnityContainer.Resolve<DirectoryTreeViewModelAdapterBase>
                (ViewModelType.ToString());

#if DEBUG
            // checks if the uniqueKey is what is expected for the enum type.
            if (ViewModelType == ViewModelTypeEnum.DesktopViewModel && checkedItems.CheckedCollectionUniqueKey != (int)DirectoryTreeViewModelAdapterEnum.Desktop)
            {
                throw new InvalidOperationException("UniqueKey is wrong!");
            }

            if (ViewModelType == ViewModelTypeEnum.CloudViewModel && checkedItems.CheckedCollectionUniqueKey != (int)DirectoryTreeViewModelAdapterEnum.Cloud)
            {
                throw new InvalidOperationException("UniqueKey is wrong!");
            }
#endif

            var checkedCollection = GetCheckedCollection(checkedItems.CheckedCollectionUniqueKey);
            if (checkedCollection.IsInFilesCollection(_fileItem.FullPathName))
            {
                fileCheckedItem = (FileCheckedItem)checkedCollection.GetFileItem(_fileItem.FullPathName);

                // 12/10/2010 - Set Base
                base.IsChecked = fileCheckedItem.IsChecked;
            }
            // Note: Otherwise, create pointer for checked item; however, this doesn't imply that this is a TRUE checked item.
            else
            {
                // 10/26/2010 - Create new instance of CheckedITem
                fileCheckedItem = new FileCheckedItem(fileItem);

                // 12/10/2010 - And add to collection
                checkedCollection.AddUpdateItemFromCollection(fileCheckedItem);
            }
        }

        // 11/15/2010
        /// <summary>
        /// Helper method used to create the file icon.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private static void CreateFileBitmapIcon()
        {
            var fileIcon = new Bitmap(Resources.File_32);
            _fileIcon = fileIcon.ConvertToTransparentBitmapSource();
        }

        #endregion
    }
}