using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Threading.Tasks;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.Enums;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.CheckedDataModel;

namespace ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.BaseClasses
{
    // 8/14/2011
    public class ProgressEventArgs : EventArgs
    {
        public double Value;
        public string Message;
    }

    // 11/15/2010 Updated to inherit from DispatcherObject.
    /// <summary>
    /// Base class for all ViewModel classes displayed by TreeViewItems.  
    /// This acts as an adapter between a raw data object and a TreeViewItem.
    /// </summary>
    public class TreeViewItemViewModel : ViewModelBase, INotifyPropertyChanged, ITreeViewItemViewModel
    {
        #region Data

        private static ITreeViewItemViewModel DummyChild;

        private readonly ObservableCollection<ITreeViewItemViewModel> _children;
        private readonly ITreeViewItemViewModel _parent;

        // 10/26/2010 - WPF Usercontrol
        protected TreeViewUserControl TreeViewUserControl; 

        private bool _isExpanded;
        private bool _isSelected;
        private bool _isChecked; // 10/23/2010
        private string _foreground = "Black"; // 10/25/2010
        private string _checkBoxBackground = "Black"; // 10/26/2010
        private int _scheduleNumber; // 12/7/2010

        // 11/14/2010 - 
        /// <summary>
        /// The <see cref="Action{T}"/> is used to communicate the start
        /// of an iteration which will provide 'Progress' feedback; for example, when
        /// the directories are being populated.
        /// </summary>
        public static event EventHandler<ProgressEventArgs> ResetProgressBar;

        // 11/14/2010
        /// <summary>
        /// The <see cref="Action{T}"/> is used to communicate the
        /// current progress value during some iteration of records.
        /// </summary>
        public static event EventHandler<ProgressEventArgs> SetProgressValue;

        // 11/15/2010
        /// <summary>
        /// The <see cref="Action"/> is used to communicate the
        /// current operation has completed.
        /// </summary>
        public static event EventHandler EndProgressBar;

        // 12/3/2010
        /// <summary>
        /// The <see cref="Action{String}"/> is used to communicate the
        /// setting the header text to the mainWindow's footer control.
        /// </summary>
        public static event EventHandler<ProgressEventArgs> SetFooterHeaderText;

       

        #endregion // Data

        #region Constructors

        /// <summary>
        /// constructor
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="IMainWindow"/></param>
        /// <param name="viewModelType"> </param>
        /// <param name="treeViewUserControl">Instance of the WPF <see cref="ImageNexusWpfTreeViewControl.TreeViewUserControl"/>.</param>
        /// <param name="parent"><see cref="TreeViewItemViewModel"/> instance as parent</param>
        /// <param name="lazyLoadChildren">Do (lazy-load) load on demand feature?</param>
        [CatchAllExceptionHandling(false, null, null)]
        protected TreeViewItemViewModel(IMainWindow mainWindow, ViewModelTypeEnum viewModelType, TreeViewUserControl treeViewUserControl, TreeViewItemViewModel parent, bool lazyLoadChildren)
            : base(mainWindow, viewModelType)
        {
            // store parent
            _parent = parent;
            // 10/26/2010 - Store WPF UserControl
            TreeViewUserControl = treeViewUserControl;

            // 1/10/2012 - Create Dummy Record
            DummyChild = new TreeViewItemViewModel(mainWindow, viewModelType);

            // create child ObservableCollection
            _children = new ObservableCollection<ITreeViewItemViewModel>();

            if (lazyLoadChildren)
                _children.Add(DummyChild);
        }

        // This is used to create the DummyChild instance.
        private TreeViewItemViewModel(IMainWindow mainWindow, ViewModelTypeEnum viewModelType)
            : base(mainWindow, viewModelType)
        {
        }

        #endregion // Constructors

        #region Presentation Members

        #region Children

        /// <summary>
        /// Returns the logical child items of this object.
        /// </summary>
        public ObservableCollection<ITreeViewItemViewModel> Children
        {
            get { return _children; }
        }

        #endregion // Children

        #region HasLoadedChildren

        // 11/30/2010 - Updated to check if null.
        /// <summary>
        /// Returns true if this object's Children have not yet been populated.
        /// </summary>
        public bool HasDummyChild
        {
            get { return Children != null && (Children.Count == 1 && Children[0] == DummyChild); }
        }

        #endregion // HasLoadedChildren

        #region IsExpanded

        /// <summary>
        /// Gets/sets whether the TreeViewItem 
        /// associated with this object is expanded.
        /// </summary>
        public virtual bool IsExpanded
        {
            get { return _isExpanded; }
            set
            {
                if (value != _isExpanded)
                {
                    _isExpanded = value;
                    OnPropertyChanged("IsExpanded");
                }

                // Expand all the way up to the root.
                if (_isExpanded && _parent != null)
                    _parent.IsExpanded = true;

                // Lazy load the child items, if necessary.
                if (HasDummyChild)
                {
                    // Remove dummy child.
                    Children.Clear();

                    LoadChildren();
                }
            }
        }

        #endregion // IsExpanded

        #region IsSelected

        /// <summary>
        /// Gets/sets whether the TreeViewItem 
        /// associated with this object is selected.
        /// </summary>
        public virtual bool IsSelected
        {
            get { return _isSelected; }
            set
            {
                if (value != _isSelected)
                {
                    _isSelected = value;
                    OnPropertyChanged("IsSelected");
                }
            }
        }

        #endregion // IsSelected

        #region IsChecked
        
        /// <summary>
        /// Gets/sets whether the TreeViewItem 
        /// associated with this object is checked.
        /// </summary>
        public virtual bool IsChecked
        {
            get { return _isChecked; }
            set
            {
                if (value != _isChecked)
                {
                    _isChecked = value;
                    OnPropertyChanged("IsChecked");
                }

                // 4/16/2011
                // Lazy load the child items, if necessary.
                if (HasDummyChild)
                {
                    // Remove dummy child.
                    Children.Clear();

                    LoadChildren();
                }
            }
        }

        #endregion // IsChecked

        #region ScheduleNumber

        // 12/7/2010
        /// <summary>
        /// Gets/sets whether the TreeViewItem 
        /// associated with this object has a schedule number.
        /// </summary>
        public virtual int ScheduleNumber
        {
            get { return _scheduleNumber; }
            set
            {
                if (value != _scheduleNumber)
                {
                    _scheduleNumber = value;
                    OnPropertyChanged("ScheduleNumber");
                }
            }
        }


        #endregion


        #region Foreground

        // 10/25/2010
        // NOTE: Apparently need to use 'String' value, since default converter in WPF will convert the name
        //       to the appropriate 'SolidColorBrush'.
        /// <summary>
        /// Gets/sets whether the TreeViewItem 
        /// associated with this object has the given foreground color.
        /// </summary>
        public virtual string Foreground
        {
            get { return _foreground; }
            set
            {
                if (value != _foreground)
                {
                    _foreground = value;
                    OnPropertyChanged("Foreground");
                }
            }
        }
        #endregion // Foreground


        // 10/26/2010
        /// <summary>
        /// Gets/sets whether the TreeViewItem 'CheckBox'
        /// associated with this object has the given background color.
        /// </summary>
        public virtual string CheckBoxBackground
        {
            get { return _checkBoxBackground; }
            set
            {
                if (value != _checkBoxBackground)
                {
                    _checkBoxBackground = value;
                    OnPropertyChanged("CheckBoxBackground");
                }
            }
        }

        #region LoadChildren

        /// <summary>
        /// Invoked when the child items need to be loaded on demand.
        /// Subclasses can override this to populate the Children collection.
        /// </summary>
        protected virtual void LoadChildren()
        {
        }

        #endregion // LoadChildren

        #region Parent

        public ITreeViewItemViewModel Parent
        {
            get { return _parent; }
        }

        #endregion // Parent

        #endregion // Presentation Members

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        [CatchAllExceptionHandling(false, null, null)]
        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion // INotifyPropertyChanged Members
        
        // 12/9/2010
        /// <summary>
        /// Gets the <see cref="CheckedCollection{T,TU}"/> singleton.
        /// </summary>
        /// <param name="key">Unique key for collection</param>
        /// <returns>Instance of CheckedCollection.</returns>
        [CatchAllExceptionHandling(false, null, null)]
        protected CheckedCollection<IFileItem, IDirectoryItem> GetCheckedCollection(int key)
        {
            return CheckedCollection<IFileItem, IDirectoryItem>.GetInstance(key);
        }

        // 8/14/2011 - Was 'UnLockUi'.
        /// <summary>
        /// Called when a task completes.
        /// </summary>
        /// <param name="obj"></param>
        [CatchAllExceptionHandling(false, null, null)]
        protected virtual void TaskCompleted(Task obj)
        {
            // Overriding classes can update the UI.
        }

        /// <summary>
        /// Updates the footer header with 'Cancelled' message.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        protected void DisplayCancelled()
        {
            OnSetFooterHeaderText(this, "Asynchronous operation cancelled.");
        }

        // 8/14/2011
        private static void OnResetProgressBar(TreeViewItemViewModel treeViewItemViewModel, double count)
        {
           if (ResetProgressBar != null)
               ResetProgressBar(treeViewItemViewModel, new ProgressEventArgs
                                          {
                                              Value = count
                                          });
        }

        // 8/14/2011
        private void SetProgressValueCallback(double iteration)
        {
            OnSetProgressValue(this, iteration);
        }

        // 8/14/2011
        private static void OnSetProgressValue(TreeViewItemViewModel treeViewItemViewModel, double iteration)
        {
            if (SetProgressValue != null)
                SetProgressValue(treeViewItemViewModel, new ProgressEventArgs
                {
                    Value = iteration
                });
        }

        // 8/14/2011
        private static void OnEndProgressBar(TreeViewItemViewModel treeViewItemViewModel)
        {
            if (EndProgressBar != null)
                EndProgressBar(treeViewItemViewModel, EventArgs.Empty);
        }

        // 8/14/2011
        private static void OnSetFooterHeaderText(TreeViewItemViewModel treeViewItemViewModel, string message)
        {
            if (SetFooterHeaderText != null)
                SetFooterHeaderText(treeViewItemViewModel, new ProgressEventArgs
                {
                    Message = message
                });
        }
            
    }
}