﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows;
using System.Windows.Media.Imaging;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;

namespace ImageNexus.BenScharbach.ICLTools.ICLExtensions
{
    // 7/4/2010
    ///<summary>
    /// The <see cref="WpfExtensions"/> is used to extend other pre-made classes with new
    /// abilities. 
    ///</summary>
    public static class WpfExtensions
    {
        // 7/4/2010
        /// <summary>
        /// Extends the <see cref="System.Drawing.Bitmap"/> class by including this new method which
        /// converts an instance of the <see cref="System.Drawing.Bitmap"/> to a new instance of the WPF 
        /// <see cref="System.Windows.Media.Imaging.BitmapImage"/> type.
        /// </summary>
        /// <param name="bitmap">this instance of <see cref="System.Drawing.Bitmap"/></param>
        /// <returns>New instance of <see cref="System.Windows.Media.Imaging.BitmapImage"/></returns>
        [CatchAllExceptionHandling(true, typeof(BitmapImage), null)]
        public static BitmapImage ToBitmapImage(this Bitmap bitmap)
        {
            //
            // Convert to proper BitmapImage format.
            //

            // 1st - convert into memory stream
            var memoryStream = new MemoryStream();
            bitmap.Save(memoryStream, System.Drawing.Imaging.ImageFormat.Bmp);

            // 2nd - create new BitmapImage type from memory stream.
            var bImg = new BitmapImage();
            bImg.BeginInit();
            bImg.StreamSource = new MemoryStream(memoryStream.ToArray());
            bImg.EndInit();

            memoryStream.Close();

            return bImg;
        }

        // 11/16/2010
        /// <summary>
        /// Extends the <see cref="System.Windows.Media.Imaging.BitmapSource"/> class by including this new method which
        /// converts an instance of the <see cref="System.Windows.Media.Imaging.BitmapSource"/> to a new instance of the WPF 
        /// <see cref="System.Windows.Media.Imaging.BitmapImage"/> type.
        /// </summary>
        /// <param name="bitmapSource">this instance of <see cref="System.Windows.Media.Imaging.BitmapSource"/></param>
        /// <returns>New instance of <see cref="System.Windows.Media.Imaging.BitmapImage"/></returns>
        [CatchAllExceptionHandling(true, typeof(BitmapImage), null)]
        public static BitmapImage ToBitmapImage(this BitmapSource bitmapSource)
        {
            //
            // Convert to proper BitmapImage format.
            //

            // 1st - Encode into bitmapFrame, to allow saving to memory stream.
            var encoder = new BmpBitmapEncoder();
            encoder.Frames.Add(BitmapFrame.Create(bitmapSource));

            // 2nd - convert into memory stream
            var memoryStream = new MemoryStream();
            encoder.Save(memoryStream);

            // 3rd - create new BitmapImage type from memory stream.
            var bImg = new BitmapImage();
            bImg.BeginInit();
            bImg.StreamSource = new MemoryStream(memoryStream.ToArray());
            bImg.EndInit();

            memoryStream.Close();

            return bImg;
        }

        // 11/11/2010
        /// <summary>
        /// Extends the <see cref="System.Drawing.Bitmap"/> class by including this new method which
        /// converts an instance of the <see cref="System.Drawing.Bitmap"/> to a new instance of the WPF
        /// <see cref="System.Windows.Media.Imaging.BitmapImage"/> type which now supports transparency.
        /// </summary>
        /// <param name="bitmap">this instance of <see cref="System.Drawing.Bitmap"/>.</param>
        /// <returns>Converted instance of <see cref="System.Windows.Media.Imaging.BitmapImage"/> with transparency.</returns>
        [CatchAllExceptionHandling(true, typeof(BitmapSource), null)]
        public static BitmapSource ConvertToTransparentBitmapSource(this Bitmap bitmap)
        {
            bitmap.MakeTransparent(); // System.Drawing.Color.White

            return System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(bitmap.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty,
                BitmapSizeOptions.FromEmptyOptions());
        }

        // 11/11/2010 - Overload version.
        /// <summary>
        /// Extends the <see cref="System.Drawing.Bitmap"/> class by including this new method which
        /// converts an instance of the <see cref="System.Drawing.Bitmap"/> to a new instance of the WPF
        /// <see cref="System.Windows.Media.Imaging.BitmapImage"/> type which now supports transparency.
        /// </summary>
        /// <param name="bitmap">this instance of <see cref="System.Drawing.Bitmap"/>.</param>
        /// <param name="transparencyColorKey">Makes the specified color transparent for this <see cref="System.Drawing.Bitmap"/>.</param>
        /// <returns>Converted instance of <see cref="System.Windows.Media.Imaging.BitmapImage"/> with transparency.</returns>
        [CatchAllExceptionHandling(true, typeof(BitmapSource), null)]
        public static BitmapSource ConvertToTransparentBitmapSource(this Bitmap bitmap, System.Drawing.Color transparencyColorKey)
        {
            bitmap.MakeTransparent(transparencyColorKey); // System.Drawing.Color.White

            return System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(bitmap.GetHbitmap(), IntPtr.Zero, Int32Rect.Empty,
                BitmapSizeOptions.FromEmptyOptions());
        }

        // 10/25/2010
        /// <summary>
        /// Extends the generic Dictionary{T,U} class by including this method which adds
        ///  or updates the current type U, depending if the key T exist, and if the 'IsChecked' WPF
        /// listview property is set.
        /// </summary>
        /// <typeparam name="T">Generic T type as Key.</typeparam>
        /// <typeparam name="TU">Generic TU type as Item.</typeparam>
        /// <param name="dictionary">This reference of <see cref="Dictionary{TKey,TValue}"/>.</param>
        /// <param name="key">Dictionary Key</param>
        /// <param name="item">Dictionary Item</param>
        /// <param name="isChecked">WPF ListView property 'IsChecked'.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public static void AddRemoveItemFromCollection<T, TU>(this Dictionary<T, TU> dictionary, T key, TU item, bool isChecked)
        {
            
            if (isChecked)
            {
                // check if exist
                if (dictionary.ContainsKey(key))
                {
                    // then update current instance
                    dictionary[key] = item;
                }
                else
                {
                    // then add to dictionary
                    dictionary.Add(key, item);
                }

            }
            else
            {
                // check if exist
                if (!dictionary.ContainsKey(key)) return;

                // otherwise, remove from dictionary
                dictionary.Remove(key);
            }
        }

        // 10/9/2011 - Updated to add or update, rather than remove.
        // 11/30/2010 - ConcurrentDictionary version.
        /// <summary>
        /// Extends the generic ConcurrentDictionary{T,U} class by including this method which adds
        ///  or updates the current type U, depending if the key T exist, and if the 'IsChecked' WPF
        /// listview property is set.
        /// </summary>
        /// <typeparam name="TKey">Generic T type as Key.</typeparam>
        /// <typeparam name="TValue">Generic TU type as Item.</typeparam>
        /// <param name="dictionary">This reference of <see cref="Dictionary{TKey,TValue}"/>.</param>
        /// <param name="key">Dictionary Key</param>
        /// <param name="item">Dictionary Item</param>
        [CatchAllExceptionHandling(false, null, null)]
        public static void AddOrUpdateItemFromCollection<TKey, TValue>(this ConcurrentDictionary<TKey, TValue> dictionary, TKey key, TValue item)
        {
            // 10/9/2011
            dictionary.AddOrUpdate(key, item, (k, i) => item);

            /*if (isChecked)
            {
                dictionary.AddOrUpdate(key, item, (k, i) => item); // 11/30/2010
            }
            else
            {
                // check if exist
                if (!dictionary.ContainsKey(key))
                {
                    return;
                }

                // 9/25/2011 - iteration count
                var counter = 0;

                // otherwise, remove from dictionary
                TValue value;
                while(!dictionary.TryRemove(key, out value))
                {
                    var message = string.Format("TryRemove iteration count is {0}", ++counter);
                    Debug.WriteLine(message);
                    Console.WriteLine(message);
                }
            }*/
        }

        // 12/6/2010
        /// <summary>
        /// Extends the base value <see cref="long"/> to now have this
        /// file size converter, which takes long value represented of bytes, and
        /// converts to the closest base value of Kb, MB, GB or TB.  This is then
        /// returned as a string for display purposes.
        /// </summary>
        /// <param name="value">this long value.</param>
        /// <returns>String representation of the value converted in Kb, MB, GB or TB.</returns>
        [CatchAllExceptionHandling(true, typeof(string), "")]
        public static string FileSizeConverter(this long value)
        {
            // retrieve given value
            var fileSize = (long)value;

            // constants
            const long kiloByte = 1024;
            const long megaByte = kiloByte * kiloByte; // 1024^2
            const long gigaByte = kiloByte * kiloByte * kiloByte; // 1024^3
            const long teraByte = megaByte * megaByte; // 1024^4

            // conversion data
            long convertedSize;
            var fileSizeOutput = string.Empty;

            // Note: Using KiloByte as 1024 bytes.
            // Convert to proper string with suffix 'Bytes, 'KB', 'MB', or 'GB'.
            if (fileSize < kiloByte)
            {
                fileSizeOutput = fileSize + " bytes";
            }
            else if (fileSize >= kiloByte && fileSize < megaByte)
            {
                convertedSize = fileSize / kiloByte;
                fileSizeOutput = convertedSize + " kB";
            }
            else if (fileSize >= megaByte && fileSize < gigaByte)
            {
                convertedSize = fileSize / megaByte;
                fileSizeOutput = convertedSize + " MB";
            }
            else if (fileSize >= gigaByte && fileSize < teraByte)
            {
                convertedSize = fileSize / gigaByte;
                fileSizeOutput = convertedSize + " GB";
            }
            else if (fileSize >= teraByte)
            {
                convertedSize = fileSize / teraByte;
                fileSizeOutput = convertedSize + " TB";
            }

            return fileSizeOutput;
        }
    }
}
