﻿using System;
using System.Reflection;
using System.ServiceModel;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;
using PostSharp.Aspects;
using PostSharp.Aspects.Advices;

// NOTE: 7/4/2011 - Moved out of the ImageCloudLock namespace to avoid applying the PostSharp Attribute 'MEFLogger'.
// ReSharper disable CheckNamespace
namespace Attributes
// ReSharper restore CheckNamespace
{
    // 12/5/2010
    [Serializable]
    public class OnWCFProxyCallTestAttribute : MethodInterceptionAspect, IInstanceScopedAspect
    {
        // 12/6/2010
        /// <summary>
        /// Set when making an asynchronous begin/end call pattern. 
        /// </summary>
        public enum AsynchronousCallEnum
        {
            /// <summary>
            /// Default - Will open and close communcation channel in same call.
            /// </summary>
            None,
            /// <summary>
            /// Keeps communication channel open.
            /// </summary>
            BeginCall,
            /// <summary>
            /// Completes asnychronous process by closing communication channel.
            /// </summary>
            EndCall
        }

        // At runtime, this field is set to a Property from the calling class, which is required.
        [ImportMember("IclDuplexChannelFactory", IsRequired = true, Order = ImportMemberOrder.BeforeIntroductions)]
        public Property<DuplexChannelFactory<IclWebCommService>> IclDuplexChannelFactory;

        // At runtime, this field is set to a Property from the calling class, which is required.
        [ImportMember("CurrentChannel", IsRequired = false, Order = ImportMemberOrder.BeforeIntroductions)]
        public Property<IclWebCommService> Channel;

        // 12/6/2010
        /// <summary>
        /// Used to set signify an asynchronous calling pattern.  When set, the
        /// communication channel will be kept open until the 'EndCall' method completes. 
        /// </summary>
        public AsynchronousCallEnum AsynchronousCallType = AsynchronousCallEnum.None;

        /// <summary>
        /// Method invoked <i>instead</i> of the method to which the aspect has been applied.
        /// </summary>
        /// <param name="methodInterceptionArgs">Advice arguments.</param>
        [DoNotUseLogger]
        public override void OnInvoke(MethodInterceptionArgs methodInterceptionArgs)
        {
            // Get Instance of Property
            var wcfComm = IclDuplexChannelFactory.Get.Invoke();
            ICommunicationObject channelCommActions = null;

            // 7/29/2011 - Check if NULLs
            if (wcfComm == null) return;
            if (methodInterceptionArgs == null) return;

            //ValidateCurrentChannelParameterExist(methodInterceptionArgs.Method);

            IclWebCommService currentChannel = wcfComm.CreateChannel();

            // 12/6/2010 - ONLY create if 'None' or 'BeginCall' states.
            if (AsynchronousCallType == AsynchronousCallEnum.None ||
                AsynchronousCallType == AsynchronousCallEnum.BeginCall)
            {
                // Create new communication Channel.
                //Channel.Set.Invoke(wcfComm.CreateChannel());
                methodInterceptionArgs.Arguments[0] = currentChannel;

            }

            // 7/29/2011 - Check if null
            // Invoke original message
            methodInterceptionArgs.Proceed();

            // 12/6/2010 - ONLY close if 'None' or 'EndCall' states.
            if (AsynchronousCallType == AsynchronousCallEnum.None ||
                AsynchronousCallType == AsynchronousCallEnum.EndCall)
            {
                // Close channel.
                channelCommActions = (ICommunicationObject)currentChannel; // Channel.Get.Invoke()
                if (channelCommActions != null) channelCommActions.Close();
#if DEBUG
                Console.WriteLine(@"WCF Channel closed successfully.");
#endif
            }
        }

        [DoNotUseLogger]
        public object CreateInstance(AdviceArgs adviceArgs)
        {
            return MemberwiseClone();
        }

        // 8/28/2011
        [DoNotUseLogger]
        public override bool CompileTimeValidate(System.Reflection.MethodBase method)
        {
            ValidateCurrentChannelParameterExist(method);

            return base.CompileTimeValidate(method);
        }

        private static void ValidateCurrentChannelParameterExist(MethodBase method)
        {
            // get method parameters
            var parameters = method.GetParameters();

            // validate at least one parameter is of the type 'IclWebCommService'.
            bool foundExpectedParameter = false;    
            bool foundExpectedParameterName = true;
            foreach (var parameterInfo in parameters)
            {
                if (parameterInfo.ParameterType.Name.Equals("IclWebCommService"))
                {
                    // Is this is valid position 0 
                    if (parameterInfo.Position != 0)
                    {
                        throw new InvalidOperationException(
                            string.Format(
                                "The parameter type 'IclWebCommService' for currentChannel MUST be at position 0 for method {0}",
                                method.Name));
                    }

                    foundExpectedParameter = true;
                }
            }

            if (!foundExpectedParameter || !foundExpectedParameterName)
            {
                throw new InvalidOperationException(
                    string.Format(
                        "At least on parameter MUST be of the type 'IclWebCommService' and named currentChannel for method {0}",
                        method.Name));
            }
        }

        [DoNotUseLogger]
        public void RuntimeInitializeInstance()
        {
            // empty
        }
    }
}
