using System;
using System.Collections.Specialized;
using System.Windows;
using System.Windows.Controls;

namespace ImageNexus.BenScharbach.ImageCloudLock.AutoScroll
{
    /// <summary>
    /// Adds the 'AutoScroll' feature to any ListBox.
    /// </summary>
    public class SelectorExtenders : DependencyObject
    {
        /// <summary>
        /// Gets the <see cref="IsAutoscrollProperty"/>.
        /// </summary>
        public static bool GetIsAutoscroll(DependencyObject obj)
        {
            return (bool)obj.GetValue(IsAutoscrollProperty);
        }

        /// <summary>
        /// Sets the <see cref="IsAutoscrollProperty"/>.
        /// </summary>
        public static void SetIsAutoscroll(DependencyObject obj, bool value)
        {
            obj.SetValue(IsAutoscrollProperty, value);
        }

        /// <summary>
        /// <see cref="IsAutoscrollProperty"/>.
        /// </summary>
        public static readonly DependencyProperty IsAutoscrollProperty =
            DependencyProperty.RegisterAttached("IsAutoscroll", typeof(bool), typeof(SelectorExtenders),
                                                new UIPropertyMetadata(default(bool), OnIsAutoscrollChanged));

        /// <summary>
        /// Callback for <see cref="IsAutoscrollProperty"/>.
        /// </summary>
        public static void OnIsAutoscrollChanged(DependencyObject dependencyObject, DependencyPropertyChangedEventArgs e)
        {
            var val = (bool)e.NewValue;
            var listBox = dependencyObject as ListBox;

            if (listBox == null)
            {
                throw new ArgumentException("Autoscroll dependency must be placed on a Listbox.");
            }

            var itemsCollection = listBox.Items;

            var autoscroller = new NotifyCollectionChangedEventHandler(
                (s1, e1) =>
                    {
                        var selectedItem = default(object);
                        switch (e1.Action)
                        {
                            case NotifyCollectionChangedAction.Add:
                            case NotifyCollectionChangedAction.Move:
                                {
                                    selectedItem = e1.NewItems[e1.NewItems.Count - 1];
                                }
                                break;
                            case NotifyCollectionChangedAction.Remove:
                                if (itemsCollection.Count < e1.OldStartingIndex)
                                {
                                    selectedItem = itemsCollection[e1.OldStartingIndex - 1];
                                }
                                else if (itemsCollection.Count > 0) 
                                {
                                    selectedItem = itemsCollection[0];
                                }
                                break;
                            case NotifyCollectionChangedAction.Reset:
                                if (itemsCollection.Count > 0) 
                                {
                                    selectedItem = itemsCollection[0];
                                }
                                break;
                        }

                        if (selectedItem == default(object))
                        {
                            return;
                        }

                        itemsCollection.MoveCurrentTo(selectedItem);
                        listBox.ScrollIntoView(selectedItem);
                    });


            var data = itemsCollection.SourceCollection as INotifyCollectionChanged;
            if (data == null)
            {
                return;
            }

            if (val)
            {
                data.CollectionChanged += autoscroller;
            }
            else
            {
                data.CollectionChanged -= autoscroller;
            }

        }
    }
}