﻿using System;
using System.IO;
using System.Text;
using System.Windows.Forms;
using ImageNexus.BenScharbach.ICLTools.StorageTools.CloudData;

namespace ImageNexus.BenScharbach.ImageCloudLock.CloudCache
{
    /// <summary>
    ///  The <see cref="CloudCacheDeleter"/> is used to delete the mirror file system that matches the online cloud backup items.
    /// </summary>
    public static class CloudCacheDeleter
    {
        // subdirectory location, within the ProgramData container.
        private const string ImageNexusCloudCache = @"\Image-Nexus\CloudCache\";

        // Location to store the 'CloudCache' data
        private static readonly string CloudCacheLocationPath = "";

         /// <summary>
        /// Constructor
        /// </summary>
        static CloudCacheDeleter()
        {
            // Retrieve ProgramData folder
            var myProgramDataPath = Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData);
            CloudCacheLocationPath = String.Concat(myProgramDataPath, ImageNexusCloudCache);
        }

        /// <summary>
        /// Deletes the given cloudItems from the cloudcache.
        /// </summary>
        /// <param name="itemsToDelete">Array of <see cref="DeleteFileCloudItem"/> to delete from cloudcache</param>
        public static void DeleteCloudCacheItems(DeleteFileCloudItem[] itemsToDelete)
        {
            if (itemsToDelete == null)
            {
                return;
            }

            if (itemsToDelete.Length == 0)
            {
                return;
            }

            try
            {
                // iterate collection to delete cloudcache items
                foreach (var deleteFileCloudItem in itemsToDelete)
                {
                    var keyName = deleteFileCloudItem.Key;

                    // skip empty records
                    if (string.IsNullOrEmpty(keyName))
                    {
                        continue;
                    }

                    // Remove end ';xxx' data
                    keyName = RemoveEndStringAtDelimeter(keyName, ";".ToCharArray());

                    // Convert directory '\\' or '/' to '\' for.
                    var convertedFilePath = keyName.Replace("\\", @"/").Replace(@"/", @"\");
                    var fullFilePath = string.Concat(CloudCacheLocationPath, convertedFilePath);

                    // 3/19/2012 - Also need to include the original extention name with the cloud fileName.
                    //           - This will avoid duplicate files that have the same name, but different extentions, 
                    //           - overriding themselves with the .txt extension.
                    var fileExtensionName = Path.GetExtension(fullFilePath);

                    // 3/19/2012 - Now tack the extension on again, since the last piece will be replaced with the '.txt'.
                    fullFilePath = string.Concat(fullFilePath, fileExtensionName);

                    // change extention to use the dummy extention .txt
                    fullFilePath = Path.ChangeExtension(fullFilePath, "txt");

                    // delete file
                    DeleteCloudCacheFile(fullFilePath);  

                    // get directory path only
                    var fullDirectoryPath = Path.GetDirectoryName(fullFilePath);

                    // skip null strings
                    if (string.IsNullOrEmpty(fullDirectoryPath))
                    {
                        continue;
                    }

                    // check if current directory exist
                    if (!Directory.Exists(fullDirectoryPath))
                    {
                        continue;
                    }

                    // *****
                    // check if current directory is empty
                    // *****

                    // split the full directory path into smaller directory pieces
                    var length = fullDirectoryPath.Split(@"\".ToCharArray(), StringSplitOptions.RemoveEmptyEntries).Length;
                    var directoryPathToCheck = fullDirectoryPath;
                    for (int i = 0; i < length; i++)
                    {
                        var files = Directory.GetFiles(directoryPathToCheck);
                        var directories = Directory.GetDirectories(directoryPathToCheck);
                        if (!(files.Length == 0 && directories.Length == 0))
                        {
                            break;
                        }

                        // deletes directory
                        DeleteCloudCacheDirectory(directoryPathToCheck);

                        var lastIndex = directoryPathToCheck.LastIndexOf(@"\", StringComparison.Ordinal);
                        var removeDirectoryLength = directoryPathToCheck.Substring(lastIndex).Length;
                        var newDirectoryLength = directoryPathToCheck.Length - removeDirectoryLength;
                        directoryPathToCheck = directoryPathToCheck.Substring(0, newDirectoryLength);
                    }
                }
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
                MessageBox.Show(e.Message, "CloudCache iteration Operation", MessageBoxButtons.OK);
            }
        }

        /// <summary>
        /// Removes the end string portion based on a given delimter.
        /// </summary>
        /// <param name="keyName"></param>
        /// <param name="delimeter"> </param>
        /// <returns></returns>
        private static string RemoveEndStringAtDelimeter(string keyName, char[] delimeter)
        {
            var splitItems = keyName.Split(delimeter, StringSplitOptions.RemoveEmptyEntries);
            var stringBuildKeyName = new StringBuilder();
            for (int index = 0; index < splitItems.Length - 1; index++)
            {
                stringBuildKeyName.Append(splitItems[index]);
            }
            keyName = stringBuildKeyName.ToString();
            return keyName;
        }

        /// <summary>
        /// Deletes a cloudcache directory
        /// </summary>
        /// <param name="directoryPath"> </param>
        private static void DeleteCloudCacheDirectory(string directoryPath)
        {
            try
            {
                // then delete current directory
                Directory.Delete(directoryPath);
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
                MessageBox.Show(e.Message, "File Delete Operation", MessageBoxButtons.OK);
            }
        }

        /// <summary>
        /// Deletes a cloudcache file.
        /// </summary>
        /// <param name="filePath"> </param>
        private static void DeleteCloudCacheFile(string filePath)
        {
            try
            {
                // delete the file
                File.Delete(filePath);
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
                MessageBox.Show(e.Message, "File Delete Operation", MessageBoxButtons.OK);
            }
        }
    }
}