﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Threading;
using ImageCloudLock;
using ImageNexus.BenScharbach.Common.FileProcessor;
using ImageCloudLock.TaskActions;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.CloudDataModel.DirectoryDataModel;
using ImageNexus.BenScharbach.ImageCloudLock.TaskActions;

namespace ImageNexus.BenScharbach.ImageCloudLock.CloudCache
{
    // 3/15/2012
    /// <summary>
    /// The <see cref="CloudCacheRetriever"/> is used to retrieve the mirror file system that matches the online cloud backup items.
    /// </summary>
    /// <remarks>The 'CloudCacheCreator' resides in the WCF service.</remarks>
    public static class CloudCacheRetriever
    {
        // subdirectory location, within the ProgramData container.
        private const string ImageNexusCloudCache = @"\Image-Nexus\CloudCache\";

        // Location to store the 'CloudCache' data
        private static readonly string CloudCacheLocationPath = "";

        /// <summary>
        /// Constructor
        /// </summary>
        static CloudCacheRetriever()
        {
            // Retrieve ProgramData folder
            var myProgramDataPath = Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData);
            CloudCacheLocationPath = String.Concat(myProgramDataPath, ImageNexusCloudCache);
        }

        /// <summary>
        /// Gets the root directories in the 'cloudcache' container.
        /// </summary>
        /// <returns>Simple array of directories as strings.</returns>
        public static bool GetCloudRootDirectories(out string[] rootDirectories)
        {
            rootDirectories = new string[0];

            try
            {
                // Read root directories in the 'cloudcache'
                rootDirectories = Directory.GetDirectories(CloudCacheLocationPath);

                // Parse the 'cloudcache' directory from paths
                var length = ParseCloudCacheNameForRoot(rootDirectories);

                return true;
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
                return false;
            }
        }

        /// <summary>
        /// Gets the sub-directories for the given directory, and returns a collection of <see cref="IDirectoryItem"/>.
        /// </summary>
        /// <param name="directoryFullPathName">Directory full path name</param>
        /// <param name="restoreCloudWindow">Instance of <see cref="RestoreCloudWindow"/></param>
        /// <param name="directoryItemViewModel">Instance of <see cref="IDirectoryItemViewModel"/></param>
        /// <param name="subDirectories">(OUT) Collection of <see cref="IDirectoryItem"/></param>
        /// <returns>true/false of result</returns>
        public static bool GetCloudDirectoriesForDirectory(string directoryFullPathName,
                                                           RestoreCloudWindow restoreCloudWindow,
                                                           IDirectoryItemViewModel directoryItemViewModel, 
                                                           out List<IDirectoryItem> subDirectories)
        {
            subDirectories = new List<IDirectoryItem>();

            try
            {
                // Convert directory '\\' or '/' to '\' for.
                var convertedDirectoryPath = directoryFullPathName.Replace("\\", @"/").Replace(@"/", @"\");
                var fullDirectoryPath = string.Concat(CloudCacheLocationPath, convertedDirectoryPath);

                // Read sub-directories for given directory path
                var subDirectoriesCollection = Directory.GetDirectories(fullDirectoryPath);

                // Parse the 'cloudcache' directory from paths 
                ParseCloudCacheName(subDirectoriesCollection);

                // Convert directory '\\' or '\' to '/'.
                ConvertCloudCacheName(subDirectoriesCollection);

                // Convert to collection of IDirectoryItem
                subDirectories = DoRetrieveDirectoriesTask.DirectoryCloudItems(fullDirectoryPath,
                                                                               subDirectoriesCollection,
                                                                               restoreCloudWindow,
                                                                               directoryItemViewModel,
                                                                               @"/".ToCharArray());

                return true;
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
                return false;
            }
        }

        /// <summary>
        /// Gets the files in the 'cloudcache' for the given directory name.
        /// </summary>
        /// <param name="directoryFullPathName">Directory full path name</param>
        /// <param name="restoreCloudWindow">Instance of <see cref="RestoreCloudWindow"/></param>
        /// <param name="cloudCacheFiles">(OUT) Collection of <see cref="IFileItem"/></param>
        /// <returns>true/false of result</returns>
        public static bool GetCloudFilesForDirectory(string directoryFullPathName, RestoreCloudWindow restoreCloudWindow, out List<IFileItem> cloudCacheFiles)
        {
            cloudCacheFiles = new List<IFileItem>();

            try
            {
                // Convert directory '\\' or '/' to '\' for.
                var convertedDirectoryPath = directoryFullPathName.Replace("\\", @"/").Replace(@"/", @"\");
                var fullDirectoryPath = string.Concat(CloudCacheLocationPath, convertedDirectoryPath);

                // Retrieve files for given directory
                var filesInDirectory = Directory.GetFiles(fullDirectoryPath);

                var filesCount = filesInDirectory.Length;

                string message = String.Format("Retrieving {0} files for directory {1}", filesCount, directoryFullPathName);
                restoreCloudWindow.SetFooterMessage(message);

                // Reset Footer's mini-progressBar; 3/2/2012 - Minus, since 0 base array length.
                restoreCloudWindow._footerProgressBarFiles.ResetFooterProgressBar(filesCount - 1); 

                // convert to 'FileCloudItem'.
                cloudCacheFiles = new List<IFileItem>(filesCount);
                var counter = 0.0f;
                foreach (var fileName in filesInDirectory)
                {
                    if (string.IsNullOrEmpty(fileName)) continue;

                    // Deserialize the XML file and retrieve the metadata.
                    CloudCacheMetaData data;
                    Processor.LoadXmlSerializedStream(fileName, out data);

                    // Note: Field 'fileFullName' is the 'filePath'.
                    cloudCacheFiles.Add(new FileCloudItem(data.FinalKeyName, data.FileName, data.FilePath,
                                                         data.DirectoryName, "", Convert.ToInt64(data.FileSize),
                                                         data.LastWriteTime));

                    // Updated Footer's progress.
                    restoreCloudWindow._footerProgressBarFiles.SetFooterProgress(counter++);

                    // Sleep for a few ms to allow UI update.
                    Thread.Sleep(1);
                }

                return true;
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
                return false;
            }
        }

       

        /// <summary>
        /// Converts directoryNames with delimeter's '\\' or '\' to '/'
        /// </summary>
        /// <param name="directories">Collection fo directories to convert</param>
        private static void ConvertCloudCacheName(string[] directories)
        {
            var length = directories.Length;
            for (int index = 0; index < length; index++)
            {
                var directoryName = directories[index];

                // Convert directory '\\' or '\' to '/'.
                directories[index] = directoryName.Replace("\\", @"\").Replace(@"\", "/");
            }
        }

        /// <summary>
        /// Parses out the 'cloudcache' name from the beginning of each directory name.
        /// </summary>
        /// <param name="directories">Collection of directories to parse.</param>
        /// <returns>Length of collection</returns>
        private static int ParseCloudCacheName(string[] directories)
        {
            // split cloudcache directory name
            var splitCloudCache = CloudCacheLocationPath.Split(@"\".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);

            var length = directories.Length;
            for (int index = 0; index < length; index++)
            {
                var directoryName = directories[index];

                // Split directories into array
                var splitItems = directoryName.Split(@"\".ToCharArray(), StringSplitOptions.RemoveEmptyEntries);

                // Create final directoryName
                var stringBuilderName = new StringBuilder();
                for (int i = splitCloudCache.Length; i < splitItems.Length; i++)
                {
                    var splitItem = splitItems[i];

                    stringBuilderName.Append(splitItem);
                    stringBuilderName.Append("/");
                }

                // Set new name
                directories[index] = stringBuilderName.ToString();
            }
            return length;
        }

        /// <summary>
        /// Parses out the final directoryName from the full 'cloudcache' directory name.
        /// </summary>
        /// <param name="directories">Collection of directories to parse.</param>
        /// <returns>Length of collection</returns>
        private static int ParseCloudCacheNameForRoot(string[] directories)
        {
            var length = directories.Length;
            for (int index = 0; index < length; index++)
            {
                var directoryName = directories[index];

                // Split directories into array
                var splitItems = directoryName.Split(@"\".ToCharArray());
                // Take final item in array
                directories[index] = splitItems[splitItems.Length - 1] + "/";
            }
            return length;
        }
    }
}