﻿using System;
using System.Diagnostics;
using System.ServiceModel;
using System.Windows;
using System.Windows.Navigation;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using Attributes;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.ICLTools.Amazon.LS;
using ImageNexus.BenScharbach.ICLTools.Amazon.LS.Amazon.LS;
using ImageNexus.BenScharbach.ICLTools.CredentialStore.Security;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;

namespace ImageNexus.BenScharbach.ImageCloudLock.CloudSetups
{
    /// <summary>
    /// Interaction logic for SetAmazonDevPay.xaml
    /// </summary>
    public partial class SetAmazonDevPay : Window
    {
        // 1/17/2012 - CredentialStore for Amazon
        private static AmazonFileCredentialStore _amazonFileCredentialStore;

        // Location to save the AmazonFileCredentialStore info.
        private const string DesktopCredentialsFileName = "DevPayDesktopCredentialsCSharp.xml";

        #region Properties

        /// <summary>
        /// Gets an instance of the <see cref="AmazonFileCredentialStore"/>.
        /// </summary>
        public static AmazonFileCredentialStore AmazonFileCredentialStore
        {
            get
            {
                return _amazonFileCredentialStore ??
                       (_amazonFileCredentialStore =
                        new AmazonFileCredentialStore(IclPid.FilePassword, DesktopCredentialsFileName));
            }
        }

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; private set; }

        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{IclWebCommService}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // 12/6/2010
        /// <summary>
        /// Gets the current communication channel.
        /// </summary>
        public IclWebCommService CurrentChannel { get; set; }

        // 12/6/2010
        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        /// <summary>
        /// constructor
        /// </summary>
        /// <param name="mainWindow"></param>
        public SetAmazonDevPay(MainWindow mainWindow)
        {
            InitializeComponent();

            // Save instance of MainWindow
            MainWindowParent = mainWindow;
            
        }

        // 4/11/2011
        [CatchAllExceptionHandling(false, null, null)]
        private void btnApply_Click(object sender, RoutedEventArgs e)
        {
            // Retrieve the Customer Application Key
            var activationKey = txtProductActKey.Text;

            // Check if values are null or empty.
            if (String.IsNullOrEmpty(activationKey))
            {
                txtErrorMessages.Text = "Your 'Activation Key' CANNOT be left blank.";
                txtProductActKey.Focus();
                return;
            }

            // Activate Desktop project and retrieve user token and S3 Acct info.
            const string productToken = IclPid.PToken;
            var amazonLsConfig = new AmazonLSConfig();
            var userCredentials = Services.ActivateDesktopProduct(activationKey, productToken, amazonLsConfig);

            // Store User data as backup
            var credentialStore = new AmazonFileCredentialStore(IclPid.FilePassword, DesktopCredentialsFileName)
            {
                UserToken = userCredentials.UserToken,
                AccessKeyId = userCredentials.AccessKeyId,
                SecretAccessKey = userCredentials.SecretAccessKey
            };
            credentialStore.Store();

            // Create collection of Registry keys to pass to WCF service
            CreateCloudRegKeys(userCredentials);
        }

        // 1/25/2012
        /// <summary>
        /// Create cloud registry keys
        /// </summary>
        /// <param name="userCredentials">Amazon user credentials</param>
        private void CreateCloudRegKeys(Credentials userCredentials)
        {
            var cloudAccountRegistry = new WcfCloudAccountRegistry
                                           {
                                               WcfCloudAccountRegistryKeys = new WcfCloudAccountRegistryKeys[3]
                                           };

            // Create Each Registry Record
            cloudAccountRegistry.WcfCloudAccountRegistryKeys[0] = new WcfCloudAccountRegistryKeys
                                                                      {
                                                                          RegistryName = CloudAcctRegistryEnums.AmazonAccessKey,
                                                                          RegistryValue = userCredentials.AccessKeyId
                                                                      };

            cloudAccountRegistry.WcfCloudAccountRegistryKeys[1] = new WcfCloudAccountRegistryKeys
                                                                      {
                                                                          RegistryName =
                                                                              CloudAcctRegistryEnums.AmazonSecretAccessKey,
                                                                          RegistryValue = userCredentials.SecretAccessKey
                                                                      };

            cloudAccountRegistry.WcfCloudAccountRegistryKeys[2] = new WcfCloudAccountRegistryKeys
                                                                      {
                                                                          RegistryName = CloudAcctRegistryEnums.AmazonUserToken,
                                                                          RegistryValue = userCredentials.UserToken
                                                                      };

            // Note: 1st param can be null, since it will be set by postsharp.
            if (DoWCFSetAmazonAccountRegistryKeys(null, cloudAccountRegistry))
            {
                return;
            }

            // Show MessageBox
            MessageBox.Show(
                "Account Activation Failed.  Verify ImageCloudLock service is running/or internet service is active and try again.",
                "Account Activation", MessageBoxButton.OK,
                MessageBoxImage.Error, MessageBoxResult.OK);
        }

        // 8/28/2011
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private bool DoWCFSetAmazonAccountRegistryKeys(IclWebCommService currentChannel, WcfCloudAccountRegistry cloudAccountRegistry)
        {
            // Update reg key for 'AccessKey'.
            if (currentChannel.SetCloudAccountRegistryKeys(cloudAccountRegistry))
            {
                // Show MessageBox
                MessageBox.Show("Your AmazonS3 account information was created successfully.", "Account Activation",
                                MessageBoxButton.OK,
                                MessageBoxImage.Information, MessageBoxResult.OK);

                // 4/29/2011 - Return to avoid failed message
                return true;
            }

            return false;
        }

        // 4/29/2011
        /// <summary>
        /// Helper method to open up a hyperlink in a browser, allowing user to get their AmazonS3 activation key.
        /// </summary>
        private void DevPayActivationKeyHyperLink_RequestNavigate(object sender, RequestNavigateEventArgs e)
        {
            Process.Start(new ProcessStartInfo(e.Uri.AbsoluteUri));
            e.Handled = true;
        }
    }
}
