﻿using System;
using System.ServiceModel;
using System.Windows;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using Attributes;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;

namespace ImageNexus.BenScharbach.ImageCloudLock.CloudSetups
{
    /// <summary>
    /// Interaction logic for SetAmazonS3AcctInfo.xaml
    /// </summary>
    public partial class SetAmazonS3AcctInfo : Window
    {
        private string _amazonSecretAccessKey;

        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; private set; }

        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{IclWebCommService}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // 12/6/2010
        /// <summary>
        /// Gets the current communication channel.
        /// </summary>
        public IclWebCommService CurrentChannel { get; set; }

        // 12/6/2010
        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="MainWindow"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public SetAmazonS3AcctInfo(MainWindow mainWindow)
        {
            InitializeComponent();

            // Save instance of MainWindow
            MainWindowParent = mainWindow;

            // Retrieve prior AmazonS3 account information, if any.
            GetAmazonS3AcctInfo();
           
        }

        /// <summary>
        /// Retrieves the Amazon account keys from the registry.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void GetAmazonS3AcctInfo()
        {
            // Create collection of Registry keys to pass to WCF service
            var cloudAccountRegistry = new WcfCloudAccountRegistry
            {
                WcfCloudAccountRegistryKeys = new WcfCloudAccountRegistryKeys[2]
            };

            // Create Each Registry Record
            cloudAccountRegistry.WcfCloudAccountRegistryKeys[0] = new WcfCloudAccountRegistryKeys { RegistryName = CloudAcctRegistryEnums.AmazonAccessKey };
            cloudAccountRegistry.WcfCloudAccountRegistryKeys[1] = new WcfCloudAccountRegistryKeys { RegistryName = CloudAcctRegistryEnums.AmazonSecretAccessKey };

            // Note: 1st param can be null, since it will be set by postsharp.
            cloudAccountRegistry = DoWCFGetAmazonAccountRegistryKeys(null, cloudAccountRegistry);

            // Set Values
            txtAmazonAccessKey.Text = (string)cloudAccountRegistry.WcfCloudAccountRegistryKeys[0].RegistryValue;
            _amazonSecretAccessKey = (string)cloudAccountRegistry.WcfCloudAccountRegistryKeys[1].RegistryValue;
        }

        // 8/28/2011
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private WcfCloudAccountRegistry DoWCFGetAmazonAccountRegistryKeys(IclWebCommService currentChannel, WcfCloudAccountRegistry cloudAccountRegistry)
        {
            // Retrieve current account info, if any, and populate fields.
            cloudAccountRegistry = currentChannel.GetCloudAccountRegistryKeys(cloudAccountRegistry);
            return cloudAccountRegistry;
        }

        /// <summary>
        /// Occurs when Amazon AccessKey got focus.
        /// </summary>
        private void txtAmazonAccessKey_GotFocus(object sender, RoutedEventArgs e)
        {
            //txtAmazonAccessKey.BorderBrush = Brushes.DarkRed;
        }

        /// <summary>
        /// Occurs when Amazon AccessKey got focus.
        /// </summary>
        private void txtAmazonAccessKey_LostFocus(object sender, RoutedEventArgs e)
        {
            
        }

        /// <summary>
        /// Occurs when Amazon SecretAccessKey got focus.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void txtAmazonSecretAccessKey_GotFocus(object sender, RoutedEventArgs e)
        {
            // Set Secret Access Key 
            txtAmazonSecretAccessKey.Text = _amazonSecretAccessKey;

        }

        /// <summary>
        /// Occurs when Amazon AccessKey got focus.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void txtAmazonSecretAccessKey_LostFocus(object sender, RoutedEventArgs e)
        {
            // Retrieve value set by user
            _amazonSecretAccessKey = txtAmazonSecretAccessKey.Text;

            // Remove the Secrete Access Key
            txtAmazonSecretAccessKey.Text = "*****************************************";

        }

        /// <summary>
        /// Occurs when the Apply button is pressed, which sets the AmazonS3 account info
        /// into the registry.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void btnApply_Click(object sender, RoutedEventArgs e)
        {
            // Retrieve the accessKey
            var accessKey = txtAmazonAccessKey.Text;
            // Retrieve the secretAccessKey
            var secretAccessKey = _amazonSecretAccessKey;

            // Check if values are null or empty.
            if (string.IsNullOrEmpty(accessKey))
            {
                txtErrorMessages.Text = "AmazonS3 'AccessKey' CANNOT be left blank.";
                txtAmazonAccessKey.Focus();
                return;
            }

            // Check if values are null or empty.
            if (string.IsNullOrEmpty(secretAccessKey))
            {
                txtErrorMessages.Text = "AmazonS3 'SecretAccessKey' CANNOT be left blank.";
                txtAmazonSecretAccessKey.Focus();
                return;
            }

            // Create collection of Registry keys to pass to WCF service
            var cloudAccountRegistry = new WcfCloudAccountRegistry
                                           {
                                               WcfCloudAccountRegistryKeys = new WcfCloudAccountRegistryKeys[2]
                                           };
            // Create Each Registry Record
            cloudAccountRegistry.WcfCloudAccountRegistryKeys[0] = new WcfCloudAccountRegistryKeys { RegistryName = CloudAcctRegistryEnums.AmazonAccessKey, RegistryValue = accessKey };
            cloudAccountRegistry.WcfCloudAccountRegistryKeys[1] = new WcfCloudAccountRegistryKeys { RegistryName = CloudAcctRegistryEnums.AmazonSecretAccessKey, RegistryValue = secretAccessKey };

            // Note: 1st param can be null, since it will be set by postsharp.
            if (DoWCFSetAmazonAccountRegistryKeys(null, cloudAccountRegistry)) return;

            // Show MessageBox
            MessageBox.Show("Save Failed.  Verify ImageCloudLock service is running and try again.", "Account Information", MessageBoxButton.OK,
                            MessageBoxImage.Error, MessageBoxResult.OK);
            
        }

        // 8/28/2011
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private bool DoWCFSetAmazonAccountRegistryKeys(IclWebCommService currentChannel, WcfCloudAccountRegistry cloudAccountRegistry)
        {
            // Update reg key for 'AccessKey'.
            if (currentChannel.SetCloudAccountRegistryKeys(cloudAccountRegistry))
            {
                // Show MessageBox
                MessageBox.Show("Your AmazonS3 account information saved successfully.", "Account Information",
                                MessageBoxButton.OK,
                                MessageBoxImage.Information, MessageBoxResult.OK);
                return true;
            }

            return false;
        }
    }
}
