﻿using System;
using System.ServiceModel;
using System.Windows;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using Attributes;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.ICLTools.CredentialStore.Security;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;

namespace ImageNexus.BenScharbach.ImageCloudLock.CloudSetups
{
    /// <summary>
    /// Interaction logic for SetAzureAcctInfo.xaml
    /// </summary>
    public partial class SetAzureAcctInfo : Window
    {
        // 1/17/2012 - CredentialStore for Azure
        private static AzureFileCredentialStore _azureFileCredentialStore;

        // Location to save the AzureFileCredentialStore info.
        private const string DesktopCredentialsFileName = "AzureDesktopCredentialsCSharp.xml";

        private string _azureSecretAccessKey;

        #region Properties

        /// <summary>
        /// Gets an instance of the <see cref="AzureFileCredentialStore"/>.
        /// </summary>
        public static AzureFileCredentialStore AzureFileCredentialStore
        {
            get
            {
                return _azureFileCredentialStore ??
                       (_azureFileCredentialStore =
                        new AzureFileCredentialStore(IclPid.FilePassword, DesktopCredentialsFileName));
            }
        }


        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; private set; }

        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{IclWebCommService}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // 12/6/2010
        /// <summary>
        /// Gets the current communication channel.
        /// </summary>
        public IclWebCommService CurrentChannel { get; set; }

        // 12/6/2010
        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="MainWindow"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public SetAzureAcctInfo(MainWindow mainWindow)
        {
            InitializeComponent();

            // Save instance of MainWindow
            MainWindowParent = mainWindow;

            // Retrieve prior Azure account information, if any.
            GetAzureAcctInfo();
        }

        /// <summary>
        /// Retrieves the Azure account keys from the registry.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void GetAzureAcctInfo()
        {
            // Create collection of Registry keys to pass to WCF service
            var cloudAccountRegistry = new WcfCloudAccountRegistry
            {
                WcfCloudAccountRegistryKeys = new WcfCloudAccountRegistryKeys[2]
            };

            // Create Each Registry Record
            cloudAccountRegistry.WcfCloudAccountRegistryKeys[0] = new WcfCloudAccountRegistryKeys { RegistryName = CloudAcctRegistryEnums.AzureAccountName };
            cloudAccountRegistry.WcfCloudAccountRegistryKeys[1] = new WcfCloudAccountRegistryKeys { RegistryName = CloudAcctRegistryEnums.AzureSecretAccessKey };

            // Note: 1st param can be null, since it will be set by postsharp.
            cloudAccountRegistry = DoWCFGetAzureAccountRegistryKeys(null, cloudAccountRegistry);

            // Set Values
            txtAzureAccountName.Text = (string)cloudAccountRegistry.WcfCloudAccountRegistryKeys[0].RegistryValue;
            _azureSecretAccessKey = (string)cloudAccountRegistry.WcfCloudAccountRegistryKeys[1].RegistryValue;
        }

        
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private WcfCloudAccountRegistry DoWCFGetAzureAccountRegistryKeys(IclWebCommService currentChannel, WcfCloudAccountRegistry cloudAccountRegistry)
        {
            // Retrieve current account info, if any, and populate fields.
            cloudAccountRegistry = currentChannel.GetCloudAccountRegistryKeys(cloudAccountRegistry);
            return cloudAccountRegistry;
        }

        /// <summary>
        /// Occurs when Azure AccountName got focus.
        /// </summary>
        private void txtAzureAccountName_GotFocus(object sender, RoutedEventArgs e)
        {
            //txtAmazonAccessKey.BorderBrush = Brushes.DarkRed;
        }

        /// <summary>
        /// Occurs when Azure AccountName lost focus.
        /// </summary>
        private void txtAzureAccountName_LostFocus(object sender, RoutedEventArgs e)
        {
            // Empty
        }

        /// <summary>
        /// Occurs when Azure SecretAccessKey got focus.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void txtAzureSecretAccessKey_GotFocus(object sender, RoutedEventArgs e)
        {
            // Set Secret Access Key 
            txtAzureSecretAccessKey.Text = _azureSecretAccessKey;

        }

        /// <summary>
        /// Occurs when Azure SecretAccessKey got focus.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void txtAzureSecretAccessKey_LostFocus(object sender, RoutedEventArgs e)
        {
            // Retrieve value set by user
            _azureSecretAccessKey = txtAzureSecretAccessKey.Text;

            // Remove the Secrete Access Key
            txtAzureSecretAccessKey.Text = "*****************************************";

        }

        /// <summary>
        /// Occurs when the Apply button is pressed, which sets the Azure account info
        /// into the registry.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void btnApply_Click(object sender, RoutedEventArgs e)
        {
            // Retrieve the account name
            var accountName = txtAzureAccountName.Text;
            // Retrieve the secretAccessKey
            var secretAccessKey = _azureSecretAccessKey;

            // Check if values are null or empty.
            if (string.IsNullOrEmpty(accountName))
            {
                txtErrorMessages.Text = "Azure 'AccountName' CANNOT be left blank.";
                txtAzureAccountName.Focus();
                return;
            }

            // Check if values are null or empty.
            if (string.IsNullOrEmpty(secretAccessKey))
            {
                txtErrorMessages.Text = "Azure 'SecretAccessKey' CANNOT be left blank.";
                txtAzureSecretAccessKey.Focus();
                return;
            }

            // Store User data as backup
            var credentialStore = new AzureFileCredentialStore(IclPid.FilePassword, DesktopCredentialsFileName)
            {
                AccountName = accountName,
                SecretAccessKey = secretAccessKey
            };
            credentialStore.Store();

            // Create collection of Registry keys to pass to WCF service
            CreateCloudRegKeys(accountName, secretAccessKey);
        }

        // 1/25/2012
        /// <summary>
        /// Create cloud registry keys
        /// </summary>
        /// <param name="accountName">Azure account name</param>
        /// <param name="secretAccessKey">Azure secret access key</param>
        private void CreateCloudRegKeys(string accountName, string secretAccessKey)
        {
            var cloudAccountRegistry = new WcfCloudAccountRegistry
                                           {
                                               WcfCloudAccountRegistryKeys = new WcfCloudAccountRegistryKeys[2]
                                           };

            // Create Each Registry Record
            cloudAccountRegistry.WcfCloudAccountRegistryKeys[0] = new WcfCloudAccountRegistryKeys
                                                                      {
                                                                          RegistryName = CloudAcctRegistryEnums.AzureAccountName,
                                                                          RegistryValue = accountName
                                                                      };

            cloudAccountRegistry.WcfCloudAccountRegistryKeys[1] = new WcfCloudAccountRegistryKeys
                                                                      {
                                                                          RegistryName =
                                                                              CloudAcctRegistryEnums.AzureSecretAccessKey,
                                                                          RegistryValue = secretAccessKey
                                                                      };

            // Note: 1st param can be null, since it will be set by postsharp.
            if (DoWCFSetAzureAccountRegistryKeys(null, cloudAccountRegistry))
            {
                return;
            }

            // Show MessageBox
            MessageBox.Show("Save Failed.  Verify ImageCloudLock service is running and try again.", "Account Information",
                            MessageBoxButton.OK,
                            MessageBoxImage.Error, MessageBoxResult.OK);
        }

        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private bool DoWCFSetAzureAccountRegistryKeys(IclWebCommService currentChannel, WcfCloudAccountRegistry cloudAccountRegistry)
        {
            // Update reg key for 'AccessKey'.
            if (currentChannel.SetCloudAccountRegistryKeys(cloudAccountRegistry))
            {
                // Show MessageBox
                MessageBox.Show("Your Azure account information saved successfully.", "Account Information",
                                MessageBoxButton.OK,
                                MessageBoxImage.Information, MessageBoxResult.OK);
                return true;
            }

            return false;
        }
    }
}
