﻿using System.Diagnostics;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Controls;
using System.Windows.Threading;
using ImageCloudLock;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl;

namespace ImageNexus.BenScharbach.ImageCloudLock
{
    // 8/8/2011
    internal class DirectoryTreeViewTasks
    {
        // 1/23/2012 - Used to update threading model for this class.
        private const bool _useThreadingModel = false;
        internal static volatile bool CancelDirTreeViewTask;

        /// <summary>
        /// Creates an instance of the <see cref="DirectoryTreeViewModelAdapter"/>.
        /// </summary>
        /// <param name="directoryTreeViewTaskParams">Instance of <see cref="DirectoryTreeViewTaskParams"/>.</param>
        internal static void CreateDirectoryTreeViewTask(DirectoryTreeViewTaskParams directoryTreeViewTaskParams)
        {
            if (_useThreadingModel)
            {
                Task.Factory.StartNew(DirectoryTreeViewAction, directoryTreeViewTaskParams);
                return;
            }

            DirectoryTreeViewAction(directoryTreeViewTaskParams);
        }

        /// <summary>
        /// Creates an instance of the <see cref="DirectoryCloudTreeViewTaskParams"/>.
        /// </summary>
        /// <param name="directoryTreeViewTaskParams">Instance of <see cref="DirectoryCloudTreeViewTaskParams"/>.</param>
        internal static void CreateDirectoryCloudTreeViewTask(DirectoryCloudTreeViewTaskParams directoryTreeViewTaskParams)
        {
            if (_useThreadingModel)
            {
                Task.Factory.StartNew(DirectoryCloudTreeViewAction, directoryTreeViewTaskParams);
                return;
            }

            DirectoryCloudTreeViewAction(directoryTreeViewTaskParams);
        }

        // 8/6/2011
        internal class DirectoryTreeViewTaskParams
        {
            public MainWindow MainWindow;
            public TreeViewUserControl TreeViewUserControl;
            public ListView ListViewInstance;
            public TreeView TreeViewInstance;
        }

        // 8/11/2011
        internal class DirectoryCloudTreeViewTaskParams
        {
            public RestoreCloudWindow RestoreCloudWindow;
            public TreeViewUserControl TreeViewUserControl;
            public ListView ListViewInstance;
            public TreeView TreeViewInstance;
            public string[] Directories;
        }

        // 8/6/2011
        /// <summary>
        /// Creates an instance of the <see cref="DirectoryTreeViewModelAdapter"/>.
        /// </summary>
        /// <param name="obj"></param>
        private static void DirectoryTreeViewAction(object obj)
        {
            var directoryTreeViewTaskParams = (DirectoryTreeViewTaskParams) obj;

            // cache
            MainWindow mainWindow = directoryTreeViewTaskParams.MainWindow;

            // 11/7/2010 - Instantiate the directory TreeView adapter and bind to treeView control.
            mainWindow._directoryTreeViewModelAdapter =
                new DirectoryTreeViewModelAdapter(mainWindow,
                                                  directoryTreeViewTaskParams.TreeViewUserControl,
                                                  directoryTreeViewTaskParams.ListViewInstance,
                                                  directoryTreeViewTaskParams.TreeViewInstance);

            // 11/14/2010 - Set Footer ProgressBar callbacks
            mainWindow._directoryTreeViewModelAdapter.SetProgressBarResetCallback(mainWindow.ResetFooterProgressBar);
            mainWindow._directoryTreeViewModelAdapter.SetProgressBarValueCallback(mainWindow.SetFooterProgress);
            mainWindow._directoryTreeViewModelAdapter.SetEndProgressBarCallback(mainWindow.EndFooterProgressBar);
            mainWindow._directoryTreeViewModelAdapter.SetFooterHeaderTextCallback(mainWindow.SetFooterHeaderText);

            if (!_useThreadingModel)
            {
                return;
            }

            while (!CancelDirTreeViewTask)
            {
                Thread.Sleep(10);
                Dispatcher.Run();
            }
        }

        /// <summary>
        /// Creates an instance of the <see cref="DirCloudTreeViewModelAdapter"/>.
        /// </summary>
        /// <param name="obj"></param>
        private static void DirectoryCloudTreeViewAction(object obj)
        {
            var directoryTreeViewTaskParams = (DirectoryCloudTreeViewTaskParams) obj;

            // cache
            var restoreCloudWindow = directoryTreeViewTaskParams.RestoreCloudWindow;
           
            // 11/8/2010 - Instantiate the cloud directory TreeView adapter and bind to treeView control.
            restoreCloudWindow._dirCloudTreeViewModelAdapter =
                new DirCloudTreeViewModelAdapter(restoreCloudWindow.MainWindowParent, directoryTreeViewTaskParams.TreeViewUserControl,
                                                 directoryTreeViewTaskParams.ListViewInstance,
                                                 directoryTreeViewTaskParams.TreeViewInstance,
                                                 directoryTreeViewTaskParams.Directories);

            // Set Footer ProgressBar callbacks
            restoreCloudWindow._dirCloudTreeViewModelAdapter.SetProgressBarResetCallback(
                restoreCloudWindow._footerProgressBarDirectories.ResetFooterProgressBarCallback);
            restoreCloudWindow._dirCloudTreeViewModelAdapter.SetProgressBarValueCallback(
                restoreCloudWindow._footerProgressBarDirectories.SetFooterProgressCallback);
            restoreCloudWindow._dirCloudTreeViewModelAdapter.SetEndProgressBarCallback(
                restoreCloudWindow._footerProgressBarDirectories.EndFooterProgressBarCallback);
            restoreCloudWindow._dirCloudTreeViewModelAdapter.SetFooterHeaderTextCallback(
                restoreCloudWindow.SetFooterHeaderTextCallback);

            if (!_useThreadingModel)
            {
                return;
            }

            while (!CancelDirTreeViewTask)
            {
                Thread.Sleep(10);
                Dispatcher.Run();
            }
        }

        // 8/17/2011
        public static void PauseAndPump(long pauseFor, Dispatcher dispatcher)
        {
            var stopWatch = new Stopwatch();
            stopWatch.Start();

            while (stopWatch.ElapsedMilliseconds <= pauseFor)
            {
                DoEvents(dispatcher);
                Thread.Sleep(1);
            }
            
        }

        // 8/6/2011
        private static void DoEvents(Dispatcher currentDispatcher)
        {
            var f = new DispatcherFrame();
            currentDispatcher.Invoke(DispatcherPriority.Background,
                                          (SendOrPostCallback)delegate(object arg)
                                          {
                                              var fr = arg as DispatcherFrame;

                                              fr.Continue = false;

                                          }, f);
            Dispatcher.PushFrame(f);
        }

    }
}
