﻿using System;
using System.IO;
using System.Linq;
using System.ServiceModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using Attributes;
using ImageCloudLock;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.Common.FileProcessor;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;
using ImageNexus.BenScharbach.ImageCloudLock.Xml;

namespace ImageNexus.BenScharbach.ImageCloudLock
{
    /// <summary>
    /// Interaction logic for FilesOperationResultsWindow.xaml
    /// </summary>
    public partial class FilesOperationResultsWindow : Window
    {
        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; private set; }
     
        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{TChannel}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        /// <summary>
        /// Gets the current communication channel.
        /// </summary>
        public IclWebCommService CurrentChannel { get; set; }
       
        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        /// <summary>
        /// Constructor
        /// </summary>
        public FilesOperationResultsWindow(MainWindow mainWindow)
        {
            InitializeComponent();

            // Store mainWindow
            MainWindowParent = mainWindow;

            // Populates the listView collection with the results from the current file operation
            PopulateListCollection();
        }

        /// <summary>
        /// Populates the WPF list collection in the window with last current log's file results.
        /// </summary>
        private void PopulateListCollection()
        {
            try
            {
                // Get current log's fileName
                // Note: 1st param can be null, since it will be set by postsharp.
                var currentLogFileName = DoWCFGetCurrentLogFileName(null);

                // check if null or emtpy
                if (string.IsNullOrEmpty(currentLogFileName))
                {
                    return;
                }

                // change extention to .xml
                currentLogFileName = Path.ChangeExtension(currentLogFileName, "xml");

                // Note: This should match the output location set in the MEFLoffer's XMLLogger class.
                // Retrieve ProgramData folder
                var myProgramDataPath = Environment.GetFolderPath(Environment.SpecialFolder.CommonApplicationData);
                var logOutputLocation = string.Concat(myProgramDataPath, @"\\Image-Nexus\ICL_Service_Logs\XmlLogs\");

                // load the log's xml file
                XmlWrapper<XmlLogFileOutputStructure> xmlFilesResultCollection;
                Processor.LoadXmlSerializedStream(currentLogFileName, logOutputLocation, out xmlFilesResultCollection);

                // Add columns to listview
                var gridView = new GridView();

                // Create 'Results' column
                var visualTree = new FrameworkElementFactory(typeof (TextBlock));
                visualTree.SetBinding(TextBlock.TextProperty, new Binding("Result"));
                visualTree.SetValue(TextBlock.FontSizeProperty, (double)10);
                visualTree.SetValue(TextBlock.ForegroundProperty, Brushes.Green);
                visualTree.SetValue(TextBlock.MinWidthProperty, (double)25);
                visualTree.SetValue(TextBlock.MaxWidthProperty, (double)75);

                // Create 'Triggers' for foreground property
                /*var styleTrigger = new Style(typeof (ListViewItem));
                var trigger = new Trigger();
                trigger.SetValue(TextBlock.TextProperty, "False");
                trigger.Setters.Add(new Setter(TextBlock.ForegroundProperty, Brushes.Red));
                styleTrigger.Triggers.Add(trigger);
                listViewFilesChecked.ItemContainerStyle = styleTrigger;*/

                gridView.Columns.Add(new GridViewColumn()
                                         {
                                             Header = "Result",
                                             CellTemplate = new DataTemplate(typeof (TextBlock))
                                                                {
                                                                    VisualTree = visualTree,
                                                                }

                                         });

                // Create 'FileName' column
                visualTree = new FrameworkElementFactory(typeof(TextBlock));
                visualTree.SetBinding(TextBlock.TextProperty, new Binding("FilePath"));
                visualTree.SetValue(TextBlock.FontSizeProperty, (double)10);
                visualTree.SetValue(TextBlock.ForegroundProperty, Brushes.Navy);
                visualTree.SetValue(TextBlock.MinWidthProperty, (double)250);

                gridView.Columns.Add(new GridViewColumn()
                                         {
                                             Header = "File Name",
                                             Width = 350,
                                             CellTemplate = new DataTemplate(typeof (TextBlock))
                                                                {
                                                                    VisualTree = visualTree,
                                                                }

                                         });

                // Create 'FileSize' column
                visualTree = new FrameworkElementFactory(typeof(TextBlock));
                visualTree.SetBinding(TextBlock.TextProperty, new Binding("FileSize"));
                visualTree.SetValue(TextBlock.FontSizeProperty, (double)10);
                visualTree.SetValue(TextBlock.ForegroundProperty, Brushes.Maroon);
                visualTree.SetValue(TextBlock.MinWidthProperty, (double)25);

                gridView.Columns.Add(new GridViewColumn()
                                         {
                                             Header = "File Size",
                                             Width = 100,
                                             CellTemplate = new DataTemplate(typeof (TextBlock))
                                                                {
                                                                    VisualTree = visualTree,
                                                                },

                                         });

                // Create 'IsReadOnly' column
                visualTree = new FrameworkElementFactory(typeof(TextBlock));
                visualTree.SetBinding(TextBlock.TextProperty, new Binding("IsReadOnly"));
                visualTree.SetValue(TextBlock.FontSizeProperty, (double)10);
                visualTree.SetValue(TextBlock.ForegroundProperty, Brushes.Black);
                visualTree.SetValue(TextBlock.MinWidthProperty, (double)25);

                gridView.Columns.Add(new GridViewColumn()
                {
                    Header = "ReadOnly",
                    Width = 100,
                    CellTemplate = new DataTemplate(typeof(TextBlock))
                    {
                        VisualTree = visualTree,
                    },

                });

                // Set gridView and Collection
                listViewFilesChecked.View = gridView;
                listViewFilesChecked.ItemsSource = xmlFilesResultCollection.CollectionToSerialize.OrderBy(sort => sort.Result);

            }
            catch (Exception e)
            {
                Console.WriteLine(e);
            }
        }

        /// <summary>
        /// Method helper for WCF call to retrieve the current log's fileName.
        /// </summary>
        /// <param name="currentChannel"></param>
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private string DoWCFGetCurrentLogFileName(IclWebCommService currentChannel)
        {
            // Call WCF service call.
            return currentChannel.GetCurrentLogFileName();
        }
    }
}
