﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Threading;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.CheckedDataModel;
using ImageNexus.BenScharbach.ICLTools.StorageTools.CloudData;
using ImageNexus.BenScharbach.ImageCloudLock.ProgressBar;

namespace ImageNexus.BenScharbach.ImageCloudLock.ParallelActions
{
    /// <summary>
    /// Returns a restore collection using a ParallelFor action.
    /// </summary>
    public class DoParallelForCreateRestoreFiles
    {
        private ObjectState _objectState;

        /// <summary>
        /// Objectstate class used to pass into parallel thread.
        /// </summary>
        private class ObjectState
        {
            public object ObjectStateName1;
            public object ObjectStateName2;
            public readonly ConcurrentBag<GetFileCloudItem> ConcurrentBag = new ConcurrentBag<GetFileCloudItem>();
            public Dispatcher Dispatcher;

            public int FileCounter;
            public int TotalCount;
            private readonly ProgressBarService _progressBar = new ProgressBarService();

            /// <summary>
            /// Constructor
            /// </summary>
            public void UpdateProgressBar()
            {
                Interlocked.Add(ref FileCounter, 1);

                Dispatcher.BeginInvoke(DispatcherPriority.Normal, new Action<ObjectState>(UpdateProgressBarCallback), this);
            }

            /// <summary>
            /// ProgressBar callback
            /// </summary>
            /// <param name="obj">Instance of <see cref="ObjectState"/></param>
            private void UpdateProgressBarCallback(ObjectState obj)
            {
                if (obj == null)
                {
                    return;
                }

                var progressPercentOverall = (obj.FileCounter * 100 / obj.TotalCount);

                if (obj._progressBar == null)
                {
                    return;
                }

                if (obj._progressBar.ProgressBarWindow == null)
                {
                    return;
                }

                obj._progressBar.ProgressBarWindow.HeaderText = "Creating the restore collection...";
                // Header-1 section
                obj._progressBar.ProgressBarWindow.ProgressBar1_MainDescription = string.Format("Iterating file: {0} of {1}.", obj.FileCounter, obj.TotalCount);
                obj._progressBar.ProgressBarWindow.ProgressBar1_DetailData = string.Format("Completed {0} %", progressPercentOverall);
            }
        }

        /// <summary>
        /// Creates the restore files collection using a parallelFor construct.
        /// </summary>
        /// <param name="fileCheckedItems">Collection of files to update</param>
        /// <returns>Array of <see cref="GetFileCloudItem"/>.</returns>
        public GetFileCloudItem[] CreateRestoreFilesCollection(List<IFileCheckedItem<IFileItem>> fileCheckedItems, object paramName1 = null, object paramName2 = null)
        {
            _objectState = new ObjectState
                                  {
                                      FileCounter = 0,
                                      TotalCount = fileCheckedItems.Count,
                                      ObjectStateName1 = paramName1,
                                      ObjectStateName2 = paramName2,
                                      Dispatcher = Dispatcher.CurrentDispatcher
                                  };

            var result = Parallel.ForEach(fileCheckedItems, () => _objectState, DoActionCallback, DoLocalFinalCallback);

            // Convert to array.
            var restoreFilesToSend = new GetFileCloudItem[_objectState.ConcurrentBag.Count];
            _objectState.ConcurrentBag.CopyTo(restoreFilesToSend, 0);

            return restoreFilesToSend;
        }

        /// <summary>
        /// ParallelFor LocalFinal callback
        /// </summary>
        /// <param name="objectState">Instance of <see cref="ObjectState"/></param>
        private static void DoLocalFinalCallback(ObjectState objectState)
        {
            // Update Progress Information
            objectState.UpdateProgressBar();
        }

        /// <summary>
        /// ParallelFor DoAction callback
        /// </summary>
        /// <param name="iterationItem">Instance of <see cref="IFileCheckedItem{IFileItem}"/></param>
        /// <param name="parallelLoopState">Instance of <see cref="ParallelLoopState"/></param>
        /// <param name="objectState">Instance of <see cref="ObjectState"/></param>
        /// <returns>Instance of <see cref="ObjectState"/></returns>
        private static ObjectState DoActionCallback(IFileCheckedItem<IFileItem> iterationItem, ParallelLoopState parallelLoopState, ObjectState objectState)
        {
            if (iterationItem == null)
            {
                return objectState;
            }

            var fileItem = iterationItem.OriginalItem;
            if (fileItem == null)
            {
                return objectState;
            }

            // Create instance of 'GetFileCloudItem' to send for retrieval request.
            objectState.ConcurrentBag.Add(new GetFileCloudItem
            {
                Key = fileItem.CloudKey,
                Name = fileItem.FileName,
                DirectoryName = fileItem.DirectoryName,
                FileSize = fileItem.FileSize,
                FullName = fileItem.FullPathName
            });

            return objectState;
        }
    }
}