﻿using System;
using System.ServiceModel;
using System.Windows.Forms;
using Attributes;
using ImageNexus.BenScharbach.ImageCloudLock;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.Common.ProgressBarInterface;
using ImageNexus.BenScharbach.Common.ProgressBarWindow;
using ImageNexus.BenScharbach.Common.ProgressBarWindow.Enums;
using ImageNexus.BenScharbach.ImageCloudLock.Enums;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;
using ImageNexus.BenScharbach.ImageCloudLock.WcfCallbacks;

namespace ImageNexus.BenScharbach.ImageCloudLock.ProgressBar
{
    public class ProgressBarService
    {
        private ProgressBarEventHandlerTypeEnum _progressBarEventHandlerTypeEnum;
        private readonly IclWebDuplexCallbacks _iclWebDuplexCallbacks;
        private ProgressBarWindow _progressBarWindow;
        private readonly MainWindow _mainWindowParent;

        #region Properties

        /// <summary>
        /// Gets the current <see cref="ProgressBarWindow"/>.
        /// </summary>
        public ProgressBarWindow ProgressBarWindow
        {
            get { return _progressBarWindow; }
        }

        // 12/6/2010
        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{TChannel}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return _mainWindowParent != null ? _mainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (_mainWindowParent != null) _mainWindowParent.IclDuplexChannelFactory = value; }
        }

        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        /// <summary>
        /// Constructor
        /// </summary>
        public ProgressBarService()
        {
            // 9/4/2011 - Retrieve service
            _mainWindowParent = (MainWindow) MainWindow.UnityContainer.Resolve(typeof (MainWindow), "MainWindowService");
            _iclWebDuplexCallbacks = _mainWindowParent.IclWebDuplexCallbacks;
            
            // 2/11/2011 - Subscribe to the 'TransferCompleted' from WCF service.
            _iclWebDuplexCallbacks.OperationCompleted += IclWebDuplexCallbacks_OperationCompleted;
        }

        /// <summary>
        /// Starts ProgressBar
        /// </summary>
        /// <param name="progressBarEventHandlerTypeEnum">Set progressBar EventHandler type to use; for example, 'Restore'.</param>
        /// <param name="progressBarTypeEnum">Set progressBar type to use; for example, 'Mini'.</param>
        /// <param name="overallStatusText">(Optional) Status text to show.</param>
        public void Start(ProgressBarEventHandlerTypeEnum progressBarEventHandlerTypeEnum, ProgressBarType progressBarTypeEnum, string overallStatusText = "Processing")
        {
            // 3/1/2012 - Check if new instance of ProgressBar Window is required.
            if (_progressBarWindow == null)
            {
                // 10/29/2010 - Create instance of ICLProgressBar window.
                _progressBarWindow = new ProgressBarWindow();
                _progressBarWindow.SetProgressBarType(progressBarTypeEnum); // 11/12/2010
                // 11/10/2010 - Subscribe to the 'CancelOperation' event in the ProgressBar.
                _progressBarWindow.CancelOperation += _progressBarWindow_CancelOperation;
                // 11/10/2010 - Subscribe to the 'TransferCompleted' from WCF service.
                _iclWebDuplexCallbacks.OperationCompleted += _progressBarWindow.OnOperationCompleted;
            }

            _progressBarWindow.Show();
            _progressBarWindow.StartProgressBars = true;

            // Subscribe to proper eventHandler for progressbar
            SubscribeProgressBarEventHandler(progressBarEventHandlerTypeEnum);

             // 1/12/2012 - Reset Labels
            ResetProgressBarLabels(progressBarEventHandlerTypeEnum);

            // 11/12/2010 - Update Status in ProgressBar window.
            _progressBarWindow.OverallStatusText = overallStatusText;
        }

        /// <summary>
        /// Set to top most window?
        /// </summary>
        /// <param name="topMost">true/false</param>
        public void SetTopMostWindow(bool topMost)
        {
            // Turn Off the progressBar from being top window.
            if (_progressBarWindow != null) _progressBarWindow.SetTopMostWindow(topMost);
        }

        /// <summary>
        /// Cancel current progressBar operation.
        /// </summary>
        public void DoCancelOperation()
        {
            // Cancel Operation.
            if (_progressBarWindow != null) _progressBarWindow.DoCancelOperation();
        }

        #region ProgresBar EventHandler Methods

        // 11/10/2010
        /// <summary>
        /// Occurs when the user clicks the Cancel button in the progressBar window.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        // ReSharper disable InconsistentNaming
        private void _progressBarWindow_CancelOperation(object sender, EventArgs e)
        // ReSharper restore InconsistentNaming
        {
             // Note: 1st param can be null, since it will be set by postsharp.
            DoWCFCancelCurrentFileOperation(null);

            // 3/1/2012 - Nullify the ProgressWindow, since it calls Close() internally.
            _progressBarWindow = null;
        }

        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private static void DoWCFCancelCurrentFileOperation(IclWebCommService currentChannel)
        {
            // send request to WCF service to cancel the current file operation.
            currentChannel.CancelCurrentFileOperation();
        }

        // 2/11/2011
        /// <summary>
        /// Used to attach to ICLWebService eventHandler for operation completed, which 
        /// will then trigger to stop the internal background worker thread.
        /// </summary>
        // ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void IclWebDuplexCallbacks_OperationCompleted(object sender, IOperationCompletedEventArgs e)
        // ReSharper restore InconsistentNaming
        {
            // 2/11/2011 - Unsubscribe from current eventHandler for progressbar
            UnSubscribeProgressBarEventHandler();

            // 3/1/2012 - Nullify the ProgressWindow, since it calls Close() internally.
            _progressBarWindow = null;
        }

        // 1/12/2012
        /// <summary>
        /// Resets the progressBar labels.
        /// </summary>
        /// <param name="progressBarEventHandlerTypeEnum"></param>
        private void ResetProgressBarLabels(ProgressBarEventHandlerTypeEnum progressBarEventHandlerTypeEnum)
        {
            // 3/1/2012
            if (_progressBarWindow == null) return;

            switch (progressBarEventHandlerTypeEnum)
            {
                case ProgressBarEventHandlerTypeEnum.BackupEventHandler:
                    // 1/12/2012 - Set Title for window
                    _progressBarWindow.Title = "Backup Operation Progress Window";
                    // 12/6/2010 - Updated ProgressBar with header information.
                    _progressBarWindow.HeaderText = "Backup Starting...";
                    // Header-1 section
                    _progressBarWindow.ProgressBar1_MainDescription = string.Format("Backed up files: {0} of {1}.", 0, 0);
                    _progressBarWindow.ProgressBar1_DetailData = string.Format("Completed {0} %", 0);
                    // Header-2 section
                    _progressBarWindow.ProgressBar2_MainDescription = "Current File -";
                    _progressBarWindow.ProgressBar2_DetailData = string.Format("Completed {0} %", 0);
                    // Set Footer Status
                    _progressBarWindow.OverallStatusText = string.Format("Backed up {0} of {1} ", 0, 0);
                    break;
                case ProgressBarEventHandlerTypeEnum.RestoreEventHandler:
                    // 1/12/2012 - Set Title for window
                    _progressBarWindow.Title = "Restore Operation Progress Window";
                    // 12/6/2010 - Updated ProgressBar with header information.
                    _progressBarWindow.HeaderText = "Restore Starting...";
                    // Header-1 section
                    _progressBarWindow.ProgressBar1_MainDescription = string.Format("Restored files: {0} of {1}.", 0, 0);
                    _progressBarWindow.ProgressBar1_DetailData = string.Format("Completed {0} %", 0);
                    // Header-2 section
                    _progressBarWindow.ProgressBar2_MainDescription = "Current File -";
                    _progressBarWindow.ProgressBar2_DetailData = string.Format("Completed {0} %", 0);
                    // Set Footer Status
                    _progressBarWindow.OverallStatusText = string.Format("Restored back {0} of {1} ", 0, 0);
                    break;
                case ProgressBarEventHandlerTypeEnum.DeleteEventHandler:
                    // 1/12/2012 - Set Title for window
                    _progressBarWindow.Title = "Delete Operation Progress Window";
                    // 12/6/2010 - Updated ProgressBar with header information.
                    _progressBarWindow.HeaderText = "Delete Starting...";
                    // Header-1 section
                    _progressBarWindow.ProgressBar1_MainDescription = string.Format("Deleted files: {0} of {1}.", 0, 0);
                    _progressBarWindow.ProgressBar1_DetailData = string.Format("Completed {0} %", 0);
                    // Header-2 section
                    _progressBarWindow.ProgressBar2_MainDescription = "Current File -";
                    _progressBarWindow.ProgressBar2_DetailData = string.Format("Completed {0} %", 0);
                    // Set Footer Status
                    _progressBarWindow.OverallStatusText = string.Format("Deleted {0} of {1} ", 0, 0);
                   
                    break;
                default:
                    throw new ArgumentOutOfRangeException("progressBarEventHandlerTypeEnum");
            }
        }


        // 2/10/2011
        /// <summary>
        /// Subscribe to the proper eventHandler for the file operation type.
        /// </summary>
        /// <param name="progressBarEventHandlerTypeEnum">Instance of <see cref="ProgressBarEventHandlerTypeEnum"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SubscribeProgressBarEventHandler(ProgressBarEventHandlerTypeEnum progressBarEventHandlerTypeEnum)
        {
            // 2/11/2011 - Store subcribe type, used for unsubscribe call
            _progressBarEventHandlerTypeEnum = progressBarEventHandlerTypeEnum;

            switch (progressBarEventHandlerTypeEnum)
            {
                case ProgressBarEventHandlerTypeEnum.BackupEventHandler:
                    // Subscribe to the 'OperationFileProgress' from WCF service.
                    _iclWebDuplexCallbacks.OperationFileProgres += _iclWebDuplexCallbacks_BackupFileProgres;
                    break;
                case ProgressBarEventHandlerTypeEnum.RestoreEventHandler:
                    // Subscribe to the 'OperationFileProgress' from WCF service.
                    _iclWebDuplexCallbacks.OperationFileProgres += _iclWebDuplexCallbacks_RestoreFileProgres;
                    break;
                case ProgressBarEventHandlerTypeEnum.DeleteEventHandler:
                    // Subscribe to the 'OperationFileProgress' from WCF service.
                    _iclWebDuplexCallbacks.OperationFileProgres += _iclWebDuplexCallbacks_DeleteFileProgres;
                    break;
                default:
                    throw new ArgumentOutOfRangeException("progressBarEventHandlerTypeEnum");
            }
        }

        // 2/10/2011
        /// <summary>
        /// Unsubscribes from the current eventHandler for the file operation type.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public void UnSubscribeProgressBarEventHandler()
        {
            switch (_progressBarEventHandlerTypeEnum)
            {
                case ProgressBarEventHandlerTypeEnum.BackupEventHandler:
                    // Subscribe to the 'OperationFileProgress' from WCF service.
                    _iclWebDuplexCallbacks.OperationFileProgres -= _iclWebDuplexCallbacks_BackupFileProgres;
                    break;
                case ProgressBarEventHandlerTypeEnum.RestoreEventHandler:
                    // Subscribe to the 'OperationFileProgress' from WCF service.
                    _iclWebDuplexCallbacks.OperationFileProgres -= _iclWebDuplexCallbacks_RestoreFileProgres;
                    break;
                case ProgressBarEventHandlerTypeEnum.DeleteEventHandler:
                    // Subscribe to the 'OperationFileProgress' from WCF service.
                    _iclWebDuplexCallbacks.OperationFileProgres -= _iclWebDuplexCallbacks_DeleteFileProgres;
                    break;
                default:
                    break;
            }
        }

        // 2/25/2012
        /// <summary>
        /// Checks for error messages sent through the progress callback, and displays the message using a MessageBox display.
        /// </summary>
        /// <param name="progressBarData"></param>
        private static void CheckForErrorMessages(OperationProgress progressBarData)
        {
            if (progressBarData.ErrorOccured)
            {
                MessageBox.Show(progressBarData.ErrorMessageToDisplay, "Operation Error", MessageBoxButtons.OK,
                                MessageBoxIcon.Error);
            }
        }

        // 2/10/2011 - Updated to seperate 'Backup' from 'Restore' event handling.
        // 12/6/2010 - Updated to include info for MainDesc headers.
        /// <summary>
        /// Captures callback which provides backup file progress data, and updates
        /// the current progress bars.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void _iclWebDuplexCallbacks_BackupFileProgres(object sender, EventArgs e)
        {
            // Get ProgressBar data
            var progressBarData = _iclWebDuplexCallbacks.CurrentFileTransferProgress;

            if (progressBarData == null || _progressBarWindow == null) return;

#if DEBUG
            Console.WriteLine(@"File Progress = " + progressBarData.PercentDoneForFile);
            Console.WriteLine(@"Overall Progress = " + progressBarData.PercentDoneOverall);
#endif

            // 12/6/2010 - Updated ProgressBar with header information.
            _progressBarWindow.HeaderText = progressBarData.CurrentFile;
            // Header-1 section
            _progressBarWindow.ProgressBar1_MainDescription = string.Format("Backed up files: {0} of {1}.",
                                                                           progressBarData.FileIteration,
                                                                           progressBarData.TotalFilesToTransfer);
            _progressBarWindow.ProgressBar1_DetailData = string.Format("Completed {0} %",
                                                                      progressBarData.PercentDoneOverall);
            // Header-2 section
            _progressBarWindow.ProgressBar2_MainDescription = (!progressBarData.IsMultiPartOperation)
                                                                  ? string.Format("Current File -")
                                                                  : string.Format("Part {0} of {1} -",
                                                                                  progressBarData.CurrentPart + 1,
                                                                                  progressBarData.TotalPartsToTransfer +
                                                                                  1);
            _progressBarWindow.ProgressBar2_DetailData = string.Format("Completed {0} %",
                      
                                                progressBarData.PercentDoneForFile);
            
            // TODO: Why 'FileSizeConverter' not showing up? (12/13/2012)
            // Set Footer Status
            /*_progressBarWindow.OverallStatusText = string.Format("Backed up {0} of {1} ",
                                                                progressBarData.CompletedBytes.FileSizeConverter(),
                                                                progressBarData.TotalBytes.FileSizeConverter());*/
           
            // Update progressbars.
            _progressBarWindow.UpdatePercentCompleteForProgressBar1(progressBarData.PercentDoneForFile);
            // Update progressbars.
            _progressBarWindow.UpdatePercentCompleteForProgressBar2(progressBarData.PercentDoneOverall);

            // 2/25/2012 - Check for errors sent from service
            CheckForErrorMessages(progressBarData);
        }

        // 2/10/2011 - Updated to seperate 'Backup' from 'Restore' event handling.
        /// <summary>
        /// Captures callback which provides restore file progress data, and updates
        /// the current progress bars.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void _iclWebDuplexCallbacks_RestoreFileProgres(object sender, EventArgs e)
        {
            // Get ProgressBar data
            var progressBarData = _iclWebDuplexCallbacks.CurrentFileTransferProgress;

            if (progressBarData == null || _progressBarWindow == null) return;

#if DEBUG
            Console.WriteLine(@"File Progress = " + progressBarData.PercentDoneForFile);
            Console.WriteLine(@"Overall Progress = " + progressBarData.PercentDoneOverall);
#endif

            // 12/6/2010 - Updated ProgressBar with header information.
            _progressBarWindow.HeaderText = progressBarData.CurrentFile;
            // Header-1 section
            _progressBarWindow.ProgressBar1_MainDescription = string.Format("Restored files: {0} of {1}.",
                                                                           progressBarData.FileIteration,
                                                                           progressBarData.TotalFilesToTransfer);
            _progressBarWindow.ProgressBar1_DetailData = string.Format("Completed {0} %",
                                                                      progressBarData.PercentDoneOverall);
            // Header-2 section
            _progressBarWindow.ProgressBar2_MainDescription = (!progressBarData.IsMultiPartOperation)
                                                                  ? string.Format("Current File -")
                                                                  : string.Format("Part {0} of {1} -",
                                                                                  progressBarData.CurrentPart + 1,
                                                                                  progressBarData.TotalPartsToTransfer +
                                                                                  1);
            _progressBarWindow.ProgressBar2_DetailData = string.Format("Completed {0} %",
                 
                                                     progressBarData.PercentDoneForFile);

            // TODO: Why 'FileSizeConverter' not showing up? (12/13/2012)
            // Set Footer Status
            /*_progressBarWindow.OverallStatusText = string.Format("Restored back {0} of {1} ",
                                                                progressBarData.CompletedBytes.FileSizeConverter(),
                                                                progressBarData.TotalBytes.FileSizeConverter());*/

            // Update progressbars.
            _progressBarWindow.UpdatePercentCompleteForProgressBar1(progressBarData.PercentDoneForFile);
            // Update progressbars.
            _progressBarWindow.UpdatePercentCompleteForProgressBar2(progressBarData.PercentDoneOverall);

            // 2/25/2012 - Check for errors sent from service
            CheckForErrorMessages(progressBarData);
        }

        // 4/19/2011
        /// <summary>
        /// Captures callback which provides delete file progress data, and updates
        /// the current progress bars.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void _iclWebDuplexCallbacks_DeleteFileProgres(object sender, EventArgs e)
        {
            // Get ProgressBar data
            var progressBarData = _iclWebDuplexCallbacks.CurrentFileTransferProgress;

            if (progressBarData == null || _progressBarWindow == null) return;

#if DEBUG
            Console.WriteLine(@"File Progress = " + progressBarData.PercentDoneForFile);
            Console.WriteLine(@"Overall Progress = " + progressBarData.PercentDoneOverall);
#endif

            // 12/6/2010 - Updated ProgressBar with header information.
            _progressBarWindow.HeaderText = progressBarData.CurrentFile;
            // Header-1 section
            _progressBarWindow.ProgressBar1_MainDescription = string.Format("Deleted files: {0} of {1}.",
                                                                           progressBarData.FileIteration,
                                                                           progressBarData.TotalFilesToTransfer);
            _progressBarWindow.ProgressBar1_DetailData = string.Format("Completed {0} %",
                                                                      progressBarData.PercentDoneOverall);
            // Header-2 section
            _progressBarWindow.ProgressBar2_MainDescription = (!progressBarData.IsMultiPartOperation)
                                                                  ? string.Format("Current File -")
                                                                  : string.Format("Part {0} of {1} -",
                                                                                  progressBarData.CurrentPart + 1,
                                                                                  progressBarData.TotalPartsToTransfer +
                                                                                  1);
            _progressBarWindow.ProgressBar2_DetailData = string.Format("Completed {0} %",
                                                                      progressBarData.PercentDoneForFile);
            // Set Footer Status
            _progressBarWindow.OverallStatusText = "Currently Deleting files...";

            // Update progressbars.
            _progressBarWindow.UpdatePercentCompleteForProgressBar1(progressBarData.PercentDoneForFile);
            // Update progressbars.
            _progressBarWindow.UpdatePercentCompleteForProgressBar2(progressBarData.PercentDoneOverall);

            // 2/25/2012 - Check for errors sent from service
            CheckForErrorMessages(progressBarData);
        }

        #endregion 
    }
}