﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Threading;
using Attributes;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsMisc;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsThreading;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.Common.ProgressBarInterface;
using ImageNexus.BenScharbach.Common.ProgressBarWindow.Enums;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryViewModelBase.Enums;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.CheckedDataModel;
using ImageNexus.BenScharbach.ICLTools.StorageTools.CloudData;
using ImageNexus.BenScharbach.Common.LogWinEvents;
using ImageNexus.BenScharbach.ImageCloudLock.Enums;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;
using ImageNexus.BenScharbach.ImageCloudLock.ParallelActions;
using ImageNexus.BenScharbach.ImageCloudLock.ProgressBar;
using Microsoft.Practices.Unity;

namespace ImageNexus.BenScharbach.ImageCloudLock
{
    /// <summary>
    /// Interaction logic for RestoreActionWindow.xaml
    /// </summary>
    public partial class RestoreActionWindow : Window
    {
        private ProgressBarService _progressBarService;

        // List of files to restore
        private readonly List<IFileCheckedItem<IFileItem>> _fileCheckedItems;

        // List of files to restore in WCF format
        private List<GetFileCloudItem> _restoreFilesToSend;

        // Batch Count
        private int _batchCount;

        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; private set; }
        
        /// <summary>
        /// Gets the current instance of <see cref="MiniProgressBar"/>.
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(IMiniProgressBar), null)]
        public IMiniProgressBar MiniProgressBar
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.MiniProgressBar : null;
            }
            set
            {
                // If null, then just wait in parallel thread.
                // Note: This will always occur when window created because PostSharp attribute tries
                // note: to set before the constructor has run.
                MainWindow windowParent = MainWindowParent; // 7/22/2011 - Cache
                if (windowParent == null)
                    Task.Factory.StartNew(() =>
                    {
                        while (windowParent == null)
                        {
                            // Just spin wait. 
                        }

                        // Not null anymore, so set value.
                        windowParent.MiniProgressBar = value;

                    });
                else
                    windowParent.MiniProgressBar = value;
            }
        }
        
        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{TChannel}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }
        
        /// <summary>
        /// Gets the current communication channel.
        /// </summary>
        public IclWebCommService CurrentChannel { get; set; }
        
        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        /// <summary>
        /// Restore action window constructor
        /// </summary>
        public RestoreActionWindow(MainWindow mainWindow, List<IFileCheckedItem<IFileItem>> fileCheckedItems)
        {
            InitializeComponent();

            // 1/11/2012
            if (fileCheckedItems == null)
            {
                MessageBox.Show("You MUST select an item first to restore.");
                Close();
                return;
            }

            // 1/11/2012
            if (fileCheckedItems.Count == 0)
            {
                MessageBox.Show("You MUST select an item first to restore.");
                Close();
                return;
            }

            // Save MainWindow ref
            MainWindowParent = mainWindow;

            // 1/27/2012 - Set CloudService to Use in Wcf.
            mainWindow.SetCloudServiceToUse();

            // 1/11/2012 - Save files to restore
            _fileCheckedItems = fileCheckedItems;

            // 1/11/2012 - Show files to restore in listbox
            ShowFilesToRestoreInListbox(string.Format("Files Selected for Restoration - {0};", _fileCheckedItems.Count), _fileCheckedItems);

            // Subscribe to the 'TransferCompleted' event.
            MainWindowParent.IclWebDuplexCallbacks.OperationCompleted += IclWebDuplexCallbacks_TransferCompleted;
        }

        // 1/11/2012
        /// <summary>
        /// Shows files to restore in the listbox
        /// </summary>
        /// <param name="header"> </param>
        /// <param name="itemsToDisplay"> </param>
        private void ShowFilesToRestoreInListbox(string header, IEnumerable<IFileCheckedItem<IFileItem>> itemsToDisplay)
        {
            // iterate the files to restore
            foreach (var item in itemsToDisplay)
            {
                restoreFeedBackListBox.Items.Add(new TextBlock() { Text = item.FullName, FontSize = 8, Foreground = Brushes.Navy });
            }

            // Set Footer for this section
            restoreFeedBackListBox.Items.Add(new TextBlock() { Text = header, FontSize = 9, Foreground = Brushes.Crimson });
        }

        // 1/11/2012
        /// <summary>
        /// Show files restored successfully in the listbox
        /// </summary>
        /// <param name="header"> </param>
        /// <param name="doWcfEndRestoreFilesFromCloud"></param>
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = true)]
        private void ShowFilesRestoredSuccessInListbox(string header, IEnumerable<DeleteFileCloudItem> doWcfEndRestoreFilesFromCloud)
        {
            // iterate the files to restore
            foreach (var item in doWcfEndRestoreFilesFromCloud)
            {
                successFeedBackListBox.Items.Add(new TextBlock() { Text = item.FullName, FontSize = 8, Foreground = Brushes.Navy });
            }

            // Set Footer for this section
            successFeedBackListBox.Items.Add(new TextBlock() { Text = header, FontSize = 9, Foreground = Brushes.Green });
        }

        /// <summary>
        /// Occurs when the transfer is complete.
        /// </summary>
        // ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void IclWebDuplexCallbacks_TransferCompleted(object sender, IOperationCompletedEventArgs e)
        // ReSharper restore InconsistentNaming
        {

            // 11/10/2010 - First check if cancelled.
            if (e.OperationCancelled)
            {
                txtResults.Foreground = new SolidColorBrush(Colors.OrangeRed);
                txtResults.Text = "Cancelled";
                return;
            }

            // Check result of transfer
            if (!e.OperationResult)
            {
                txtResults.Foreground = new SolidColorBrush(Colors.Red);
                txtResults.Text = "Failed";
            }
            else
            {
                txtResults.Foreground = new SolidColorBrush(Colors.Green);
                txtResults.Text = "Success";
            }

        }

        // 1/11/2012 - Starts Restore operation
        private void btnStartRestore_Click(object sender, RoutedEventArgs e)
        {
            // Confirm restore request
            var result =
                MessageBox.Show("This will RESTORE the checked items from your cloud account.  Do you want to continue?",
                                "Delete items", MessageBoxButton.YesNo, MessageBoxImage.Question, MessageBoxResult.None);

            // If No, then return
            if (result == MessageBoxResult.No) return;

            // Show WCF ProgressBar Window
            _progressBarService = new ProgressBarService(); 
            _progressBarService.Start(ProgressBarEventHandlerTypeEnum.RestoreEventHandler, ProgressBarType.StandardDualBar);

            // Get collection of files to restore
            var doParallelCreateFiles = new DoParallelForCreateRestoreFiles();
            _restoreFilesToSend = doParallelCreateFiles.CreateRestoreFilesCollection(_fileCheckedItems).ToList();

            // Start Restore Operation
            RestoreFilesFromCloud();
        }

        // 12/14/2010 - 
        [ShowWaitCursor]
        [CatchAllExceptionHandling(false, null, null)]
        [OnGuiThread(DispatcherPriority.Normal, Asynchronous = false)]
        private void RestoreFilesFromCloud()
        {
            // if null, return
            if (_restoreFilesToSend == null)
            {
                return;
            }

            // if empty, return
            if (_restoreFilesToSend.Count == 0)
            {
                return;
            }

            // Transfer restore files in batches to WCF service.
            bool clearPriorData = true;
            do
            {
                // 1/14/2012 - Take a batch of files to send to WCF.  When the WCF EndCall is reached, this method will
                //             be recalled if there are remaining records.
                const int batchCount = 50;
                var restoreFilesToSendBatched = new GetFileCloudItem[batchCount];
                var copyCount = (_restoreFilesToSend.Count >= batchCount) ? batchCount : _restoreFilesToSend.Count;
                _restoreFilesToSend.CopyTo(0, restoreFilesToSendBatched, 0, copyCount);

                // Check if batch smaller than batchCount; resize if necessary
                if (_restoreFilesToSend.Count < batchCount)
                {
                    Array.Resize(ref restoreFilesToSendBatched, _restoreFilesToSend.Count);
                }

                // remove range
                _restoreFilesToSend.RemoveRange(0, copyCount);

                // 1/14/2012 - Clear prior data
                // Note: 1st param can be null, since it will be set by postsharp.
                DoWCFAddRestoreFilesBatched(null, restoreFilesToSendBatched, clearPriorData);
                clearPriorData = false;
            }
            while (_restoreFilesToSend.Count > 0);


            // 12/14/2010 - Create AsyncCallObject to pass to callback.
            var asyncCallObject = new AsyncRestoreCallObject<IclWebCommService>
            {
                FileCheckedItems = _fileCheckedItems, // 5/14/2011
                CurrentChannel = null,
                ResultObject = null,
            };

            // Note: 1st param can be null, since it will be set by postsharp.
            DoWCFRestoreFilesCall(null, asyncCallObject);
        }
        
        /// <summary>
        /// Method helper for WCF call to add restore batch files.
        /// </summary>
        /// <param name="currentChannel"></param>
        /// <param name="filesToRestore">Collection of files to delete batched.</param>
        /// <param name="clearPriorData">Clears the prior data from service's internal collection.</param>
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private void DoWCFAddRestoreFilesBatched(IclWebCommService currentChannel, GetFileCloudItem[] filesToRestore, bool clearPriorData)
        {
            // Call WCF service call.
            currentChannel.AddRestoreFilesBatched(filesToRestore, clearPriorData);
        }

        /// <summary>
        /// Method helper for WCF call.
        /// </summary>
        /// <param name="currentChannel"></param>
        /// <param name="asyncCallObject"></param>
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.BeginCall)]
        private void DoWCFRestoreFilesCall(IclWebCommService currentChannel, AsyncRestoreCallObject<IclWebCommService> asyncCallObject)
        {
            // Call WCF service call.
            asyncCallObject.CurrentChannel = currentChannel;
            currentChannel.BeginRestoreFilesFromCloud(OnEndRestoreFilesFromCloudCallback, asyncCallObject);
        }
        
        /// <summary>
        /// WCF Callback when BeginRestoreFilesFromCloud async operation completes.
        /// </summary>
        /// <param name="asyncResult"></param>
        [OnWCFProxyCall(AsynchronousCallType = OnWCFProxyCallAttribute.AsynchronousCallEnum.EndCall)]
        private void OnEndRestoreFilesFromCloudCallback(IAsyncResult asyncResult)
        {
            try
            {

                var asyncCallObject = asyncResult.AsyncState as AsyncRestoreCallObject<IclWebCommService>;
                if (asyncCallObject == null)
                    throw new InvalidOperationException("The AsyncCallObject cannot be null.");

                // Note: 1st param can be null, since it will be set by postsharp.
                DeleteFileCloudItem[] doWcfEndRestoreFilesFromCloud = asyncCallObject.CurrentChannel.EndRestoreFilesFromCloud(asyncResult);
                if (doWcfEndRestoreFilesFromCloud == null) return;

                // Convert to Dictionary format
                Dictionary<string, DeleteFileCloudItem> successfulRestoredFileCloudItems = doWcfEndRestoreFilesFromCloud.ToDictionary(key => key.Key);
                if (successfulRestoredFileCloudItems.Count == 0) return;

                //
                // Show Results to listbox
                //
                // Set Header for this section
                ShowFilesRestoredSuccessInListbox(string.Format("Files Restored Successfully = {0};", doWcfEndRestoreFilesFromCloud.Length), doWcfEndRestoreFilesFromCloud);

                // Clear out the entire CheckedCollections for the cloud, since the restore window is closed.
                var checkedCollection = ((IMainWindow)MainWindowParent).UnityContainer.Resolve<DirectoryTreeViewModelAdapterBase>(ViewModelTypeEnum.CloudViewModel.ToString());
                checkedCollection.ClearAllCheckedItems();

                // Check for additional batches
                RestoreFilesFromCloud();

            }
            catch (Exception err)
            {
                LogWinEventsHelper.LogErrorMessage("RestoreActionWindow.OnEndRestoreFilesFromCloudCallback method threw an exception - " + err.Message);
            }
        }

        // 12/5/2010 - AsyncState used for asynchronous calls.
        private class AsyncRestoreCallObject<TChannel>
        {
            // 5/14/20911
            public List<IFileCheckedItem<IFileItem>> FileCheckedItems;

            /// <summary>
            /// Stores the current WCF factory 'Channel'.
            /// </summary>
            public TChannel CurrentChannel;

            /// <summary>
            /// Stores some misc object state, if any.
            /// </summary>
            public object ResultObject;

        }

        /// <summary>
        /// Occurs when the user closes this window.
        /// </summary>
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            // Turn Off the progressBar from being top window.
            if (_progressBarService != null)
                _progressBarService.SetTopMostWindow(false);

            _batchCount = 0;
        }
    }
}
