﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.Threading.Tasks;
using Attributes;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.ICLTools.StorageTools.BackupData;
using ImageNexus.BenScharbach.ImageCloudLock;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;

namespace ImageCloudLock.TaskActions
{
    // 1/25/2012
    /// <summary>
    ///  Does the 'Backup operation for the Directories checked' in a Parallel task.
    /// </summary>
    public class DoBackupDirectoriesTask
    {
        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; set; }

        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{TChannel}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        #region Events

        /// <summary>
        /// Callback when task is completed.
        /// </summary>
        public event Action<Task> TaskCompleted;

        #endregion

        private class TaskObjectState
        {
            public MainWindow MainWindow;
            public DoBackupDirectoriesTask ThisTask;
            public string LocationPath;
            public string DriveName;
            public List<DirectoryBackupItem> DirectoriesToBackup;
        }

        /// <summary>
        /// Runs the Backup directories progress for the checked items.
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="MainWindow"/></param>
        /// <param name="locationPath"> </param>
        /// <param name="driveName"></param>
        /// <param name="directoriesToBackup"></param>
        /// <returns></returns>
        public Task DoBackupDirectoriesRoutine(MainWindow mainWindow, string locationPath, string driveName, List<DirectoryBackupItem> directoriesToBackup)
        {
            // Set
            MainWindowParent = mainWindow;

            Task task = Task.Factory.StartNew(DoTaskActionCallback,
                                             new TaskObjectState
                                                 {
                                                     MainWindow = mainWindow,
                                                     ThisTask = this,
                                                     LocationPath = locationPath,
                                                     DriveName = driveName,
                                                     DirectoriesToBackup = directoriesToBackup
                                                 }).ContinueWith(TaskIsCompletedCallback);

            return task;
        }

        /// <summary>
        /// Task Callback which triggers the <see cref="TaskCompleted"/> event.
        /// </summary>
        /// <param name="obj"></param>
        private void TaskIsCompletedCallback(Task obj)
        {
            if (TaskCompleted != null)
            {
                TaskCompleted(obj);
            }
        }

        /// <summary>
        /// Action callback which does the start-up initilization routines.
        /// </summary>
        /// <param name="arg"></param>
        /// <returns></returns>
        private void DoTaskActionCallback(object arg)
        {
            // Retrieve Task's objectState
            var objectState = (TaskObjectState)arg;

            // Doing chunks of 40 items. - YES!
            var counter = 0f;
            const int quantityToBatch = 40;
            var count = objectState.DirectoriesToBackup.Count;
            var currentBatch = new DirectoryBackupItem[quantityToBatch];

            for (var index = 0; index < count; index += quantityToBatch)
            {
                // Create batch of items
                var isPartialBatch = !(index + quantityToBatch <
                                       count);

                var quantityToCopy = (index + quantityToBatch <
                                      count)
                                         ? quantityToBatch
                                         : count -
                                           index;

                // Resize Array on final call.
                if (isPartialBatch)
                {
                    Array.Resize(ref currentBatch, quantityToCopy);
                }

                // Copy batch files
                objectState.DirectoriesToBackup.CopyTo(index, currentBatch, 0, quantityToCopy);

                // Upload batch to WCF service.
                // Note: 1st param can be null, since it will be set by postsharp.
                objectState.ThisTask.DoWcfSetDirectoriesToBackupByBatch(null, objectState.DriveName, objectState.LocationPath, currentBatch);

                Array.Clear(currentBatch, 0, quantityToCopy);

                // Update Progressbar.
                //MiniProgressBar.SetProgressBar2_HeaderText(string.Format("Directories {0} of {1}", counter++, count));
                //MiniProgressBar.SetProgressBar2_Value(counter);
            }
           
        }

        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private void DoWcfSetDirectoriesToBackupByBatch(IclWebCommService currentChannel, string driveName, string locationPath, DirectoryBackupItem[] currentBatch)
        {
            // Upload batch to WCF service.
            currentChannel.SetDirectoriesToBackupByBatch(driveName, locationPath, currentBatch);
        }
    }
}