﻿using System;
using System.ServiceModel;
using System.Threading.Tasks;
using ImageCloudLock;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;

// Adds the Unity Extenstions.

namespace ImageNexus.BenScharbach.ImageCloudLock.TaskActions
{
    // 1/25/2012
    /// <summary>
    ///  Does the 'Shut-Down routines for the MainWindow' in a Parallel task.
    /// </summary>
    public class DoIclShutDownRoutinesTask
    {
        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; set; }

        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{TChannel}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        #region Events

        /// <summary>
        /// Callback when task is completed.
        /// </summary>
        public event Action<Task> TaskCompleted;

        #endregion

        private class TaskObjectState
        {
            public MainWindow MainWindow;
            public System.Windows.Application Application;
            public object ObjectStateName3;
        }

        /// <summary>
        /// Runs the Shut-down routines for ICL application.
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="MainWindow"/></param>
        /// <param name="appCurrent"></param>
        /// <param name="paramName3"></param>
        /// <returns></returns>
        public Task DoShutDownRoutines(MainWindow mainWindow, System.Windows.Application appCurrent, object paramName3 = null)
        {
            Task task = Task.Factory.StartNew(DoTaskActionCallback,
                                             new TaskObjectState
                                                 {
                                                     MainWindow = mainWindow,
                                                     Application = appCurrent,
                                                     ObjectStateName3 = paramName3
                                                 });

            // Add Completion callback (Note: If ContinueWith added to Task above, then 'Result' will be missing!)
            task.ContinueWith(TaskIsCompletedCallback);

            return task;
        }

        /// <summary>
        /// Task Callback which triggers the <see cref="TaskCompleted"/> event.
        /// </summary>
        /// <param name="obj"></param>
        private void TaskIsCompletedCallback(Task obj)
        {
            if (TaskCompleted != null)
            {
                TaskCompleted(obj);
            }
        }

        /// <summary>
        /// Action callback which does the start-up initilization routines.
        /// </summary>
        /// <param name="arg"></param>
        /// <returns></returns>
        private void DoTaskActionCallback(object arg)
        {
            // Retrieve Task's objectState
            var objectState = (TaskObjectState)arg;
            var mainWindow = objectState.MainWindow;
            
            mainWindow.WaitMessage.SetMessageToDiplay("Save Backup Checked Items...");
            // Save any items user might have just picked in the Backup 'TreeView' items control.
            mainWindow.SaveBackupCheckedItems();

            // Clear BackUp Lists from service memory.
            // Note: 1st param can be null, since it will be set by postsharp.
            mainWindow.WaitMessage.SetMessageToDiplay("Clear Backup List...");
            mainWindow.ClearBackupList(null);

            // Dispose of resources.
            mainWindow.WaitMessage.SetMessageToDiplay("Dispose of Progress Bar...");
            mainWindow.DisposeOfProgressBarWindow();

            // Flush/Close/Recreate Service Listeners
            // Note: 1st param can be null, since it will be set by postsharp.
            mainWindow.WaitMessage.SetMessageToDiplay("Flush and Close all Trace Listeners...");
            mainWindow.FlushAndCloseAllTraceListeners(null);

            // Cancel Task
            mainWindow.WaitMessage.SetMessageToDiplay("Dispose of Poll Logger Service...");
            mainWindow.DisposeOfPollLoggerMessages();

            // Cancel Task
            mainWindow.WaitMessage.SetMessageToDiplay("Dispose of Poll Wcf Service...");
            mainWindow.DisposeOfPollWcfService();

            // Safely close connection using new extenion.
            mainWindow.WaitMessage.SetMessageToDiplay("Close Wcf Channels...");
            mainWindow.CloseWcfChannels();

            // Null Refs
            mainWindow.WaitMessage.SetMessageToDiplay("Null Pointers Refs...");
            mainWindow.NullPointerRefs();

            // Save User Settings.
            mainWindow.WaitMessage.SetMessageToDiplay("Save User Settings...");
            mainWindow.DoSaveUserSettings();

            // Dispose of Mini-ProgressBar
            mainWindow.WaitMessage.SetMessageToDiplay("Dispose of Mini-ProgressBar...");
            mainWindow.DisposeOfMiniProgressBar();
           
        }
    }
}