﻿using System;
using System.ServiceModel;
using System.Threading.Tasks;
using ImageCloudLock;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.LogWinEvents;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;

namespace ImageNexus.BenScharbach.ImageCloudLock.TaskActions
{
    // 1/25/2012
    /// <summary>
    ///  Does the 'Start-Up constructors routines for the MainWindow' in a Parallel task.
    /// </summary>
    public class DoIclStartUpRoutinesTask
    {
        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; set; }

        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{TChannel}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        #region Events

        /// <summary>
        /// Callback when task is completed.
        /// </summary>
        public event Action<Task<TaskObjectResult>> TaskCompleted;

        #endregion

        // 1/31/2012
        public enum InitComponentEnum
        {
            CheckForXpOs,
            StartDirectoryTreeViewTask,
            SetEventHandlersForCallback,
            CreateServiceCallbacks,
            CreateWcfPoller,
            CreateServiceChannels,
            CreateBackupTreeView,
            CreateItemTemplate,
        }

        // 1/31/2012
        public class TaskObjectResult
        {
            public bool Result;
            public string Message;
            public InitComponentEnum InitComponent;
        }

        private class TaskObjectState
        {
            public MainWindow MainWindow;
            public System.Windows.Application Application;
            public object ObjectStateName3;
        }

        /// <summary>
        /// Runs the Start-up initialization routines for ICL application.
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="MainWindow"/></param>
        /// <param name="appCurrent"></param>
        /// <param name="paramName3"></param>
        /// <returns></returns>
        public Task<TaskObjectResult> DoStartUpInitializeRoutines(MainWindow mainWindow, System.Windows.Application appCurrent, object paramName3 = null)
        {
            var task = Task.Factory.StartNew(new Func<object, TaskObjectResult>(DoTaskActionCallback),
                                             new TaskObjectState
                                                 {
                                                     MainWindow = mainWindow,
                                                     Application = appCurrent,
                                                     ObjectStateName3 = paramName3
                                                 });

            // Add Completion callback (Note: If ContinueWith added to Task above, then 'Result' will be missing!)
            task.ContinueWith(TaskIsCompletedCallback);

            return task;
        }

        /// <summary>
        /// Task Callback which triggers the <see cref="TaskCompleted"/> event.
        /// </summary>
        /// <param name="obj"></param>
        private void TaskIsCompletedCallback(Task<TaskObjectResult> obj)
        {
            if (TaskCompleted != null)
            {
                TaskCompleted(obj);
            }
        }

        /// <summary>
        /// Action callback which does the start-up initilization routines.
        /// </summary>
        /// <param name="arg"></param>
        /// <returns>true/false of result</returns>
        private TaskObjectResult DoTaskActionCallback(object arg)
        {
            // Retrieve Task's objectState
            var objectState = (TaskObjectState)arg;
            var mainWindow = objectState.MainWindow;

            var objectResult = new TaskObjectResult
            {
                InitComponent = InitComponentEnum.CheckForXpOs,
                Message = "",
                Result = true
            };

            // 2/20/2011 - Check if WindowsXp - Used to remove scheduling.
            mainWindow.WaitMessage.SetMessageToDiplay("Checking OS Type...");
            if (!mainWindow.CheckIfWindowsXp())
            {
                var result = new TaskObjectResult
                                 {
                                     InitComponent = InitComponentEnum.CheckForXpOs,
                                     Message = "Failed on Checking Os Type",
                                     Result = false
                                 };

                return result;
            }

            // 11/14/2010 - Set 'Source' location for log events.
            LogWinEventsHelper.LogSourceName = "ICLWebService";

            // Start DirectoryTreeViewTasks
            mainWindow.WaitMessage.SetMessageToDiplay("Starting Directory Tasks...");
            if (!mainWindow.StartDirectoryTreeViewTasks())
            {
                var result = new TaskObjectResult
                {
                    InitComponent = InitComponentEnum.StartDirectoryTreeViewTask,
                    Message = "Failed on Starting Directory Tasks",
                    Result = false
                };

                return result;
            }

            // 12/3/2010 - Set EventHandler for Cancel operation
            mainWindow.WaitMessage.SetMessageToDiplay("Setting EventHandlers...");
            if (!mainWindow.SetEventHandlerForCancelOrder())
            {
                var result = new TaskObjectResult
                {
                    InitComponent = InitComponentEnum.SetEventHandlersForCallback,
                    Message = "Failed on Setting EventHandlers",
                    Result = false
                };

                return result;
            }

            // Create instance of WCF Callback service.
            mainWindow.WaitMessage.SetMessageToDiplay("Creating Service Callbacks...");
            if (!mainWindow.CreateIclWebDuplexCallbacks())
            {
                var result = new TaskObjectResult
                {
                    InitComponent = InitComponentEnum.CreateServiceCallbacks,
                    Message = "Failed on Creating Service Callbacks",
                    Result = false
                };

                return result;
            }

            // 1/23/2012 - Start Polling method for status of service.
            mainWindow.WaitMessage.SetMessageToDiplay("Creating WCF Polling Task...");
            if (!mainWindow.CreateDoPollWcfServiceTask())
            {
                var result = new TaskObjectResult
                {
                    InitComponent = InitComponentEnum.CreateWcfPoller,
                    Message = "Failed on Creating WCF Polling Task",
                    Result = false
                };

                return result;
            }

            // 8/7/2011 - Refactored out
            // 12/5/2010 - Create ChannelFactory.
            mainWindow.WaitMessage.SetMessageToDiplay("Creating Service Channel...");
            if (!mainWindow.CreateChannelFactoryAndSetDefaultValues())
            {
                var result = new TaskObjectResult
                {
                    InitComponent = InitComponentEnum.CreateServiceChannels,
                    Message = "Failed on Creating Service Channel",
                    Result = false
                };

                return result;  
            }

            // 11/3/2010 - Backup TreeView control
            mainWindow.WaitMessage.SetMessageToDiplay("Creating Backup TreeView...");
            if (!mainWindow.CreateBackupTreeViewItems())
            {
                var result = new TaskObjectResult
                {
                    InitComponent = InitComponentEnum.CreateBackupTreeView,
                    Message = "Failed on Creating Backup TreeView",
                    Result = false
                };

                return result;
            }

            // 12/7/2010 - Update the ItemTemplates style for TreeViewUserControl
            mainWindow.WaitMessage.SetMessageToDiplay("Setting Item Template Type...");
            if (!mainWindow.SetItemTemplatesType())
            {
                var result = new TaskObjectResult
                {
                    InitComponent = InitComponentEnum.CreateItemTemplate,
                    Message = "Failed on Setting Item Template Type",
                    Result = false
                };

                return result;
            }

            // 4/29/2011 - Check if Credentials exist; otherwise, show Reg Window.
            /*mainWindow.WaitMessage.SetMessageToDiplay("Checking Account-Info...");
            if (!mainWindow.CheckAccountInfoExist())
            {
                var result = new TaskObjectResult
                {
                    InitComponent = InitComponentEnum.CheckingAcctInfo,
                    Message = "Account Info Required",
                    Result = false
                };

                return result;
            }*/

            return objectResult;
        }
    }
}