﻿using System;
using System.Diagnostics;
using System.ServiceModel;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using Attributes;
using ImageCloudLock;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.Common.LogWinEvents;
using ImageNexus.BenScharbach.Common.MEFLogger.LogWindowParts;
using ImageNexus.BenScharbach.Common.MEFLogger.WcfCustomListener;
using ImageNexus.BenScharbach.Common.MEFLoggerInterfaces.Enums;
using ImageNexus.BenScharbach.Common.MEFLoggerWpfWindow;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;

namespace ImageNexus.BenScharbach.ImageCloudLock.TaskActions
{
    // 1/23/2012
    /// <summary>
    ///  Does the 'Polling of the Logger Messages' in a Parallel task.
    /// </summary>
    public class DoPollLoggerMessages
    {
        private CancellationTokenSource _cancellationTokenSource;
        private CancellationToken _cancellationToken;
        private Task _task;

        private static readonly Stopwatch TimerToSleep1 = new Stopwatch();
        private static readonly TimeSpan TimerSleepMax = new TimeSpan(0, 0, 0, 0, 6000);

        private readonly LogWindow _logWindow;

        private class TaskObjectState
        {
            public MainWindow MainWindowParent;
            public DoPollLoggerMessages ThisTask;
            public CancellationToken CancellationToken;
        }

        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; private set; }

        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{TChannel}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }
        
        /// <summary>
        /// Starts the LoggerWindow when the '/UseLogger' param is passed at startup.
        /// </summary>
        public static bool UseInternalLogger { get; set; }

        #endregion

         /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="MainWindow"/></param>
        public DoPollLoggerMessages(MainWindow mainWindow)
        {
            if (mainWindow == null)
            {
                throw new ArgumentNullException("mainWindow");
            }
           
            MainWindowParent = mainWindow;

            // Create instance of logger window
            _logWindow = new LogWindow();
             UseInternalLogger = true;
        }

        /// <summary>
        /// Shows the internal Logger window.
        /// </summary>
        public void ShowLogWindow()
        {
            if (_logWindow == null)
            {
                return;
            }

            _logWindow.Show();
        }

        /// <summary>
        /// Closes the internal logger window
        /// </summary>
        public void CloseLogWindow()
        {
            LogWindow.KillWindow = true;
            _logWindow.Close();
        }

        /// <summary>
        /// Cancels the current task and waits until it completes the cancellation process.
        /// </summary>
        public void Cancel()
        {
            // 1/23/2012 - Kill WCF Poll Task
            _cancellationTokenSource.Cancel();

            if (!_task.Wait(25))
            {
                Thread.Sleep(10);
            }
        }

        /// <summary>
        /// Creates the LogWindow and Starts the Polling task.
        /// </summary>
        [DoNotUseLogger]
        [CatchAllExceptionHandling(false, null, null)]
        public void StartAsyncPollWcfLoggingMessages()
        {
            // Create new instance of cancellation token.
            _cancellationTokenSource = new CancellationTokenSource();
            _cancellationToken = _cancellationTokenSource.Token;

            _task = Task.Factory.StartNew(OnPollWcfLoggerMessagesCallback, new TaskObjectState
                                                                               {
                                                                                   MainWindowParent = MainWindowParent,
                                                                                   ThisTask = this,
                                                                                   CancellationToken = _cancellationToken
                                                                               }, _cancellationToken);

#if DEBUG
            _task.ContinueWith((Task task) =>
            {
                MessageBox.Show("Wcf Poll Task exit correctly!", "WCF Poll");
            });
#endif
        }

        /// <summary>
        /// Callback to poll Wcf Logger Messages.
        /// </summary>
        private void OnPollWcfLoggerMessagesCallback(object arg)
        {
            try
            {
                // Retrieve Task's objectState
                var objectState = (TaskObjectState)arg;

                while (true)
                {
                    // Check if Task cancel request.
                    var cancellationToken = objectState.CancellationToken;
                    if (cancellationToken.IsCancellationRequested)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                    }

                    try
                    {
                        PollWcfServiceForListenerMessages(objectState);
                    }
                    catch (Exception ex)
                    {
                        // Thrown during startup, since WCF channel is null!
                        // This Capture keeps the Task from crashing and exiting too soon.
                    }

                    // Start StopWatch timer
                    TimerToSleep1.Reset();
                    TimerToSleep1.Start();

                    while (TimerToSleep1.Elapsed.TotalMilliseconds < TimerSleepMax.TotalMilliseconds)
                    {
                        Thread.Sleep(500);
                    }
                }
            }
            catch (OperationCanceledException)
            {
                LogWinEventsHelper.LogWarningMessage("Task 'DoPollLoggerMessages' method received the cancel request.  Operation Aborted.");
            }
        }

        // 7/10/2011
        [DoNotUseLogger]
        [CatchAllExceptionHandling(false, null, null)]
        public void PollWcfServiceForListenerMessages(object arg)
        {
            // Retrieve Task's objectState
            var objectState = (TaskObjectState)arg;
            var thisTask = objectState.ThisTask;

            // Note: 1st param can be null, since it will be set by postsharp.
            WcfTraceListenerMessage[] wcfMessages;
            if (thisTask.DoWCFGetTraceListenerMessages(null, out wcfMessages)) return;

            var listenerCount = Trace.Listeners.Count;
            Console.WriteLine(listenerCount);

            // Redirect to proper WPF Console.
            foreach (var wcfTraceListenerMessage in wcfMessages)
            {
                var documentId = wcfTraceListenerMessage.DocumentId;

                foreach (var message in wcfTraceListenerMessage.Messages)
                {
                    // Write Message using the OnGuiThread callback
                    thisTask.WriteMessageFromWcf(message, documentId);
                } // End ForEach
            } // End ForEach
        }

        [DoNotUseLogger]
        private void WriteMessageFromWcf(string message, int documentId)
        {
            // TODO: Update MessageTypeEnum to come from WCF callback.
            LogWindowCtl.WriteMessageFromWcf(MessageTypeEnum.Information, message, documentId);
        }

        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private bool DoWCFGetTraceListenerMessages(IclWebCommService currentChannel, out WcfTraceListenerMessage[] wcfMessages)
        {
            return !currentChannel.GetWcfTraceListenerMessages(out wcfMessages);
        }
    }
}