﻿using System;
using System.Diagnostics;
using System.ServiceModel;
using System.ServiceProcess;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Media;
using ImageCloudLock;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.Common.LogWinEvents;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;
using ImageNexus.BenScharbach.ImageCloudLock.WcfCallbacks;

namespace ImageNexus.BenScharbach.ImageCloudLock.TaskActions
{
    // 1/23/2012
    /// <summary>
    ///  Does the 'Polling of the Wcf Service' in a Parallel task.
    /// </summary>
    public class DoPollWcfServiceTask
    {
        private CancellationTokenSource _cancellationTokenSource;
        private CancellationToken _cancellationToken; 
        private Task _task;

        // WCF Callbacks (Event handling)
        private readonly IclWebDuplexCallbacks _iclWebDuplexCallbacks;

        private static readonly ServiceController _controller = new ServiceController { MachineName = ".", ServiceName = "ICLWebService" };

        private static readonly Stopwatch TimerToSleep2 = new Stopwatch();
        private static readonly TimeSpan TimerSleepMax2 = new TimeSpan(0, 0, 0, 0, 5000);
       

        private class TaskObjectState
        {
            public MainWindow MainWindowParent;
            public DoPollWcfServiceTask ThisTask;
            public ServiceController Controller;
            public CancellationToken CancellationToken;
        }

        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; private set; }

        #endregion

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="MainWindow"/></param>
        /// <param name="iclWebDuplexCallbacks">Instance of <see cref="IclWebDuplexCallbacks"/></param>
        public DoPollWcfServiceTask(MainWindow mainWindow, IclWebDuplexCallbacks iclWebDuplexCallbacks)
        {
            if (mainWindow == null)
            {
                throw new ArgumentNullException("mainWindow");
            }

            if (iclWebDuplexCallbacks == null)
            {
                throw new ArgumentNullException("iclWebDuplexCallbacks");
            }

            MainWindowParent = mainWindow;
            _iclWebDuplexCallbacks = iclWebDuplexCallbacks;

            VerifyWCFServiceRunning();
        }

        /// <summary>
        /// Starts the Wcf service.
        /// </summary>
        public void StartWcfService()
        {
            // Refresh status
            _controller.Refresh();

            _controller.Start();
            _controller.WaitForStatus(ServiceControllerStatus.Running);
        }

        /// <summary>
        /// Stops the Wcf service.
        /// </summary>
        public void StopWcfService()
        {
            // Refresh status
            _controller.Refresh();

            _controller.Stop();
            _controller.WaitForStatus(ServiceControllerStatus.Stopped);
        }

        /// <summary>
        /// Cancels the current task and waits until it completes the cancellation process.
        /// </summary>
        public void Cancel()
        {
            // 1/23/2012 - Kill WCF Poll Task
            _cancellationTokenSource.Cancel();

            if (!_task.Wait(25))
            {
                Thread.Sleep(10);
            }
        }

        // 12/6/2010
        /// <summary>
        /// Starts a timer, set to trigger ever 3 seconds, to check the status of the
        /// windows service 'ICLWebService', which contains the WCF required service.
        /// Depending on the status, this method will update the MainWindow status ellipse
        /// to Red, Yellow, or Green.
        /// </summary>
        [DoNotUseLogger]
        [CatchAllExceptionHandling(false, null, null)]
        public void StartAsyncPollWcfService()
        {
            // check if already started
            if (_task != null)
            {
                throw new InvalidOperationException("This task is already running!");
            }

            // Create new instance of cancellation token.
            _cancellationTokenSource = new CancellationTokenSource();
            _cancellationToken = _cancellationTokenSource.Token;

            // Create async timer for polling WCF service.
            _task = Task.Factory.StartNew(OnWcfPollCallback, new TaskObjectState
                                                                                  {
                                                                                      Controller = _controller,
                                                                                      MainWindowParent = MainWindowParent,
                                                                                      ThisTask = this,
                                                                                      CancellationToken = _cancellationToken
                                                                                  }, _cancellationToken);

#if DEBUG
            _task.ContinueWith((Task task) =>
            {
                MessageBox.Show("Wcf poll controller task exit correctly!","WCF Poll");
            });
#endif
        }

        // 12/6/2010
        /// <summary>
        /// Verifies the Endpoint is reachable for the WCF service.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void VerifyWCFServiceRunning()
        {
            try
            {
                // Create WCF Proxy
                var testService = new IclWebCommServiceClient(new InstanceContext(_iclWebDuplexCallbacks));
                testService.Open();
            }
            catch (EndpointNotFoundException ex)
            {
                // Service down - Set Red.
                MainWindowParent.wcfServiceStatusEllipse.Fill = new SolidColorBrush { Color = Colors.Red };

                // Check status of service
                if (_controller.Status.Equals(ServiceControllerStatus.Paused))
                {
                    // Set Yellow
                    MainWindowParent.wcfServiceStatusEllipse.Fill = new SolidColorBrush { Color = Colors.Yellow };

                    _controller.Continue();
                    _controller.WaitForStatus(ServiceControllerStatus.Running);

                    // Set Green
                    MainWindowParent.wcfServiceStatusEllipse.Fill = new SolidColorBrush { Color = Colors.Green };
                }
                else
                {
                    _controller.Start();
                    _controller.WaitForStatus(ServiceControllerStatus.Running);

                    // Set Green
                    MainWindowParent.wcfServiceStatusEllipse.Fill = new SolidColorBrush { Color = Colors.Green };
                }
            }
        }

        /// <summary>
        /// This WCF callback peridiocally polls the WCF service to check if the service is running.
        /// </summary>
        /// <param name="arg"></param>
        [CatchAllExceptionHandling(false, null, null)]
        private void OnWcfPollCallback(object arg)
        {
            try
            {
                // Retrieve Task's objectState
                var objectState = (TaskObjectState)arg;

                while (true)
                {
                    // Check if Task cancel request.
                    var cancellationToken = objectState.CancellationToken;
                    if (cancellationToken.IsCancellationRequested)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                    }

                    OnTimerCallback(objectState);

                    // Start StopWatch timer
                    TimerToSleep2.Reset();
                    TimerToSleep2.Start();

                    while (TimerToSleep2.Elapsed.TotalMilliseconds < TimerSleepMax2.TotalMilliseconds)
                    {
                        Thread.Sleep(10);
                    }
                }
            }
            catch (OperationCanceledException)
            {
                LogWinEventsHelper.LogWarningMessage("Task 'DoPollWcfServiceTask' method received the cancel request.  Operation Aborted.");
            }
        }

        // 7/4/2011 - // NOTE: Avoid Anonymous callbacks with timer.
        [DoNotUseLogger]
        private static void OnTimerCallback(object state)
        {
            try
            {
                // Cast object to MainWindows
                var objectState = (TaskObjectState)state;
                var mainWindow = objectState.MainWindowParent;
                var serviceController = objectState.Controller;

                // Refresh status.
                serviceController.Refresh();

                // Check windows service status.
                // Check status of service
                if (serviceController.Status.Equals(ServiceControllerStatus.Paused))
                {
                    // Set Yellow
                    mainWindow.SetStatusEllipseColor(Colors.Yellow);
                }
                else if (serviceController.Status.Equals(ServiceControllerStatus.Stopped))
                {
                    // Service down - Set Red.
                    mainWindow.SetStatusEllipseColor(Colors.Red);
                }
                else if (serviceController.Status.Equals(ServiceControllerStatus.Running))
                {
                    // Set Green
                    mainWindow.SetStatusEllipseColor(Colors.Green);
                }
            }
            catch (Exception err)
            {
                Console.WriteLine(@"StartAsyncPollWCFService method threw the exception - {0}", err.Message);
                throw;
            }
        }
    }
}