﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.Threading.Tasks;
using Attributes;
using ImageCloudLock;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.ICLTools.StorageTools.BackupData;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;

namespace ImageNexus.BenScharbach.ImageCloudLock.TaskActions
{
    // 1/26/2012
    /// <summary>
    ///  Does the 'Restore operation for the Directories checked' in a Parallel task.
    /// </summary>
    public class DoRestoreDirectoriesTask
    {
        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; set; }

        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{TChannel}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        #region Events

        /// <summary>
        /// Callback when task is completed.
        /// </summary>
        public event Action<Task<List<DirectoryBackupItem>>> TaskCompleted;

        #endregion

        private class TaskObjectState
        {
            public MainWindow MainWindow;
            public DoRestoreDirectoriesTask ThisTask;
            public string DriveName;
        }

        /// <summary>
        /// Runs the Restore directories progress for the checked items.
        /// </summary>
        /// <param name="mainWindow">Instance of <see cref="MainWindow"/></param>
        /// <param name="driveName"></param>
        /// <returns></returns>
        public Task<List<DirectoryBackupItem>> DoRestoreDirectoriesRoutine(MainWindow mainWindow, string driveName)
        {
            // Set
            MainWindowParent = mainWindow;

            Task<List<DirectoryBackupItem>> task =
                Task.Factory.StartNew(new Func<object, List<DirectoryBackupItem>>(DoTaskActionCallback),
                                      new TaskObjectState
                                          {
                                              MainWindow = mainWindow,
                                              ThisTask = this,
                                              DriveName = driveName,
                                          });

            // Add Completion callback (Note: If ContinueWith added to Task above, then 'Result' will be missing!)
            task.ContinueWith(TaskIsCompletedCallback);

            return task;
        }

        /// <summary>
        /// Task Callback which triggers the <see cref="TaskCompleted"/> event.
        /// </summary>
        /// <param name="obj"></param>
        private void TaskIsCompletedCallback(Task obj)
        {
            if (TaskCompleted != null)
            {
                TaskCompleted((Task<List<DirectoryBackupItem>>) obj);
            }
        }

        /// <summary>
        /// Action callback which does the start-up initilization routines.
        /// </summary>
        /// <param name="arg"></param>
        /// <returns></returns>
        private List<DirectoryBackupItem> DoTaskActionCallback(object arg)
        {
            // Retrieve Task's objectState
            var objectState = (TaskObjectState)arg;

            try
            {
                // 1st - Call 'Start' method
                // Note: 1st param can be null, since it will be set by postsharp.
                int totalCount;
                if (!objectState.ThisTask.DoWcfStartGetDirectoriesforBackup(null, objectState.DriveName, out totalCount))
                {
                    return null;
                }

                // Set MaxValue for progressBar.
                //MiniProgressBar.SetProgressBar2_MaxValue(totalCount);

                // 2nd - Retrieve BackupList files from WCF service.
                var directoriesForBackupList = new List<DirectoryBackupItem>(totalCount);
               
                var counter = 0f;
                const int quantityToBatch = 40;
                for (var index = 0; index < totalCount; index += quantityToBatch)
                {
                    // Retrieve record at given index and add to list.
                    // Note: 1st param can be null, since it will be set by postsharp.
                    DirectoryBackupItem[] directoriesforBackupByBatch = objectState.ThisTask.DoWcfGetDirectoriesforBackupByBatch(null, quantityToBatch);

                    // Check if null collection
                    if (directoriesforBackupByBatch == null)
                    {
                        continue;
                    }

                    // Add to collection
                    directoriesForBackupList.AddRange(directoriesforBackupByBatch);

                    // Update Progressbar.
                    //MiniProgressBar.SetProgressBar2_HeaderText(string.Format("Directories {0} of {1}", counter++, totalCount));
                    //MiniProgressBar.SetProgressBar2_Value(counter);
                }

                return directoriesForBackupList;
            }
            finally
            {
                // 3rd - Call 'End' method
                // Note: 1st param can be null, since it will be set by postsharp.
                objectState.ThisTask.DoWcfEndGetDirectoriesForBackup(null);
            }
        }

        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private bool DoWcfStartGetDirectoriesforBackup(IclWebCommService currentChannel, string driveName, out int totalCount)
        {
            // Upload batch to WCF service.
            return currentChannel.StartGetDirectoriesforBackup(out totalCount, driveName);
        }

        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private DirectoryBackupItem[] DoWcfGetDirectoriesforBackupByBatch(IclWebCommService currentChannel, int quantityToBatch)
        {
            // Upload batch to WCF service.
            return currentChannel.GetDirectoriesforBackupByBatch(quantityToBatch);
        }

        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private void DoWcfEndGetDirectoriesForBackup(IclWebCommService currentChannel)
        {
            // Upload batch to WCF service.
            currentChannel.EndGetDirectoriesForBackup();
        }
    }
}