﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.Threading;
using System.Threading.Tasks;
using Attributes;
using ImageCloudLock;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.CloudDataModel.DirectoryDataModel;
using ImageNexus.BenScharbach.ICLCommon.ICLInterface;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;

namespace ImageNexus.BenScharbach.ImageCloudLock.TaskActions
{
    /// <summary>
    ///  Does the 'Retrieve directories' in a Parallel task.
    /// </summary>
    public class DoRetrieveDirectoriesTask
    {
        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; set; }

        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{TChannel}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        #endregion

        private class TaskObjectState
        {
            public IDirectoryItemViewModel DirectoryItemViewModel;
            public string DirectoryName;
            public RestoreCloudWindow RestoreCloudWindow;
        }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="paramName1"></param>
        /// <param name="paramName2"></param>
        /// <param name="paramName3"></param>
        /// <returns></returns>
        public Task<List<IDirectoryItem>> DoGetCloudDirectoriesForDirectory(IDirectoryItemViewModel paramName1, string paramName2 = null, RestoreCloudWindow paramName3 = null)
        {
            var task = Task.Factory.StartNew(new Func<object, List<IDirectoryItem>>(DoTaskActionCallback),
                                             new TaskObjectState
                                                 {
                                                     DirectoryItemViewModel = paramName1,
                                                     DirectoryName = paramName2,
                                                     RestoreCloudWindow = paramName3
                                                 });

            return task;
        }

        /// <summary>
        /// Does the action callback for the given task.
        /// </summary>
        /// <param name="arg"></param>
        /// <returns></returns>
        private List<IDirectoryItem> DoTaskActionCallback(object arg)
        {
            try
            {
                // Retrieve Task's objectState
                var objectState = (TaskObjectState)arg;

                // Note: 1st param can be null, since it will be set by postsharp.
                var cloudRetrieveDirectories = CloudRetrieveDirectories(null, objectState);

                string[] directories = cloudRetrieveDirectories.Directories;
                if (directories == null) return new List<IDirectoryItem>();

                var directoriesCount = directories.Length;
                if (directoriesCount == 0) return new List<IDirectoryItem>();

                string message = string.Format("Retrieving {0} sub-directories for directory {1}", directoriesCount, objectState.DirectoryName);
                var restoreCloudWindow = objectState.RestoreCloudWindow;
                restoreCloudWindow.SetFooterMessage(message);

                // Reset Footer's mini-progressBar; 3/2/2012 - Minus, since 0 base array length.
                restoreCloudWindow._footerProgressBarDirectories.ResetFooterProgressBar(directoriesCount - 1);

                // convert to 'DirectoryCloudItem'.
                var directoryCloudItems = DirectoryCloudItems(objectState.DirectoryName, directories, restoreCloudWindow, objectState.DirectoryItemViewModel, @"/".ToCharArray());

                // return results
                return directoryCloudItems;
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }

            return null;
        }

        /// <summary>
        /// Iterates a string collection of directory names, and returns a collection of <see cref="IDirectoryItem"/>.
        /// </summary>
        /// <param name="fullDirectoryPath"> </param>
        /// <param name="directories">Array of directory names to convert</param>
        /// <param name="restoreCloudWindow">Instance of <see cref="RestoreCloudWindow"/></param>
        /// <param name="directoryItemViewModel">Instance of <see cref="IDirectoryItemViewModel"/></param>
        /// <param name="delimiter">Delimeter to use to split the directories</param>
        /// <returns>Collection of <see cref="IDirectoryItem"/></returns>
        internal static List<IDirectoryItem> DirectoryCloudItems(string fullDirectoryPath, string[] directories, 
                                                                 RestoreCloudWindow restoreCloudWindow, IDirectoryItemViewModel directoryItemViewModel, char[] delimiter)
        {
            var directoriesCount = directories.Length;

            string message = string.Format("Retrieving {0} sub-directories for directory {1}", directoriesCount, fullDirectoryPath);
            restoreCloudWindow.SetFooterMessage(message);

            // Reset Footer's mini-progressBar; 3/2/2012 - Minus, since 0 base array length.
            restoreCloudWindow._footerProgressBarDirectories.ResetFooterProgressBar(directoriesCount - 1); 

            var counter = 0;
            var directoryCloudItems = new List<IDirectoryItem>(directoriesCount);
            foreach (var directoryFullName in directories)
            {
                if (string.IsNullOrEmpty(directoryFullName)) continue;

                // if AmazonS3, then parse the 'DirectoryName' from the fullPath given.
                var dirName = directoryFullName;
                if (true) //cbAmazonS3.IsChecked
                {
                    // Split out directories into collection, using delimiter '/'.
                    var subDirectoriesSplit = directoryFullName.Split(delimiter);

                    // index to 2nd to last
                    var indexToUse = 0;
                    if (subDirectoriesSplit.Length >= 2)
                        indexToUse = subDirectoriesSplit.Length - 2;

                    // Take 2nd from last item in collection.
                    dirName = subDirectoriesSplit[indexToUse];
                }

                directoryCloudItems.Add(new DirectoryCloudItem((IMainWindow) restoreCloudWindow.MainWindowParent,
                                                               directoryItemViewModel, dirName, directoryFullName));

                // Updated Footer's progress.
                restoreCloudWindow._footerProgressBarDirectories.SetFooterProgress(counter++);

                // Sleep for a few ms to allow UI update.
                Thread.Sleep(1);

            } // End ForEach
            return directoryCloudItems;
        }

        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private static CloudRetrieveDirectories CloudRetrieveDirectories(IclWebCommService currentChannel, TaskObjectState objectState)
        {
            // Start WCF service to retrieve cloud directories
            var cloudRetrieveDirectories =
                currentChannel.CloudRetrieveDirectories(objectState.DirectoryName, "/");

            return cloudRetrieveDirectories;
        }
    }
}