﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.ServiceModel;
using System.Threading;
using System.Threading.Tasks;
using Attributes;
using ImageCloudLock;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.CloudDataModel.DirectoryDataModel;
using ImageNexus.BenScharbach.ICLTools.StorageTools.CloudData;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;

namespace ImageNexus.BenScharbach.ImageCloudLock.TaskActions
{
    /// <summary>
    ///  Does the 'Retrieve files' in a Parallel task.
    /// </summary>
    public class DoRetrieveFilesTask
    {
        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; set; }

        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{TChannel}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        } 

        #endregion

        private class TaskObjectState
        {
            public string DirectoryName;
            public RestoreCloudWindow RestoreCloudWindow;
            public object ObjectStateName3;
        }

#if DEBUG
        /// <summary>
        ///  Fake iteration to test Task UI response
        /// </summary>
        private List<IFileItem> DoGetCloudFilesForDirectoryTest(object arg)
        {
            var stopWatch = new Stopwatch();
            stopWatch.Start();

            while (stopWatch.ElapsedMilliseconds < 10000)
            {
                // do nothing
            }

            return new List<IFileItem>();
        }
#endif

        public Task<List<IFileItem>> DoGetCloudFilesforDirectory(string paramName1, RestoreCloudWindow paramName2 = null, object paramName3 = null)
        {
            var task = Task.Factory.StartNew(new Func<object, List<IFileItem>>(DoTaskActionCallback),
                                             new TaskObjectState
                                                 {
                                                     DirectoryName = paramName1,
                                                     RestoreCloudWindow = paramName2,
                                                     ObjectStateName3 = paramName3
                                                 });

            return task;
        }

        private List<IFileItem> DoTaskActionCallback(object arg)
        {
            // Retrieve Task's objectState
            var objectState = (TaskObjectState)arg;

            // Note: 1st param can be null, since it will be set by postsharp.
            var cloudRetrieveFiles = DoWCFCloudRetrieveFiles(null, objectState);

            GetFileCloudItem[] getFileCloudItems = cloudRetrieveFiles.Files;
            if (getFileCloudItems == null) return new List<IFileItem>();

            var filesCount = getFileCloudItems.Length;
            if (filesCount == 0) return new List<IFileItem>();

            string message = String.Format("Retrieving {0} files for directory {1}", filesCount, objectState.DirectoryName);
            objectState.RestoreCloudWindow.SetFooterMessage(message);

            // Reset Footer's mini-progressBar; 3/2/2012 - Minus, since 0 base array length.
            objectState.RestoreCloudWindow._footerProgressBarFiles.ResetFooterProgressBar(filesCount - 1); 

            // convert to 'FileCloudItem'.
            var fileCloudItems = new List<IFileItem>(filesCount);
            var counter = 0.0f;
            foreach (var getFileCloudItem in getFileCloudItems)
            {
                if (getFileCloudItem == null) continue;

                fileCloudItems.Add(new FileCloudItem(getFileCloudItem.Key, getFileCloudItem.Name, getFileCloudItem.FullName,
                                                     getFileCloudItem.DirectoryName, "", getFileCloudItem.FileSize,
                                                     getFileCloudItem.LastModified));

                // Updated Footer's progress.
                objectState.RestoreCloudWindow._footerProgressBarFiles.SetFooterProgress(counter++);

                // Sleep for a few ms to allow UI update.
                Thread.Sleep(10);
            }

            return fileCloudItems;
        }

        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private static CloudRetrieveFiles DoWCFCloudRetrieveFiles(IclWebCommService currentChannel, TaskObjectState objectState)
        {
            // Start WCF service to retrieve cloud files
            var cloudRetrieveFiles =
                currentChannel.CloudRetrieveFiles(objectState.DirectoryName, "/");

            return cloudRetrieveFiles;
        }
    }
}