﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.Threading;
using System.Threading.Tasks;
using System.Windows;
using Attributes;
using ImageCloudLock;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsMisc;
using ImageCloudLock.TaskActions;
using ImageNexus.BenScharbach.Common.AttsInjectorHelper;
using ImageNexus.BenScharbach.Common.ProgressBarInterface;
using ImageNexus.BenScharbach.Common.ProgressBarWindow;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeInterfaces.Interfaces;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.Delegates;
using ImageNexus.BenScharbach.ICLCommon.DirectoryTreeViewModel.DesktopDataModel.CheckedDataModel;
using ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.CheckedDataModel;
using ImageNexus.BenScharbach.ICLTools.StorageTools.BackupData;
using ImageNexus.BenScharbach.ImageCloudLock.IclWebCommServiceRef;
using ImageNexus.BenScharbach.ImageCloudLock.Interfaces;
using ImageNexus.BenScharbach.ImageCloudLock.TaskActions;

namespace ImageNexus.BenScharbach.ImageCloudLock.TreeViewItems
{
    // 11/3/2010
    /// <summary>
    /// The <see cref="BackupTreeViewItems"/> is used to handle the
    /// user selections made within the 'Backup' treeView control on the
    /// main application window.
    /// </summary>
    public class BackupTreeViewItems : IBackupTreeViewItems
    {
        // 11/7/2010 - Save instance of Adapter Class 
        private readonly DirectoryTreeViewModelAdapter _directoryTreeViewModelAdapter;

        // 10/30/2010
        private List<FileBackupItem> _filesBackupCollection = new List<FileBackupItem>();
        // 10/30/2010
        private List<DirectoryBackupItem> _directoriesBackupCollection = new List<DirectoryBackupItem>();
       

        #region Properties

        /// <summary>
        /// Owner of this window.
        /// </summary>
        public MainWindow MainWindowParent { get; private set; }

        // 12/6/2010
        /// <summary>
        /// Gets the current <see cref="DuplexChannelFactory{TChannel}"/>.
        /// </summary>
        public DuplexChannelFactory<IclWebCommService> IclDuplexChannelFactory
        {
            get
            {
                return MainWindowParent != null ? MainWindowParent.IclDuplexChannelFactory : null;
            }
            set { if (MainWindowParent != null) MainWindowParent.IclDuplexChannelFactory = value; }
        }

        // 12/6/2010
        /// <summary>
        /// Gets the current communication channel.
        /// </summary>
        public IclWebCommService CurrentChannel { get; set; }

        // 12/6/2010
        // NOTE: The 'Set' is required to keep PostSharp from crashing - bug.
        /// <summary>
        /// Gets the <see cref="ICommunicationObject"/> interface for the <see cref="IclDuplexChannelFactory"/>
        /// instance.  
        /// </summary>
        [CatchAllExceptionHandling(true, typeof(ICommunicationObject), null)]
        public ICommunicationObject CommunicationObject
        {
            get { return IclDuplexChannelFactory; }
            private set { throw new InvalidOperationException(); }
        }

        /// <summary>
        /// Gets the current instance of <see cref="MiniProgressBar"/>.
        /// </summary>
        public IMiniProgressBar MiniProgressBar { get; set; }

        /// <summary>
        /// Collection of <see cref="FileBackupItem"/>.
        /// </summary>
        public List<FileBackupItem> FilesBackupCollection
        {
            get { return _filesBackupCollection; }
        }

        /// <summary>
        /// Collection of <see cref="DirectoryBackupItem"/>.
        /// </summary>
        public List<DirectoryBackupItem> DirectoriesBackupCollection
        {
            get { return _directoriesBackupCollection; }
        }

        
        #endregion

        /// <summary>
        /// Constructor
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public BackupTreeViewItems(MainWindow mainWindow, DirectoryTreeViewModelAdapter directoryTreeViewModelAdapter)
        {
            // Save MainWindow ref
            MainWindowParent = mainWindow;

            // 11/3/2010 - Save reference to treeView control.
            _directoryTreeViewModelAdapter = directoryTreeViewModelAdapter;

            // 10/25/2010 - Capture Drive 'Selected' event, in order to check for 'CheckedCollection' items.
            _directoryTreeViewModelAdapter.DriveSelected += treeViewUserControl1_DriveSelected;
        }

        // 10/25/2010
        /// <summary>
        /// Eventhandler to capture the 'DriveSelected' event.
        /// </summary>
// ReSharper disable InconsistentNaming
        [CatchAllExceptionHandling(false, null, null)]
        private void treeViewUserControl1_DriveSelected(object sender, SelectedItemEventArgs e)
// ReSharper restore InconsistentNaming
        {
            // 1st - Backup any collections for prior selected drive, if any.
            if (_directoryTreeViewModelAdapter.TreeViewUserControl.PriorDriveSelected != null)
            {
                SaveItemsForBackup(_directoryTreeViewModelAdapter.TreeViewUserControl.PriorDriveSelected, true);
            }

            // 2nd - Retrieve collection for current selected drive, if any.
            RestorePriorBackupCollections();
        }

        // 10/25/2010
        /// <summary>
        /// Restores the prior backup collections for the Files and Directories to the treeView control.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void RestorePriorBackupCollections()
        {
            //
            // Restore the prior 'CheckedCollection', if any.
            //

            // Update collections
            RestorePriorBackupItems();

            // restore files.
            if (_filesBackupCollection != null && _filesBackupCollection.Count > 0)
            {
                // 10/26/2010 - Convert to new CheckedItem type.
                List<IFileCheckedItem<IFileItem>> fileCheckedItems;
                FilesConvertToType(_filesBackupCollection, out fileCheckedItems);

                // 2nd - Set Collection back into TreeView control.
                _directoryTreeViewModelAdapter.SetCheckedItems(fileCheckedItems);
            }

            // restore directories.
            if (_directoriesBackupCollection != null && _directoriesBackupCollection.Count > 0)
            {
                // 10/26/2010 - Convert to new CheckedItem type.
                List<IDirectoryCheckedItem<IDirectoryItem>> directoryCheckedItems;
                DirectoryConvertToType(_directoriesBackupCollection, out directoryCheckedItems);

                // 2nd - Set Collection back into TreeView control.
                _directoryTreeViewModelAdapter.SetCheckedItems(directoryCheckedItems);
            }
        }

        // 10/25/2010; 2/10/2011
        /// <summary>
        /// Saves the current files and directory collections for backup.
        /// </summary>
        /// <param name="driveName">Drive to save backup data for.</param>
        /// <param name="clearCollections">Clear all checkeditems?</param>
        private void SaveItemsForBackup(string driveName, bool clearCollections)
        {
            try // 12/2/2010
            {
                // 11/11/2010 - ONLY update when the 'IsDirty' flag is marked.
                if (!_directoryTreeViewModelAdapter.IsDirty)
                {
#if DEBUG
                    Console.WriteLine(@"No Checked items were updated; the 'IsDirty' flag is false.  Save Operation aborted.");
#endif
                    return;
                }

                // Retrieve Files checked in treeView.
                List<IFileCheckedItem<IFileItem>> fileCheckedItems;
                _directoryTreeViewModelAdapter.GetListOfCheckedItems(out fileCheckedItems);

                // 10/26/2010 - Convert to new BackupItem type.
                FilesConvertToType(fileCheckedItems, out _filesBackupCollection);

                // Retrieve Directories checked in treeView.
                List<IDirectoryCheckedItem<IDirectoryItem>> directoryCheckedItems;
                _directoryTreeViewModelAdapter.GetListOfCheckedItems(out directoryCheckedItems);

                // 10/26/2010 - Convert to new BackupItem type.
                DirectoryConvertToType(directoryCheckedItems, out _directoriesBackupCollection);

                // check if items return is null
                if (fileCheckedItems == null) // || fileCheckedItems.Count == 0)
                {
                    MessageBox.Show("No items are currently checked.");
                    return;
                }

                // 8/20/2011
                SetFilesToBackup(driveName, _filesBackupCollection, _directoriesBackupCollection);
            }
            finally
            {
                // 2/10/2011 - Check if should clear out collections.
                if (clearCollections)
                {
                    // 10/27/2010 - Clear out all prior checkedItems.
                    _directoryTreeViewModelAdapter.ClearAllCheckedItems();
                    // 10/30/2010 - Clear out internal collections too.
                    _filesBackupCollection.Clear();
                    _directoriesBackupCollection.Clear();
                }
            }
        }

        // 10/30/2010
        /// <summary>
        /// Sets a given simple array into the internal collection.
        /// </summary>
        /// <param name="files">Array of <see cref="FileBackupItem"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetFilesBackupCollection(FileBackupItem[] files)
        {
            // If nothing to add, then just return.
            if (files == null) return;

            // Check if collection is null
            if (_filesBackupCollection == null)
                _filesBackupCollection = new List<FileBackupItem>(files.Length);

            _filesBackupCollection.Clear();
            _filesBackupCollection.AddRange(files);
        }

        // 10/30/2010
        /// <summary>
        /// Sets a given simple array into the internal collection.
        /// </summary>
        /// <param name="directories">Array of <see cref="DirectoryBackupItem"/>.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetDirectoriesBackupCollection(DirectoryBackupItem[] directories)
        {
            // If nothing to add, then just return.
            if (directories == null) return;

            // Check if collection is null
            if (_directoriesBackupCollection == null)
                _directoriesBackupCollection = new List<DirectoryBackupItem>(directories.Length);

            _directoriesBackupCollection.Clear();
            _directoriesBackupCollection.AddRange(directories);
        }

        #region Convert Methods

        // 10/26/2010
        /// <summary>
        /// Converts a given <see cref="IFileCheckedItem{T}"/> type to the backup type <see cref="FileBackupItem"/>.
        /// </summary>
        /// <param name="fileCheckedItems">Collection of <see cref="IFileCheckedItem{T}"/> to convert.</param>
        /// <param name="fileBackupItems">(OUT) Collection of new <see cref="FileBackupItem"/> for backup.</param>
        [CatchAllExceptionHandling(false, null, null)]
        private static void FilesConvertToType(List<IFileCheckedItem<IFileItem>> fileCheckedItems, out List<FileBackupItem> fileBackupItems)
        {
            // Create conversion backup list.
            fileBackupItems = new List<FileBackupItem>(fileCheckedItems.Count);

            // 12/2/2010 - If empty, then return.
            if (fileCheckedItems.Count == 0) return;

            // iterate collection and add new FileBackupItem type.
            foreach (var fileCheckedItem in fileCheckedItems)
            {
                // 12/10/2010 - Store ONLY isChecked items.
                if (fileCheckedItem.IsChecked)
                    fileBackupItems.Add(new FileBackupItem
                                            {
                                                DirectoryName = fileCheckedItem.DirectoryName,
                                                FullName = fileCheckedItem.FullName,
                                                Name = fileCheckedItem.Name,
                                                ScheduleNumber = fileCheckedItem.ScheduleNumber // 12/7/2010
                                            });
            }
        }

        // 10/26/2010 - Overload method
        /// <summary>
        /// Converts a given <see cref="FileBackupItem"/> type to the backup type <see cref="IFileCheckedItem{T}"/>.
        /// </summary>
        /// <param name="fileBackupItems">Collection of <see cref="FileBackupItem"/> to convert.</param>
        /// <param name="fileCheckedItems">(OUT) Collection of new <see cref="IFileCheckedItem{T}"/> for backup.</param>
        [CatchAllExceptionHandling(false, null, null)]
        private static void FilesConvertToType(List<FileBackupItem> fileBackupItems, out List<IFileCheckedItem<IFileItem>> fileCheckedItems)
        {
            // Create conversion backup list.
            fileCheckedItems = new List<IFileCheckedItem<IFileItem>>(fileBackupItems.Count);

            // 12/2/2010 - If empty, then return.
            if (fileBackupItems.Count == 0) return;

            // iterate collection and add new FileBackupItem type.
            foreach (var fileCheckedItem in fileBackupItems)
            {
                fileCheckedItems.Add(new FileCheckedItem
                {
                    DirectoryName = fileCheckedItem.DirectoryName,
                    FullName = fileCheckedItem.FullName,
                    Name = fileCheckedItem.Name,
                    ScheduleNumber = fileCheckedItem.ScheduleNumber, // 12/7/2010
                    IsChecked =  true // 12/10/2010 - All items being restored ARE checked!
                });
            }
        }

        // 10/26/2010 
        /// <summary>
        /// Converts a given <see cref="IDirectoryCheckedItem{TU}"/> type to the backup type <see cref="DirectoryBackupItem"/>.
        /// </summary>
        /// <param name="directoryCheckedItems">Collection of <see cref="IDirectoryCheckedItem{TU}"/> to convert.</param>
        /// <param name="directoryBackupItems">(OUT) Collection of new <see cref="DirectoryBackupItem"/> for backup.</param>
        [CatchAllExceptionHandling(false, null, null)]
        private static void DirectoryConvertToType(List<IDirectoryCheckedItem<IDirectoryItem>> directoryCheckedItems, out List<DirectoryBackupItem> directoryBackupItems)
        {
            // Create conversion backup list.
            directoryBackupItems = new List<DirectoryBackupItem>(directoryCheckedItems.Count);

            // 12/2/2010 - If empty, then return.
            if (directoryCheckedItems.Count == 0) return;

            // iterate collection and add new FileBackupItem type.
            foreach (var fileCheckedItem in directoryCheckedItems)
            {
                // 12/10/2010 - Store when isChecked or isIndeterminate items.
                if (fileCheckedItem.IsChecked || fileCheckedItem.IsIndeterminate)
                    directoryBackupItems.Add(new DirectoryBackupItem
                    {
                        FullName = fileCheckedItem.FullName,
                        Name = fileCheckedItem.Name,
                        ScheduleNumber = fileCheckedItem.ScheduleNumber, // 12/8/2010
                        IsIndeterminate = fileCheckedItem.IsIndeterminate // 12/9/2010
                    });
            }
        }

        // 10/26/2010 - Overload method
        /// <summary>
        /// Converts a given <see cref="DirectoryBackupItem"/> type to the backup type <see cref="IDirectoryCheckedItem{TU}"/>.
        /// </summary>
        /// <param name="directoryBackupItems">Collection of <see cref="DirectoryBackupItem"/> to convert.</param>
        /// <param name="directoryCheckedItems">(OUT) Collection of new <see cref="IDirectoryCheckedItem{TU}"/> for backup.</param>
        [CatchAllExceptionHandling(false, null, null)]
        private static void DirectoryConvertToType(List<DirectoryBackupItem> directoryBackupItems, out List<IDirectoryCheckedItem<IDirectoryItem>> directoryCheckedItems)
        {
            // Create conversion backup list.
            directoryCheckedItems = new List<IDirectoryCheckedItem<IDirectoryItem>>(directoryBackupItems.Count);

            // 12/2/2010 - If empty, then return.
            if (directoryBackupItems.Count == 0) return;

            // iterate collection and add new FileBackupItem type.
            foreach (var fileCheckedItem in directoryBackupItems)
            {
                directoryCheckedItems.Add(new DirectoryCheckedItem
                {
                    FullName = fileCheckedItem.FullName,
                    Name = fileCheckedItem.Name,
                    ScheduleNumber = fileCheckedItem.ScheduleNumber, // 12/8/2010
                    IsIndeterminate = fileCheckedItem.IsIndeterminate, // 12/9/2010
                    IsChecked = !fileCheckedItem.IsIndeterminate // 12/10/2010 - Only if not IsDeterminate state.
                });
            }
        }

        #endregion

        // 11/3/2010
        /// <summary>
        /// Called by main application when window closing event occurs, 
        /// which in turn, calls the SaveItemsForBackup.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public void TriggerSaveItemsForBackup()
        {
            if (_directoryTreeViewModelAdapter != null && _directoryTreeViewModelAdapter.TreeViewUserControl != null &&
                _directoryTreeViewModelAdapter.TreeViewUserControl.CurrentDriveSelected != null)
            {
                SaveItemsForBackup(_directoryTreeViewModelAdapter.TreeViewUserControl.CurrentDriveSelected, false);
            }
        }

        #region BackupData

        // 10/30/2010
        /// <summary>
        /// Delegate callback for the TreeView save routines.
        /// </summary>
        /// <param name="driveName">Current driveName these collections belong to.</param>
        /// <param name="filesToBackup">Collection of <see cref="FileBackupItem"/> to store.</param>
        /// <param name="directoriesToBackup">Collection of <see cref="DirectoryBackupItem"/> to store.</param>
        [ShowWaitCursor]
        [CatchAllExceptionHandling(false, null, null)]
        private void SetFilesToBackup(string driveName, List<FileBackupItem> filesToBackup, List<DirectoryBackupItem> directoriesToBackup)
        {
            // Start Backup process
            SetFilesToBackupHelper(driveName, filesToBackup, directoriesToBackup);

        }

        // 12/13/2010 - Optimized: Updated the backup process for both Files and Directories to now batch, which reduces the
        //                         overhead for each WCF call. - Originally, time took several minutes, now was seconds.
        // 12/1/2010; 12/3/2010 - Updated to use parallel Tasks.
        /// <summary>
        /// Helper method, which completes the files and directories backup process by using
        /// the 3 step WCF method call process.
        /// </summary>
        /// <param name="driveName">Current driveName these collections belong to.</param>
        /// <param name="filesToBackup">Collection of <see cref="FileBackupItem"/> to store.</param>
        /// <param name="directoriesToBackup">Collection of <see cref="DirectoryBackupItem"/> to store.</param>
        [CatchAllExceptionHandling(false, null, null)]
        private void SetFilesToBackupHelper(string driveName, List<FileBackupItem> filesToBackup, List<DirectoryBackupItem> directoriesToBackup)
        {
            // Set MaxValue for progressBar.
            /*var filesCount = filesToBackup.Count; // Cache value to optimize.
            MiniProgressBar.SetProgressBar1_MaxValue(filesCount);
            var directoriesCount = directoriesToBackup.Count;
            MiniProgressBar.SetProgressBar2_MaxValue(directoriesCount);*/

            // Retrieve current user's MyDocs folder
            var myDocsPath = Environment.GetFolderPath(Environment.SpecialFolder.Personal);

            // Step 1: Call 'Start' WCF method.
            // Note: 1st param can be null, since it will be set by postsharp.
            DoWcfStartSetFilesToBackup(null, driveName);

            // Send files to WCF
            // Create Parallel tasks for each iteration.
            //var backupFilesTask = Task.Factory.StartNew(DoBackupFilesCallback, objectState);
            var backupFilesTask = new DoBackupFilesTask();
            var filesTask = backupFilesTask.DoBackupfilesRoutine(MainWindowParent, myDocsPath,
                                                                 driveName, filesToBackup);

            // Send directories to WCF
            // Create Parallel tasks for each iteration.
            //var backupDirectoriesTask = Task.Factory.StartNew(DoBackupDirectoriesCallback, objectState);
            var backupDirectoriesTask = new DoBackupDirectoriesTask();
            var directoriesTask = backupDirectoriesTask.DoBackupDirectoriesRoutine(MainWindowParent,
                                                                                   myDocsPath,
                                                                                   driveName,
                                                                                   directoriesToBackup);
            // WaitAll
            Task.WaitAll(filesTask, directoriesTask);

            // Step 3: Call 'End' WCF Method to complete backup process.
            // Note: 1st param can be null, since it will be set by postsharp.
            DoWcfEndSetFilesToBackup(null, driveName, myDocsPath);

        }

        // 1/26/2012
        /// <summary>
        /// Wcf call to start to set files for backup.
        /// </summary>
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private void DoWcfStartSetFilesToBackup(IclWebCommService currentChannel, string driveName)
        {
            currentChannel.StartSetFilesToBackup(driveName);
        }

        // 1/26/2012
        /// <summary>
        /// Wcf call to start to set files for backup.
        /// </summary>
        [ServiceExceptionHandling]
        [OnWCFProxyCallTest(AsynchronousCallType = OnWCFProxyCallTestAttribute.AsynchronousCallEnum.None)]
        private void DoWcfEndSetFilesToBackup(IclWebCommService currentChannel, string driveName, string locationPath)
        {
            currentChannel.EndSetFilesToBackup(driveName, locationPath);
        }

        #endregion

        #region RestoreData

        // 12/3/2010 - Updated to use new Parallel.Tasks calls.
        // 11/3/2010; 12/2/2010 - Updated to use new factored 3-step WCF methods.
        /// <summary>
        /// Occurs prior to the TreeView updating its control with the internal collections; therefore,
        /// this is where we update the collections before they are displayed.
        /// </summary>
        [ShowWaitCursor]
        [CatchAllExceptionHandling(false, null, null)]
        [MiniProgressBar("Retrieving your backup list...", typeof(MiniProgressBar))]
        private void RestorePriorBackupItems()
        {
            var driveName = MainWindowParent.treeViewUserControl1.CurrentDriveSelected;

            // Get files from WCF
            List<FileBackupItem> getFilesCollection = GetFilesForBackupList(driveName);

            // Get directories from WCF
            List<DirectoryBackupItem> getDirectoriesCollection = GetDirectoriesForBackupList(driveName);

            // Set collections into TreeView control
            if (getFilesCollection != null) // 12/3/2010
            {
                SetFilesBackupCollection(getFilesCollection.ToArray());
            }

            if (getDirectoriesCollection != null) // 12/3/2010
            {
                SetDirectoriesBackupCollection(getDirectoriesCollection.ToArray());
            }
        }

        /// <summary>
        /// Retrieves the 'Files' backup list for the given <paramref name="driveName"/>.
        /// </summary>
        /// <param name="driveName">DriveName to obtain the backup list for.</param>
        /// <returns>Collection of <see cref="FileBackupItem"/>.</returns>
        //[CatchAllExceptionHandling(false, null, null)]
        //[MiniProgressBar("Retrieving your backup list...", typeof(MiniProgressBar))]
        public List<FileBackupItem> GetFilesForBackupList(string driveName)
        {
            // Create Restore-Directories Task
            var getFilesTask = new DoRestoreFilesTask();
            var filesTask = getFilesTask.DoRestoreFilesRoutine(MainWindowParent, driveName);

            // Wait until completed.
            while (!filesTask.Wait(25))
            {
                Thread.Sleep(10);
            }

            return filesTask.Result;
        }

        /// <summary>
        /// Retrieves the 'Directories' backup list for the given <paramref name="driveName"/>.
        /// </summary>
        /// <param name="driveName">DriveName to obtain the backup list for.</param>
        /// <returns>Collection of <see cref="DirectoryBackupItem"/>.</returns>
        //[CatchAllExceptionHandling(false, null, null)]
        //[MiniProgressBar("Retrieving your backup list...", typeof(MiniProgressBar))]
        public List<DirectoryBackupItem> GetDirectoriesForBackupList(string driveName)
        {
            // Create Restore-Directories Task
            var getDirectoriesTask = new DoRestoreDirectoriesTask();
            var directoriesTask = getDirectoriesTask.DoRestoreDirectoriesRoutine(MainWindowParent, driveName);
            
            // Wait until completed.
            while (!directoriesTask.Wait(25))
            {
                Thread.Sleep(10);
            }

            return directoriesTask.Result;
        }

        #endregion
    }
}
