﻿using System;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Linq;
using ImageNexus.BenScharbach.ICLTools.ICLExtensions;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;
using ImageNexus.BenScharbach.Common.ReflectionHelper;

namespace ImageNexus.BenScharbach.ICLCommon.WpfTreeViewControl.CheckedDataModel
{
    // 10/24/2010
    /// <summary>
    /// This singleton <see cref="CheckedCollection{T, TU}"/> stores the collection of <see cref="IFileCheckedItem{T}"/> type and <see cref="IDirectoryCheckedItem{TU}"/> which has the
    /// 'IsChecked' marked true, using a unique <see cref="Guid"/> for fast updates and retrieval.
    /// </summary>
    public sealed class CheckedCollection<T, TU>
    {
        // stores the checkedCollection instance by a unique key value.
        private static readonly Dictionary<int, CheckedCollection<T, TU>> Instances = new Dictionary<int,CheckedCollection<T,TU>>();

        // 11/30/2010 - Updated to new .Net-4 ConcurrentDictionary.
        // 10/25/2010 - 2nd Dictionary which stores using the FilePath as key.
        private readonly ConcurrentDictionary<string, IFileCheckedItem<T>> _checkedItemsStoredByPath = new ConcurrentDictionary<string, IFileCheckedItem<T>>();

        // 11/30/2010 - Updated to new .Net-4 ConcurrentDictionary.
        // 10/25/2010 - Dictionary of directories to backup, stored using the DirectoryPath as key.
        private readonly ConcurrentDictionary<string, IDirectoryCheckedItem<TU>> _checkedDirectoriesStoredByPath = new ConcurrentDictionary<string, IDirectoryCheckedItem<TU>>();

        #region Properties
#pragma warning disable 1574
        // 11/11/2010
        private bool _isDirty;

        /// <summary>
        /// Gets if checked collection has been updated by user.
        /// </summary>
        /// <remarks>
        /// This is marked false when the <see cref="SetCheckedItems"/> method is called.
        /// </remarks>
        public bool IsDirty
        {
            get { return _isDirty; }
            private set { _isDirty = value; }
        }
#pragma warning restore 1574

        #endregion

        // Explicit static constructor to tell C# compiler
        // not to mark type as beforefieldinit
        static CheckedCollection()
        {
        }

        /// <summary>
        /// Returns an instance of this singleton <see cref="CheckedCollection{T, TU}"/> class.
        /// </summary>
        /// <param name="key"></param>
        public static CheckedCollection<T, TU> GetInstance(int key)
        {
            // 1st check if key exist
            if (Instances.ContainsKey(key))
            {
                // get CheckedCollection and return
                return Instances[key];
            }

            var checkedCollection = new CheckedCollection<T, TU>();
            Instances.Add(key, checkedCollection);

            return checkedCollection;
        }

        // 10/9/2011 - Updated to add or update, rather than remove.
        /// <summary>
        /// Adds or updated the given <paramref name="fileCheckedItem"/> from the internal collection.
        /// </summary>
        /// <param name="fileCheckedItem">Instance of <see cref="IFileCheckedItem{T}"/> to update</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void AddUpdateItemFromCollection(IFileCheckedItem<T> fileCheckedItem)
        {
            // 11/11/2010 - Updated to set 'IsDirty' to TRUE.
            IsDirty = true;
           
            // 12/2/2010 - Refactored out to new method.
            AddUpdateItem(fileCheckedItem);
        }

        // 10/9/2011 - Updated to add or update, rather than remove.
        // 12/2/2010 - Note: This allows internal calls to bypass the 'IsDirty' flag setting.
        /// <summary>
        /// Adds or updated the given <paramref name="fileCheckedItem"/> from the internal collection.
        /// </summary>
        /// <param name="fileCheckedItem">Instance of <see cref="IFileCheckedItem{T}"/> to update</param>
        [CatchAllExceptionHandling(false, null, null)]
        private void AddUpdateItem(IFileCheckedItem<T> fileCheckedItem)
        {
            // Add to 2nd Dictionary using the extension method
            _checkedItemsStoredByPath.AddOrUpdateItemFromCollection(fileCheckedItem.FullName, fileCheckedItem);
        }

        // 10/9/2011 - Updated to add or update, rather than remove.
        // 10/25/2010 -Signature Overload method.
        /// <summary>
        /// Adds or removes the given <paramref name="directoryCheckedItem"/> from the internal collection.
        /// </summary>
        /// <param name="directoryCheckedItem">Instance of <see cref="IDirectoryCheckedItem{TU}"/> to update</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void AddUpdateItemFromCollection(IDirectoryCheckedItem<TU> directoryCheckedItem)
        {
            // 11/11/2010 - Updated to set 'IsDirty' to TRUE.
            IsDirty = true;

            // 12/2/2010 - Refactored out to new method.
            AddUpdateItem(directoryCheckedItem);
        }

        // 10/9/2011 - Updated to add or update, rather than remove.
        // 12/2/210- Note: This allows internal calls to bypass the 'IsDirty' flag setting.
        /// <summary>
        /// Adds or updates the given <paramref name="directoryCheckedItem"/> from the internal collection.
        /// </summary>
        /// <param name="directoryCheckedItem">Instance of <see cref="IDirectoryCheckedItem{TU}"/> to update</param>
        [CatchAllExceptionHandling(false, null, null)]
        private void AddUpdateItem(IDirectoryCheckedItem<TU> directoryCheckedItem)
        {
            // Add to Dictionary using the extension method
            _checkedDirectoriesStoredByPath.AddOrUpdateItemFromCollection(directoryCheckedItem.FullName, directoryCheckedItem);
        }

        /// <summary>
        /// Checks if given <paramref name="searchKey"/> is in the internal collection, using
        /// the <see cref="searchKey"/> as search string.
        /// </summary>
        /// <param name="searchKey">File's full name as search key</param>
        /// <returns>True/False if in collection</returns>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        public bool IsInFilesCollection(string searchKey)
        {
            // check if empty
            if (_checkedItemsStoredByPath.Count == 0) return false;

            // check using files's string path
            return _checkedItemsStoredByPath.ContainsKey(searchKey); // was fileInfoItem.FullPathName

        }

        // 12/8/2010
        /// <summary>
        /// Updates a given <see cref="IFileCheckedItem{T}"/> instances <paramref name="propertyName"/>
        /// with the given <paramref name="value"/>.
        /// </summary>
        /// <param name="searchKey">File's full name as search key</param>
        /// <param name="propertyName">Property name to update</param>
        /// <param name="value">Value to set</param>
        /// <exception cref="ArgumentException">Thrown when the given <paramref name="propertyName"/> cannot be found.</exception>
        [CatchAllExceptionHandling(false, null, null)]
        public void UpdateFilePropertyValue(string searchKey, string propertyName, object value)
        {
            // 1st - Retrieve IFileCheckedItem to update.
            IFileCheckedItem<T> fileCheckedItem;
            if (!_checkedItemsStoredByPath.TryGetValue(searchKey, out fileCheckedItem)) return;

            // Check if 'PropertyName' given exist.
            if (ReflectionHelperTool.DoesPropertyNameExist(fileCheckedItem, propertyName))
            {
                // Update value.
                ReflectionHelperTool.SetPropertyValue(fileCheckedItem, propertyName, value);
            }
            else
            {
                throw new ArgumentException("PropertyName given does not exist in given target.", "propertyName");
            }
        }

        // 12/8/2010
        /// <summary>
        /// Gets an instance of <see cref="IFileCheckedItem{T}"/> for the
        /// given <paramref name="searchKey"/>.
        /// </summary>
        /// <param name="searchKey">File's full name as search key</param>
        /// <returns>Instance of <see cref="IFileCheckedItem{T}"/></returns>
        [CatchAllExceptionHandling(true, typeof(object), null)]
        public IFileCheckedItem<T> GetFileItem(string searchKey)
        {
            IFileCheckedItem<T> fileCheckedItem;
            return _checkedItemsStoredByPath.TryGetValue(searchKey, out fileCheckedItem) ? fileCheckedItem : null;
        }


        // 10/25/2010 - 
        /// <summary>
        /// Checks if given <paramref name="searchKey"/> is in the internal collection, using
        /// the <see cref="searchKey"/> as search string.
        /// </summary>
        /// <param name="searchKey">Directory's full name as search key</param>
        /// <returns>True/False if in collection</returns>
        [CatchAllExceptionHandling(true, typeof(bool), false)]
        public bool IsInDirectoriesCollection(string searchKey)
        {
            // check if empty
            if (_checkedDirectoriesStoredByPath.Count == 0) return false;
            
            // check using directory's string path
            return _checkedDirectoriesStoredByPath.ContainsKey(searchKey); // was DirectoryInfoI.FullName

        }

        // 12/8/2010
        /// <summary>
        /// Updates a given <see cref="IFileCheckedItem{T}"/> instances <paramref name="propertyName"/>
        /// with the given <paramref name="value"/>.
        /// </summary>
        /// <param name="searchKey">File's full name as search key</param>
        /// <param name="propertyName">Property name to update</param>
        /// <param name="value">Value to set</param>
        /// <exception cref="ArgumentException">Thrown when the given <paramref name="propertyName"/> cannot be found.</exception>
        [CatchAllExceptionHandling(false, null, null)]
        public void UpdateDirectoryPropertyValue(string searchKey, string propertyName, object value)
        {
            // 1st - Retrieve IFileCheckedItem to update.
            IDirectoryCheckedItem<TU> directoryCheckedItem;
            if (!_checkedDirectoriesStoredByPath.TryGetValue(searchKey, out directoryCheckedItem)) return;

            // Check if 'PropertyName' given exist.
            if (ReflectionHelperTool.DoesPropertyNameExist(directoryCheckedItem, propertyName))
            {
                // Update value.
                ReflectionHelperTool.SetPropertyValue(directoryCheckedItem, propertyName, value);
            }
            else
            {
                throw new ArgumentException("PropertyName given does not exist in given target.", "propertyName");
            }
        }

        // 12/8/2010
        /// <summary>
        /// Gets an instance of <see cref="IDirectoryCheckedItem{TU}"/> for the
        /// given <paramref name="searchKey"/>.
        /// </summary>
        /// <param name="searchKey">File's full name as search key</param>
        /// <returns>Instance of <see cref="IFileCheckedItem{TU}"/></returns>
        [CatchAllExceptionHandling(true, typeof(object), null)]
        public IDirectoryCheckedItem<TU> GetDirectoryItem(string searchKey)
        {
            IDirectoryCheckedItem<TU> directoryCheckedItem;
            return _checkedDirectoriesStoredByPath.TryGetValue(searchKey, out directoryCheckedItem) ? directoryCheckedItem : null;
        }

        // 10/26/2010
        /// <summary>
        /// Returns a total count of all children directories contain within the given <paramref name="fullName"/>, set 
        /// as checked.
        /// </summary>
        /// <remarks>
        /// The match is done by taking the given <paramref name="fullName"/>, and
        /// checking if any files contain within the internal collection start with given string.
        /// </remarks>
        /// <param name="fullName">Directory parent name</param>
        /// <returns>Total count of directories contain in directory which are checked.</returns>
        [CatchAllExceptionHandling(true, typeof(int), 0)]
        public int TotalCountForDirectoriesInDirectory(string fullName)
        {
            return _checkedDirectoriesStoredByPath.Where(directoryItem => directoryItem.Key.StartsWith(fullName)).Count(); // was directoryInfoItem.DirectoryInfoI.FullName
        }

        // 10/26/2010
        /// <summary>
        /// Returns a total count of all children files contain within the given <paramref name="fullName"/>, set 
        /// as checked.
        /// </summary>
        /// <remarks>
        /// The match is done by taking the given <paramref name="fullName"/>, and
        /// checking if any files contain within the internal collection start with given string.
        /// </remarks>
        /// <param name="fullName">Directory parent name</param>
        /// <returns>Total count of files contain in directory which are checked.</returns>
        [CatchAllExceptionHandling(true, typeof(int), 0)]
        public int TotalCountForFilesInDirectory(string fullName)
        {
            return _checkedItemsStoredByPath.Where(fileItem => fileItem.Key.StartsWith(fullName)).Count(); // was directoryInfoItem.DirectoryInfoI.FullName
        }

        // 10/24/2010; // 10/9/2011 - Return CheckedItems = true.
        /// <summary>
        /// Gets the current collection of checked <see cref="IFileCheckedItem{T}"/> instances, and
        /// returns as a <see cref="List{FileCheckedItem}"/>.
        /// </summary>
        /// <param name="checkedItems">(OUT) Collection of <see cref="IFileCheckedItem{T}"/> instances</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void GetCheckedItems(out List<IFileCheckedItem<T>> checkedItems) 
        {
            // retrieve values collection from dictionary
            var valueItems = new IFileCheckedItem<T>[_checkedItemsStoredByPath.Values.Count];
            _checkedItemsStoredByPath.Values.CopyTo(valueItems, 0);
            
            // 10/9/2011 - Return ONLY checked items = true.
            // populate only the 'FileInfo' instances
            checkedItems = valueItems.AsParallel().Where(item => item.IsChecked).ToList();
        }

        // 10/9/2011 - Return both CheckedItems and Inderminate items = true.
        // 10/25/2010 - signature overload method
        /// <summary>
        /// Gets the current collection of checked <see cref="IDirectoryCheckedItem{TU}"/> instances, and
        /// returns as a <see cref="List{DirectoryCheckedItem}"/>.
        /// </summary>
        /// <param name="checkedItems">(OUT) Collection of <see cref="IDirectoryCheckedItem{TU}"/> instances</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void GetCheckedItems(out List<IDirectoryCheckedItem<TU>> checkedItems)
        {
            // retrieve values collection from dictionary
            var valueItems = new IDirectoryCheckedItem<TU>[_checkedDirectoriesStoredByPath.Values.Count];
            _checkedDirectoriesStoredByPath.Values.CopyTo(valueItems, 0);

            // 10/9/2011 - Return both CheckedItems and Inderminate items = true.
            // populate only the 'DirectoryInfo' instances
            checkedItems = valueItems.AsParallel().Where(item => item.IsChecked || item.IsIndeterminate).ToList();
        }

        // 10/24/2010; 12/2/2010: Updated to call internal 'AddUpdateItem' method to bypass 'IsDirty' setting.
        /// <summary>
        /// Clears the internal collection and updates with given <paramref name="checkedItems"/>
        /// collection.
        /// </summary>
        /// <param name="checkedItems">Collection of files which are marked as 'IsChecked'.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetCheckedItems(List<IFileCheckedItem<T>> checkedItems)
        {
            // check if null
            if (checkedItems == null)
                throw new ArgumentNullException("checkedItems");

            // Clear both dictionaries
            _checkedItemsStoredByPath.Clear();

            // Populate with new FileCheckedItem instances using given collection
            foreach (var checkedItem in checkedItems)
            {
                // add to both dictionaries
                AddUpdateItem(checkedItem);
            }

            // 11/11/2010 - Reset 'IsDirty' flag.
            // NOTE: This MUST come after the 'AddUpdateItem' call above, otherwise, the 'IsDirty' will be erroneously marked true.
            IsDirty = false;
        }

        // 10/25/2010 - Overload method; 12/2/2010: Updated to call internal 'AddUpdateItem' method to bypass 'IsDirty' setting.
        /// <summary>
        /// Clears the internal collection and updates with given <paramref name="checkedItems"/>
        /// collection.
        /// </summary>
        /// <param name="checkedItems">Collection of directories which are marked as 'IsChecked'.</param>
        [CatchAllExceptionHandling(false, null, null)]
        public void SetCheckedItems(List<IDirectoryCheckedItem<TU>> checkedItems)
        {
            // check if null
            if (checkedItems == null)
                throw new ArgumentNullException("checkedItems");

            // Clear dictionary
            _checkedDirectoriesStoredByPath.Clear();

            // Populate with new DirectoryInfoItem instances using given collection
            foreach (var checkedItem in checkedItems)
            {
                // add to both dictionaries
                AddUpdateItem(checkedItem);
            }

            // 11/11/2010 - Reset 'IsDirty' flag.
            // NOTE: This MUST come after the 'AddUpdateItem' call above, otherwise, the 'IsDirty' will be erroneously marked true.
            IsDirty = false;
        }

        // 10/27/2010
        /// <summary>
        /// Clears out all internal collections of any checked items.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public void ClearAllCheckedItems()
        {
            _checkedItemsStoredByPath.Clear();
            _checkedDirectoriesStoredByPath.Clear();
        }
    }
}
