﻿using System;
using System.ComponentModel;
using System.Threading;
using System.Windows.Controls;
using ImageNexus.BenScharbach.Common.AttributesCodeInjector.AttsException;


namespace ProgressBarUserControl
{
    class UpdateProgressBar
    {
        // BackgroundWorker thread used to monitor upload event handler.
        private BackgroundWorker _worker;
        // ProgressBar instance to update
        private readonly ProgressBar _progressBar;

        private int _percentCompleted;

        #region Properties

        /// <summary>
        /// <see cref="BackgroundWorker"/> thread used to update progressbar.
        /// </summary>
        public BackgroundWorker Worker
        {
            get { return _worker; }
            set { _worker = value; }
        }

        /// <summary>
        /// Gets or sets percent complete into progress bar.
        /// </summary>
        public int PercentCompleted
        {
            get { return _percentCompleted; }
            set
            {
                _percentCompleted = value;

                // 11/10/2010 - Updated to now start the worker thread itself.
                if (Worker == null)
                {
                    StartProgressBarThread();
                    //throw new InvalidOperationException("Internal BackgroundWorker instance is null.  The StartProgressBarThread() method must be called before this property can be used.");
                }

                // 10/29/2010 - Updated to capture this error - Thrown peridically at last percent update, which
                //              occurs due to the event 'completed' killing both threads first.
                try
                {
                    // 8/22/2011 - Pass in _progressBar as 2nd param.
                    // Update the ProgressBar by calling the BackgroundWorker ReportProgress method. 
                    if (Worker != null) Worker.ReportProgress(value, _progressBar);
                }
                catch (InvalidOperationException)
                {
                    // Just return and ignore error.
                    return;
                }

            }
        }

        #endregion

        /// <summary>
        /// Constructor
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public UpdateProgressBar(ProgressBar progressBar)
        {
            _progressBar = progressBar;
        }

        /// <summary>
        /// Starts the <see cref="BackgroundWorker"/> thread, which monitors the
        /// given EventHandler.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public void StartProgressBarThread()
        {
            // Create BackgroundWorker thread.
            Worker = new BackgroundWorker { WorkerReportsProgress = true, WorkerSupportsCancellation = true };

            // Set DoWork delegate callback method.
            Worker.DoWork += _worker_DoWork;

            // Set the ProgressChanged delegate callback
            Worker.ProgressChanged += OnProgressChangedEventHandler;

             // Set the WorkCompleted delegate callback
            Worker.RunWorkerCompleted += OnRunWorkerCompletedEventHandler;

            // Start work - 8/22/2011: Pass in ProgressBar instance.
            Worker.RunWorkerAsync(_progressBar);
        }

        // 8/22/2011
        /// <summary>
        /// Occurs when the main task completes.
        /// </summary>
        private void OnRunWorkerCompletedEventHandler(object sender1, RunWorkerCompletedEventArgs args)
        {
            // 8/22/2011 - Get UserState
            var progressBarArg = (ProgressBar)args.UserState;

            // set bar to 100%
            if (progressBarArg != null)
            {
                progressBarArg.Value = 100;
            }
        }

        // 8/22/2011
        /// <summary>
        /// Occurs when the progress is updated.
        /// </summary>
        private static void OnProgressChangedEventHandler(object s, ProgressChangedEventArgs args)
        {
            var worker = s as BackgroundWorker;
            var progressBarArg = (ProgressBar) args.UserState;

            // update progressbar
            if (progressBarArg != null)
            {
                progressBarArg.Value = args.ProgressPercentage;
            }
        }

        /// <summary>
        /// Stops the <see cref="BackgroundWorker"/> thread from running.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        public void StopProgressBarThread()
        {
            if (Worker == null)
            {
                return;
            }

            Worker.CancelAsync();

            // 11/10/2010 - Dispose of current instance
            Worker.Dispose();
            Worker = null;
        }

        /// <summary>
        /// Delegate callback for the DoWork() method.
        /// </summary>
        [CatchAllExceptionHandling(false, null, null)]
        private void _worker_DoWork(object sender, DoWorkEventArgs e)
        {
            var worker = sender as BackgroundWorker;

            // Stay in loop until forced to cancel.
            if (worker != null)
                while (!worker.CancellationPending)
                {
                    // just loop
                    Thread.Sleep(10);
                }
        }
    }
}
